@@ -20,7 +20,6 @@
 #include "ADM_assert.h"
 #include "ADM_render/GUI_render.h"
 
-
 #include "DIA_factory.h"
 #ifdef USE_VDPAU
 #include "ADM_coreVdpau/ADM_coreVdpau.h"
@@ -32,7 +31,9 @@
 extern void 		AVDM_audioPref( void );
 extern const char* getNativeRendererDesc(int type);
 
-
+#if defined(USE_DXVA2) || defined(USE_VDPAU) || defined(USE_LIBVA) || defined(USE_VIDEOTOOLBOX)
+    #define HW_ACCELERATED_DECODING
+#endif
 
 uint8_t DIA_Preferences(void);
 
@@ -44,8 +45,6 @@ uint8_t DIA_Preferences(void)
 {
 uint32_t olddevice,newdevice;
 
-bool     use_odml=0;
-uint32_t autosplit=0;
 uint32_t render;
 
 bool     useSwap=0;
@@ -63,18 +62,26 @@ bool     doAutoUpdate=false;
 bool     loadDefault=false;
 char     *alsaDevice=NULL;
 
-bool     balternate_mp3_tag=true;
 bool     lastReadDirAsTarget=false;
+bool     copyModeSanitizeDts=false;
 bool     altKeyboardShortcuts=false;
 bool     swapUpDown=false;
 
 uint32_t pp_type=3;
 uint32_t pp_value=5;
 
+uint32_t editor_cache_size=16;
+
+#ifdef USE_DXVA2
 bool     bdxva2=false;
+bool     bdxva2_override_version=false;
+bool     bdxva2_override_profile=false;
+#endif
 bool     bvdpau=false;
-bool     bxvba=false;
 bool     blibva=false;
+#ifdef USE_VIDEOTOOLBOX
+bool     bvideotoolbox=false;
+#endif
 bool     hzd,vzd,dring;
 bool     capsMMX,capsMMXEXT,caps3DNOW,caps3DNOWEXT,capsSSE,capsSSE2,capsSSE3,capsSSSE3,capsAll;
 bool     hasOpenGl=false;
@@ -100,8 +107,8 @@ std::string currentSdlDriver=getSdlDriverName();
 
 
         // Default pp
-         if(!prefs->get(DEFAULT_POSTPROC_TYPE,&pp_type)) pp_type=3;
-         if(!prefs->get(DEFAULT_POSTPROC_VALUE,&pp_value)) pp_value=3;
+         if(!prefs->get(DEFAULT_POSTPROC_TYPE,&pp_type)) pp_type=0;
+         if(!prefs->get(DEFAULT_POSTPROC_VALUE,&pp_value)) pp_value=0;
 #define DOME(x,y) y=!!(pp_type & x)
 
     DOME(1,hzd);
@@ -141,18 +148,26 @@ std::string currentSdlDriver=getSdlDriverName();
         if( prefs->get(DEVICE_AUDIO_ALSA_DEVICE, &alsaDevice) != RC_OK )
                 alsaDevice = ADM_strdup("plughw:0,0");
 #endif
+        // Video cache
+        prefs->get(FEATURES_CACHE_SIZE,&editor_cache_size);
+#ifdef USE_DXVA2
         // dxva2
-        
-        prefs->get(FEATURES_DXVA2,&bdxva2);                
+        prefs->get(FEATURES_DXVA2,&bdxva2);
+        prefs->get(FEATURES_DXVA2_OVERRIDE_BLACKLIST_VERSION,&bdxva2_override_version);
+        prefs->get(FEATURES_DXVA2_OVERRIDE_BLACKLIST_PROFILE,&bdxva2_override_profile);
+#endif
+#ifdef USE_VDPAU
         // vdpau
         prefs->get(FEATURES_VDPAU,&bvdpau);
-        // xvba
-        prefs->get(FEATURES_XVBA,&bxvba);
+#endif
+#ifdef USE_LIBVA
         // libva
         prefs->get(FEATURES_LIBVA,&blibva);
-        // Alternate mp3 tag (haali)
-        prefs->get(FEATURES_ALTERNATE_MP3_TAG,&balternate_mp3_tag);
-
+#endif
+#ifdef USE_VIDEOTOOLBOX
+        // VideoToolbox
+        prefs->get(FEATURES_VIDEOTOOLBOX,&bvideotoolbox);
+#endif
         // Video renderer
         if(prefs->get(VIDEODEVICE,&render)!=RC_OK)
         {
@@ -168,6 +183,11 @@ std::string currentSdlDriver=getSdlDriverName();
         // Make users happy who prefer the output dir to be the same as the input dir
         prefs->get(FEATURES_USE_LAST_READ_DIR_AS_TARGET,&lastReadDirAsTarget);
 
+        // Some muxers may bail with irregular DTS and some devices may fail to play such videos.
+        // Don't let DTS get too close to each other by delaying frames, drop frames if necessary.
+        // This results in more videos that can't be saved in copy mode.
+        prefs->get(FEATURES_COPY_MODE_SANITIZE_DTS,&copyModeSanitizeDts);
+
         // PgUp and PgDown are cumbersome to reach on some laptops, offer alternative kbd shortcuts
         prefs->get(KEYBOARD_SHORTCUTS_USE_ALTERNATE_KBD_SHORTCUTS,&altKeyboardShortcuts);
 
@@ -188,13 +208,6 @@ std::string currentSdlDriver=getSdlDriverName();
         if(!prefs->get(PRIORITY_PLAYBACK, &playbackPriority))
         playbackPriority=0;
 
-        // VCD/SVCD split point
-        if(!prefs->get(MPEGSPLIT_AUTOSPLIT, &autosplit))
-                autosplit=690;
-
-        // Open DML (Gmv)
-        if(!prefs->get(FEATURES_USE_ODML, &use_odml))
-          use_odml=0;
 #if defined(ALSA_SUPPORT) || defined (OSS_SUPPORT)
 		// Master or PCM for audio
         if(!prefs->get(FEATURES_AUDIOBAR_USES_MASTER, &useMaster))
@@ -214,18 +227,23 @@ std::string currentSdlDriver=getSdlDriverName();
         olddevice=newdevice=AVDM_getCurrentDevice();
         // Audio device
         /************************ Build diaelems ****************************************/
+#ifdef HW_ACCELERATED_DECODING
+    #if defined(USE_DXVA2)
         diaElemToggle useDxva2(&bdxva2,QT_TRANSLATE_NOOP("adm","Decode video using DXVA2 (windows)"));
+        diaElemToggle dxva2OverrideVersion(&bdxva2_override_version,QT_TRANSLATE_NOOP("adm","Ignore driver blacklist (Intel)"));
+        diaElemToggle dxva2OverrideProfile(&bdxva2_override_profile,QT_TRANSLATE_NOOP("adm","Ignore codec blacklist (Intel, HEVC 10bit)"));
+    #elif defined(USE_VIDEOTOOLBOX)
+        diaElemToggle useVideoToolbox(&bvideotoolbox,QT_TRANSLATE_NOOP("adm","Decode video using VideoToolbox (macOS)"));
+    #else
         diaElemToggle useVdpau(&bvdpau,QT_TRANSLATE_NOOP("adm","Decode video using VDPAU (NVIDIA)"));
-        diaElemToggle useXvba(&bxvba,QT_TRANSLATE_NOOP("adm","Decode video using XVBA (AMD)"));
         diaElemToggle useLibVA(&blibva,QT_TRANSLATE_NOOP("adm","Decode video using LIBVA (INTEL)"));
+    #endif
+    #ifndef USE_VIDEOTOOLBOX
+        diaElemReadOnlyText hwAccelText(NULL,QT_TRANSLATE_NOOP("adm","If you use Hw decoding, it is better to use the matching display driver"),NULL);
+    #endif
+#endif
         diaElemToggle useOpenGl(&hasOpenGl,QT_TRANSLATE_NOOP("adm","Enable openGl support"));
-
-        bool foo=0;
-        diaElemToggle hwAccelText(&foo,QT_TRANSLATE_NOOP("adm","If you use Hw decoding, it is better to use the matching display driver"));
-
-
         diaElemToggle allowAnyMpeg(&mpeg_no_limit,QT_TRANSLATE_NOOP("adm","_Accept non-standard audio frequency for DVD"));
-        diaElemToggle openDml(&use_odml,QT_TRANSLATE_NOOP("adm","Create _OpenDML files"));
         diaElemToggle resetEncoder(&loadDefault,QT_TRANSLATE_NOOP("adm","_Revert to saved default output settings on video load"));
         diaElemToggle enableAltShortcuts(&altKeyboardShortcuts,QT_TRANSLATE_NOOP("adm","_Enable alternative keyboard shortcuts"));
         diaElemToggle swapUpDownKeys(&swapUpDown,QT_TRANSLATE_NOOP("adm","Re_verse UP and DOWN arrow keys for navigation"));
@@ -284,11 +302,12 @@ std::string currentSdlDriver=getSdlDriverName();
         framePriority.swallow(&menuIndexPriority);
         framePriority.swallow(&menuPlaybackPriority);
 
-        diaElemUInteger autoSplit(&autosplit,QT_TRANSLATE_NOOP("adm","_Split MPEG files every (MB):"),10,4096);
-
-        diaElemToggle   togTagMp3(&balternate_mp3_tag,QT_TRANSLATE_NOOP("adm","_Use alternative tag for MP3 in .mp4"));
-
         diaElemToggle useLastReadAsTarget(&lastReadDirAsTarget,QT_TRANSLATE_NOOP("adm","_Default to the directory of the last read file for saving"));
+        diaElemToggle sanitizeDtsInCopyMode(&copyModeSanitizeDts,QT_TRANSLATE_NOOP("adm","_Sanitize decode time stamps (DTS) in copy mode"));
+
+        diaElemFrame frameCache(QT_TRANSLATE_NOOP("adm","Caching of decoded pictures"));
+        diaElemUInteger cacheSize(&editor_cache_size,QT_TRANSLATE_NOOP("adm","_Cache size:"),8,16);
+        frameCache.swallow(&cacheSize);
 
         diaMenuEntry videoMode[]={
                              {RENDER_GTK, getNativeRendererDesc(0), NULL}
@@ -301,10 +320,6 @@ std::string currentSdlDriver=getSdlDriverName();
 #ifdef USE_DXVA2
                              ,{RENDER_DXVA2,   QT_TRANSLATE_NOOP("adm","DXVA2 (best)"),NULL}
 #endif
-#ifdef USE_XVBA
-                             //,{RENDER_XVBA,   QT_TRANSLATE_NOOP("adm","XVBA (best)"),NULL}
-#endif
-
 #ifdef USE_OPENGL
                              ,{RENDER_QTOPENGL,   QT_TRANSLATE_NOOP("adm","OpenGL (best)"),NULL}
 #endif
@@ -439,8 +454,8 @@ std::string currentSdlDriver=getSdlDriverName();
 
 
         /* Output */
-        diaElem *diaOutput[]={&autoSplit,&openDml,&allowAnyMpeg,&togTagMp3,&useLastReadAsTarget};
-        diaElemTabs tabOutput(QT_TRANSLATE_NOOP("adm","Output"),5,(diaElem **)diaOutput);
+        diaElem *diaOutput[]={&allowAnyMpeg,&sanitizeDtsInCopyMode,&useLastReadAsTarget,&frameCache};
+        diaElemTabs tabOutput(QT_TRANSLATE_NOOP("adm","Output"),4,(diaElem **)diaOutput);
 
         /* Audio */
 
@@ -479,8 +494,16 @@ std::string currentSdlDriver=getSdlDriverName();
 #endif
         diaElemTabs tabVideo(QT_TRANSLATE_NOOP("adm","Display"),sizeof(diaVideo)/sizeof(diaElem *),(diaElem **)diaVideo);
         /* HW accel */
-        diaElem *diaHwDecoding[]={&useVdpau,&useXvba,&useLibVA,&useDxva2,&hwAccelText};
-        diaElemTabs tabHwDecoding(QT_TRANSLATE_NOOP("adm","HW Accel"),5,(diaElem **)diaHwDecoding);
+#ifdef USE_DXVA2
+        diaElem *diaHwDecoding[]={&useDxva2,&dxva2OverrideVersion,&dxva2OverrideProfile,&hwAccelText};
+        diaElemTabs tabHwDecoding(QT_TRANSLATE_NOOP("adm","HW Accel"),4,(diaElem **)diaHwDecoding);
+#elif defined(USE_VIDEOTOOLBOX)
+        diaElem *diaHwDecoding[]={&useVideoToolbox};
+        diaElemTabs tabHwDecoding(QT_TRANSLATE_NOOP("adm","HW Accel"),1,(diaElem **)diaHwDecoding);
+#elif defined(HW_ACCELERATED_DECODING)
+        diaElem *diaHwDecoding[]={&useVdpau,&useLibVA,&hwAccelText};
+        diaElemTabs tabHwDecoding(QT_TRANSLATE_NOOP("adm","HW Accel"),3,(diaElem **)diaHwDecoding);
+#endif
 
         /* CPU tab */
         diaElem *diaCpu[]={&frameSimd};
@@ -497,32 +520,27 @@ std::string currentSdlDriver=getSdlDriverName();
         diaElemTabs tabAvisynth("Avisynth",2,(diaElem **)diaAvisynth);
 
         /* Global Glyph tab */
-
-
-                diaElemTabs *tabs[]={&tabUser,&tabOutput,&tabAudio,&tabVideo,
-                                &tabHwDecoding,  &tabCpu,&tabThreading, &tabAvisynth};
-                
-       void *factoryCookiez=diaFactoryRunTabsPrepare(QT_TRANSLATE_NOOP("adm","Preferences"),8,tabs);
-
+#ifdef HW_ACCELERATED_DECODING
+        diaElemTabs *tabs[]={&tabUser, &tabOutput, &tabAudio, &tabVideo, &tabHwDecoding, &tabCpu, &tabThreading, &tabAvisynth};
+        void *factoryCookiez=diaFactoryRunTabsPrepare(QT_TRANSLATE_NOOP("adm","Preferences"),8,tabs);
+#else
+        diaElemTabs *tabs[]={&tabUser, &tabOutput, &tabAudio, &tabVideo, &tabCpu, &tabThreading, &tabAvisynth};
+        void *factoryCookiez=diaFactoryRunTabsPrepare(QT_TRANSLATE_NOOP("adm","Preferences"),7,tabs);
+#endif
 // Now we can disable stuff if needed
-  
-#ifndef USE_DXVA2
-       useDxva2.enable(false);       
+#if defined(HW_ACCELERATED_DECODING) && !defined(USE_VIDEOTOOLBOX) && !defined(USE_DXVA2)
+    #ifndef USE_VDPAU
+        useVdpau.enable(false);
+    #endif
+    #ifndef USE_LIBVA
+        useLibVA.enable(false);
+    #endif
 #endif
 
-#ifndef USE_VDPAU       
-        useVdpau.enable(false);              
-#endif
-#ifndef USE_XVBA
-        useXvba.enable(false);              
-#endif
-#ifndef USE_LIBVA
-        useLibVA.enable(false);              
-#endif
-      
-#ifndef  USE_OPENGL
+#ifndef USE_OPENGL
         useOpenGl.enable(false);
 #endif
+
         if( diaFactoryRunTabsFinish(factoryCookiez))
 	{
         	//
@@ -594,11 +612,8 @@ std::string currentSdlDriver=getSdlDriverName();
             prefs->set(UPDATE_ENABLED,doAutoUpdate);
             // Video render
             prefs->set(VIDEODEVICE,render);
-            // Odml
-            prefs->set(FEATURES_USE_ODML, use_odml);
-                            // Split
-            prefs->set(MPEGSPLIT_AUTOSPLIT, autosplit);
-
+            // Video cache
+            prefs->set(FEATURES_CACHE_SIZE, editor_cache_size);
             // number of threads
             prefs->set(FEATURES_THREADING_LAVC, lavcThreads);
             // Encoding priority
@@ -614,19 +629,28 @@ std::string currentSdlDriver=getSdlDriverName();
             prefs->set(MESSAGE_LEVEL,msglevel);
             // Discard changes to output config on video load
             prefs->set(RESET_ENCODER_ON_VIDEO_LOAD, loadDefault);
-
+#ifdef USE_VDPAU
             // VDPAU
             prefs->set(FEATURES_VDPAU,bvdpau);
+#endif
+#ifdef USE_DXVA2
             // DXVA2
             prefs->set(FEATURES_DXVA2,bdxva2);
-            // XVBA
-            prefs->set(FEATURES_XVBA,bxvba);
+            prefs->set(FEATURES_DXVA2_OVERRIDE_BLACKLIST_VERSION,bdxva2_override_version);
+            prefs->set(FEATURES_DXVA2_OVERRIDE_BLACKLIST_PROFILE,bdxva2_override_profile);
+#endif
+#ifdef USE_LIBVA
             // LIBVA
             prefs->set(FEATURES_LIBVA,blibva);
-            // Alternate mp3 tag (haali)
-            prefs->set(FEATURES_ALTERNATE_MP3_TAG,balternate_mp3_tag);
+#endif
+#ifdef USE_VIDEOTOOLBOX
+            // VideoToolbox
+            prefs->set(FEATURES_VIDEOTOOLBOX,bvideotoolbox);
+#endif
             // Make users happy who prefer the output dir to be the same as the input dir
             prefs->set(FEATURES_USE_LAST_READ_DIR_AS_TARGET,lastReadDirAsTarget);
+            // Don't let DTS get too close to each other by delaying frames, drop frames if necessary.
+            prefs->set(FEATURES_COPY_MODE_SANITIZE_DTS,copyModeSanitizeDts);
             // Enable alternate keyboard shortcuts
             prefs->set(KEYBOARD_SHORTCUTS_USE_ALTERNATE_KBD_SHORTCUTS,altKeyboardShortcuts);
             // Allow to use the UP key to navigate back, DOWN to navigate forward
@@ -672,19 +696,4 @@ std::string currentSdlDriver=getSdlDriverName();
 
 	return 1;
 }
-extern int DIA_getMPParams( uint32_t *pplevel, uint32_t *ppstrength,bool *swap);
-void setpp(void)
-{
-        uint32_t type,strength;
-        bool uv=0;
-
-        if(!prefs->get(DEFAULT_POSTPROC_TYPE,&type)) type=3;
-        if(!prefs->get(DEFAULT_POSTPROC_VALUE,&strength)) strength=3;
-        if( DIA_getMPParams( &type,&strength,&uv))
-        {
-                prefs->set(DEFAULT_POSTPROC_TYPE,type);
-                prefs->set(DEFAULT_POSTPROC_VALUE,strength);
-        }
-//	video_body->setPostProc(type, strength, uv);
-}
 //EOF
