@@ -20,7 +20,6 @@
 class ADM_audioStream;
 class ADM_Audiocodec;
 class decoders;
-class COL_Generic2YV12;
 class EditorCache;
 #include "ADM_Video.h"
 /**
@@ -35,6 +34,7 @@ public:
     ADM_Audiocodec   *codec;
     WAVHeader        wavheader;
     bool             vbr;
+    bool             isbr;
     uint64_t         duration;
     uint64_t         size;
 
@@ -45,6 +45,7 @@ public:
                 codec=NULL;
                 duration=size=0;
                 vbr=false;
+                isbr=false;
                 memset(&wavheader,0,sizeof(wavheader));
         }
     ~ADM_audioStreamTrack();
@@ -66,6 +67,8 @@ public:
       
 
       uint32_t _nb_video_frames; /// Really needed ?
+      uint32_t paramCacheSize;
+      uint8_t *paramCache; /// Reuse decoded SPS and friends
       EditorCache *_videoCache; /// Decoded video cache
 
       /* Timeing info */
@@ -76,6 +79,7 @@ public:
       uint64_t timeIncrementInUs; /// in case the video has no PTS, time increment (us)
 
       uint64_t firstFramePts; /// Pts of firstFrame
+      uint32_t decoderDelay; /// Nb of frames passed to decoder before the first picture pops out
 
     _VIDEOS()
     {
@@ -84,14 +88,16 @@ public:
         decoder=NULL;
         color=NULL;
         _nb_video_frames=0;
+        paramCacheSize=0;
+        paramCache=NULL;
         _videoCache=NULL;
         lastSentFrame=0;
         lastDecodedPts=0;
         lastReadPts=0;
         timeIncrementInUs=0;
         firstFramePts=0;
+        decoderDelay=0;
     }
-
 };
 
 /**
