@@ -18,6 +18,7 @@
 #include <math.h>
 #include "ADM_cpp.h"
 #include "ADM_default.h"
+#include "ADM_vidMisc.h"
 #include "ADM_edit.hxx"
 
 #if 0
@@ -208,7 +209,7 @@ uint32_t seg;
     \fn NextPicture
     \brief decode & returns the next picture
 */
-bool        ADM_Composer::nextPicture(ADMImage *image,bool dontcross)
+bool ADM_Composer::nextPicture(ADMImage *image, int flags)
 {
 uint64_t pts;
 uint64_t tail;
@@ -228,13 +229,38 @@ uint64_t tail;
             {
                 // Got it
                 image->duplicate(cached);
+                vid->lastReadPts=cached->Pts;
             }else
                 goto np_nextSeg;
         }else 
         { // Not in cache, decode next one
-            if(false== nextPictureInternal(seg->_reference,image))
+          // but don't go past the end of the last segment
+            uint64_t maxPts=0;
+            if(_currentSegment+1 == _segments.getNbSegments())
+                maxPts=seg->_refStartTimeUs+seg->_durationUs;
+            if(false== nextPictureInternal(seg->_reference,image,maxPts))
             {
-                goto np_nextSeg;
+                if(flags & 2)
+                {
+                    goto np_nextSeg;
+                }
+                uint64_t nkfPts=maxPts;
+                uint32_t frame=0;
+                if(searchNextKeyFrameInRef(seg->_reference,refPts,&nkfPts) && nkfPts<maxPts)
+                {
+                    frame=_segments.intraTimeToFrame(seg->_reference,nkfPts);
+                    ADM_warning("Trying to skip to the next keyframe %" PRIu32" at %s in ref\n",frame,ADM_us2plain(nkfPts));
+                }
+                if(frame && DecodePictureUpToIntra(seg->_reference,frame))
+                {
+                    ADM_info("Successfully skipped to the next keyframe\n");
+                    cached=vid->_videoCache->getByPts(nkfPts);
+                    if(cached)
+                        image->duplicate(cached);
+                }else
+                {
+                    goto np_nextSeg;
+                }
             }
         }
         // Got it, update timing
@@ -256,7 +282,7 @@ uint64_t tail;
 
 // Try to get an image for the following segment....
 np_nextSeg:
-        if(true==dontcross)
+        if(flags & 1)
         {
             ADM_warning("Not allowed to cross segment\n");
             return false;
@@ -284,7 +310,7 @@ np_nextSeg:
                 dont want to decode the targetPts if it is a keyframe
                 but have the frame before
 */
-bool ADM_Composer::decodeTillPictureAtPts(uint64_t targetPts,ADMImage *image)
+uint8_t ADM_Composer::decodeTillPictureAtPts(uint64_t targetPts,ADMImage *image)
 {
  // Go to the previous keyframe and decode forward...
                 uint32_t thisSeg=_currentSegment;
@@ -306,22 +332,22 @@ bool ADM_Composer::decodeTillPictureAtPts(uint64_t targetPts,ADMImage *image)
                 uint64_t previousKf;
                 if(false==searchPreviousKeyFrameInRef(ref,refTime,&previousKf))
                 {
-                    ADM_warning("Cannot find previous keyframe in ref %d, time=%" PRIu64" \n",ref,refTime);
+                    ADM_warning("Cannot find previous keyframe in ref %d for ref time %s\n",ref,ADM_us2plain(refTime));
                     return false;
                 }
                 // go to it...
                 if(false==seektoTime(ref,previousKf,false))
                 {
-                    ADM_warning("Cannot seek to time=%" PRIu64" \n",previousKf);
+                    ADM_warning("Cannot seek to ref time %s\n",ADM_us2plain(previousKf));
                     return false;            
                 }
                 // Now forward till we reach out frame
                 while(1)
                 {
-                    if(false==nextPicture(image,true))
+                    if(false==nextPicture(image,3))
                     {
                             ADM_warning("Error in decoding forward\n");
-                            return false;
+                            return ADM_IGN;
                     }
                     if(image->Pts>=targetPts)
                             break;
@@ -330,10 +356,10 @@ bool ADM_Composer::decodeTillPictureAtPts(uint64_t targetPts,ADMImage *image)
                 }
                 if(image->Pts!=targetPts)
                 {
-                    ADM_error("Could not retrieve our own frame at PTS=%" PRIu64" ms\n",targetPts/1000);
-                    return false;
+                    ADM_warning("Could not retrieve our own frame at %s\n",ADM_us2plain(targetPts));
+                    return ADM_IGN;
                 }
-                return true;
+                return 1;
 }
 /**
     \fn previousPicture
@@ -366,7 +392,7 @@ bool        ADM_Composer::previousPicture(ADMImage *image)
         }
         ADMImage *cached=vid->_videoCache->getBefore(refPts);
        
-        if(cached)
+        if(cached && cached->Pts<refPts)
         {
             if(cached->Pts>=seg->_refStartTimeUs) // It might be in the cache but belonging to the previous seg
             {
@@ -397,12 +423,18 @@ bool        ADM_Composer::previousPicture(ADMImage *image)
         seg=_segments.getSegment(segNo);
         if(segNo==_currentSegment) // Still in the same segment..
         {
-                if(false==decodeTillPictureAtPts(targetPts,image))
+                switch(decodeTillPictureAtPts(targetPts,image))
                 {
-                    ADM_error("Cannot decode till our current pts\n");
-                    return false;
+                    case 1:
+                        cached=vid->_videoCache->getBefore(refPts);
+                        break;
+                    case ADM_IGN:
+                        ADM_warning("Cannot decode till our current pts\n");
+                        cached=vid->_videoCache->getLast();
+                        break;
+                    default:
+                        return false;
                 }
-                cached=vid->_videoCache->getBefore(refPts);
                 if(cached)
                 {
                     if(cached->Pts>=seg->_refStartTimeUs)
@@ -433,10 +465,13 @@ bool        ADM_Composer::previousPicture(ADMImage *image)
         vid=_segments.getRefVideo(seg->_reference);
         
 
-        decodeTillPictureAtPts(targetPts,image);
+        if(false==decodeTillPictureAtPts(targetPts,image))
+        {
+            ADM_error("Cannot decode to get a candidate.\n");
+            return false;
+        }
         _currentSegment=segNo;
         // We may have overshot...
-        uint64_t last=vid->lastDecodedPts;
         _segments.LinearToRefTime(_currentSegment,targetPts,&refPts);
         ADMImage *candidate=vid->_videoCache->getLast();
         if(!candidate)
@@ -501,37 +536,62 @@ uint32_t segNo;
 */
 uint8_t ADM_Composer::dupe(ADMImage *src,ADMImage *dst,_VIDEOS *vid)
 {
-                if(src->_colorspace!=ADM_COLOR_YV12)
-                {
-                        // We need to do some colorspace conversion
-                        // Is there already one ?
-                        if(!vid->color)
-                        {
-                              //  vid->color=new COL_Generic2YV12(src->_width,src->_height,src->_colorspace);
-                              vid->color=new ADMColorScalerSimple(src->_width,src->_height,
-                                                                 src->_colorspace,ADM_COLOR_YV12);
-                        }
-                        // Since it is not YV12 it MUST be a ref
-                        ADM_assert(src->isRef());
-                        
-                        int       srcStrides[3];
-                        uint8_t  *srcPlanes[3];
-                        int       dstStrides[3];
-                        uint8_t  *dstPlanes[3];
+    if(src->_colorspace!=ADM_COLOR_YV12)
+    {
+        // We need to do some colorspace conversion
+        // Is there already one ?
+        if(!vid->color)
+        {
+            vid->color=new ADMColorScalerSimple(src->_width,src->_height,src->_colorspace,ADM_COLOR_YV12);
+        }
+        // Since it is not YV12 it MUST be a ref
+        ADM_assert(src->isRef());
 
-                        src->GetPitches(srcStrides);
-                        src->GetReadPlanes(srcPlanes);
-                        dst->GetPitches(dstStrides);
-                        dst->GetWritePlanes(dstPlanes);
+        int srcStrides[3];
+        uint8_t *srcPlanes[3];
+        int dstStrides[3];
+        uint8_t *dstPlanes[3];
+        bool needToSwapUV=false;
+        // The src image has U & V already swapped, so that YUV420P output by sws_scale
+        // has swapped colors, meaning correct colors when interpreted as YV12.
+        // If all the data is in Y we have to swap planes here.
+        switch(src->_colorspace)
+        {
+            case ADM_COLOR_RGB32A:
+            case ADM_COLOR_RGB24:
+            case ADM_COLOR_BGR24:
+            case ADM_COLOR_RGB555:
+            case ADM_COLOR_YUV422:
+            case ADM_COLOR_UYVY422:
+                needToSwapUV=true;
+            default:break;
+        }
 
-                        vid->color->convertPlanes(srcStrides,dstStrides,srcPlanes,dstPlanes);
-                        return 1;
-                }
-                // nothing to do
-//                if(_pp.swapuv)
-//#warning handle swap uv
-                dst->duplicate(src);
-                return 1;
+        src->GetPitches(srcStrides);
+        src->GetReadPlanes(srcPlanes);
+        dstStrides[0]=dst->GetPitch(PLANAR_Y);
+        dstPlanes[0]=dst->GetWritePtr(PLANAR_Y);
+        if(needToSwapUV)
+        {
+            dstStrides[2]=dst->GetPitch(PLANAR_U);
+            dstStrides[1]=dst->GetPitch(PLANAR_V);
+            dstPlanes[2]=dst->GetWritePtr(PLANAR_U);
+            dstPlanes[1]=dst->GetWritePtr(PLANAR_V);
+        }else
+        {
+            dstStrides[1]=dst->GetPitch(PLANAR_U);
+            dstStrides[2]=dst->GetPitch(PLANAR_V);
+            dstPlanes[1]=dst->GetWritePtr(PLANAR_U);
+            dstPlanes[2]=dst->GetWritePtr(PLANAR_V);
+        }
+
+        vid->color->convertPlanes(srcStrides,dstStrides,srcPlanes,dstPlanes);
+        return 1;
+    }
+    // nothing to do
+    //#warning handle swap uv
+    dst->duplicate(src);
+    return 1;
 }
 /**
     \fn setPostProc
@@ -592,10 +652,8 @@ bool        ADM_Composer::switchToSegment(uint32_t s,bool dontdecode)
         // If the refStartTime is 0, it is the first image
         // But the fist image might not be = 0
       _VIDEOS *vid=_segments.getRefVideo(seg->_reference);
-      vidHeader 	*demuxer=vid->_aviheader;  
       ADM_assert(vid);
       uint64_t from=seg->_refStartTimeUs;
-      uint64_t pts,dts;
 
       if(!from) from=vid->firstFramePts;
 
