@@ -21,7 +21,11 @@ using std::string;
 #include "ADM_edit.hxx"
 #include "ADM_vidMisc.h"
 
+#if 1
 #define pivotPrintf(...) {}
+#else
+#define pivotPrintf printf
+#endif
 
 /**
     \fn getNKFramePTS
@@ -32,7 +36,7 @@ uint64_t refTime,nkTime,segTime;
 int lastSeg=_segments.getNbSegments();
 uint32_t seg;
 bool switchedSeg=false;
-bool r;
+bool r=false;
     // 1- Convert frameTime to segments
     if(false== _segments.convertLinearTimeToSeg(  *frameTime, &seg, &segTime))
     {
@@ -46,10 +50,20 @@ again:
     // 2- Now search the previous keyframe in the ref image...
     // The time in reference = relTime+segmentStartTime
     refTime=s->_refStartTimeUs+segTime; // Absolute time in the reference image
-    
-    if(switchedSeg && refTime>0) refTime--; // The next segment may start with a keyframe
 
-    r=searchNextKeyFrameInRef(ref,refTime,&nkTime);
+    if(switchedSeg && !refTime)
+    {
+        _VIDEOS *vid=_segments.getRefVideo(ref);
+        if(!vid->firstFramePts)
+        {
+            nkTime=0;
+            r=true;
+        }
+    }
+    if(switchedSeg && refTime>0)
+        refTime--; // The next segment may start with a keyframe
+    if(r==false)
+        r=searchNextKeyFrameInRef(ref,refTime,&nkTime);
 
     // 3- if it does not belong to the same seg  ....
     if(r==false || nkTime >= (s->_refStartTimeUs+s->_durationUs))
@@ -68,6 +82,7 @@ again:
         seg++;
         switchedSeg=true;
         segTime=0;
+        r=false;
         goto again;
     }
     // Gotit, now convert it to the linear time
@@ -85,7 +100,6 @@ again:
 bool			ADM_Composer::getPKFramePTS(uint64_t *frameTime)
 {
 uint64_t refTime,nkTime,segTime;
-int lastSeg=_segments.getNbSegments();
 uint32_t seg;
 bool r;
     // 1- Convert frameTime to segments
@@ -103,10 +117,12 @@ bool r;
           if(!s->_refStartTimeUs)
           {
               uint64_t pts=vid->firstFramePts;
-              //
-              *frameTime+=pts;
-              ADM_warning("This video does not start at 0 but at %" PRIu64" ms, compensating\n",pts/1000);
-              _segments.convertLinearTimeToSeg(  *frameTime, &seg, &segTime);
+              if(pts)
+              {
+                  *frameTime+=pts;
+                  ADM_warning("This video does not start at 0 but at %" PRIu64" ms, compensating\n",pts/1000);
+                  _segments.convertLinearTimeToSeg(*frameTime, &seg, &segTime);
+              }
            }
       }
     // 
@@ -151,10 +167,8 @@ again:
 
 bool			ADM_Composer::getPtsDtsDelta(uint64_t frameTime, uint64_t *outDelta)
 {
-uint64_t refTime,nkTime,segTime;
-int lastSeg=_segments.getNbSegments();
+uint64_t refTime,segTime;
 uint32_t seg;
-bool r;
     // 1- Convert frameTime to segments
     if(false== _segments.convertLinearTimeToSeg(  frameTime, &seg, &segTime))
     {
@@ -195,13 +209,19 @@ bool r;
 */
 bool ADM_Composer::searchNextKeyFrameInRef(int ref,uint64_t refTime,uint64_t *nkTime)
 {
-    // Search from the end till we get a keyframe
+    int curFrame=0;
     _VIDEOS *v=_segments.getRefVideo(ref);
+    // look up the frame we are starting from...
+    if(!getFrameNumFromPtsOrBefore(v,refTime,curFrame))
+    {
+        ADM_warning("Cannot identify the frame for time %s in reference.\n",ADM_us2plain(refTime));
+    }
     uint32_t nbFrame=v->_nb_video_frames;
     uint64_t pts,dts;
-    for(int i=0;i<nbFrame;i++)
+    curFrame -= 16; // safety margin to deal with early B-frames
+    if(curFrame < 0) curFrame = 0;
+    for(int i=curFrame;i<nbFrame;i++)
     {
-        uint64_t p;
         uint32_t flags;
         v->_aviheader->getFlags(i,&flags);
         if(!(flags & AVI_KEY_FRAME)) continue;
@@ -230,18 +250,27 @@ bool ADM_Composer::searchNextKeyFrameInRef(int ref,uint64_t refTime,uint64_t *nk
  */
 bool ADM_Composer::getFrameNumFromPtsOrBefore(_VIDEOS *v,uint64_t refTime,int &frameNumber)
 {
+    if(refTime == ADM_NO_PTS)
+        return false;
+    if(refTime <= v->firstFramePts)
+    {
+        frameNumber = 0;
+        return true;
+    }
+
     uint32_t nbFrame = v->_nb_video_frames;
-    uint64_t pts, dts;
+    uint64_t pts, dts, margin = v->timeIncrementInUs * 16;
+    const uint64_t rangeBegin = (refTime < margin)? 0 : (refTime - margin);
+    const uint64_t rangeEnd = refTime + margin;
     uint32_t curFrame = nbFrame >> 1;
     uint32_t splitMoval = (curFrame + 1) >> 1;
     uint32_t minFrame = 0;
+    bool worked = false;
     pivotPrintf("Looking for frame with pts %" PRIu64" us (%s)\n",refTime,ADM_us2plain(refTime));
     // Try to find the frame that as the timestamp close enough to refTime, while being smaller
     do 
     {
         // seems like the pts determination, not always works -> retry if necessary
-        bool worked;
-        uint32_t skipped = 0;
         uint32_t tmpFrame = curFrame;
         do
         {
@@ -249,54 +278,47 @@ bool ADM_Composer::getFrameNumFromPtsOrBefore(_VIDEOS *v,uint64_t refTime,int &f
             worked = v->_aviheader->getPtsDts(tmpFrame,&pts,&dts);
             if(worked && pts != ADM_NO_PTS)
                 break; // found
-            tmpFrame--;
-            if(!tmpFrame)
+            if(tmpFrame <= minFrame)
             {
-                ADM_warning("The whole segment is corrupted. Aborting the search");
-                return false;
+                ADM_warning("Frames between %" PRIu32" and %" PRIu32" have no PTS.\n",minFrame,curFrame);
+                break;
             }
-            skipped++;
+            tmpFrame--;
         } while(true);
-        if(pts == refTime)
-            break;
-        if(skipped && pts < refTime)
-        { // if we slid past the target frame...
-            pivotPrintf("skipped %" PRIu32" frames seeking back\n",skipped);
-            skipped = 0;
-            tmpFrame = curFrame; // ...go back to the frame where we first encountered corrupted video...
+        if(pts == ADM_NO_PTS)
+        {
+            tmpFrame = curFrame + 1;
             do
-            { // ...and seek forward until we get a valid pts
-                pts = ADM_NO_PTS;
+            {
+                if(tmpFrame >= nbFrame)
+                {
+                    ADM_warning("The whole segment is corrupted, aborting seek.\n");
+                    return false;
+                }
+                minFrame = tmpFrame;
                 worked = v->_aviheader->getPtsDts(tmpFrame,&pts,&dts);
                 if(worked && pts != ADM_NO_PTS)
-                    break; // found                
+                    break; // found
                 tmpFrame++;
-                if(tmpFrame == nbFrame)
-                {
-                    ADM_warning("The whole segment is corrupted. Aborting the search");
-                    return false;
-                }
-                skipped++;
             } while(true);
-            if(skipped)
-            {
-                pivotPrintf("skipped %" PRIu32" frames seeking forward\n",skipped);
-                if(pts > refTime)
-                {
-                    ADM_warning("The video at the specified time in ref is corrupted. Aborting seek");
-                    return false;
-                }else
-                {
-                    minFrame = tmpFrame;
-                }
-            }
-            curFrame = tmpFrame;
         }
+        curFrame = tmpFrame; // resync
+        // are we lucky?
+        if(pts == refTime)
+        {
+            pivotPrintf("Perfect match for frame %" PRIu32"\n",curFrame);
+            frameNumber = curFrame;
+            return true;
+        }
+        // nope, are we at least close enough?
+        if(pts != ADM_NO_PTS && pts >= rangeBegin && pts <= rangeEnd)
+            break; // switch to linear search
+        // nope, continue to bisect
         pivotPrintf("SplitMoval=%d\n",splitMoval);
-        if(!splitMoval)
-            break;        
+        if(splitMoval < 16)
+            break;
         pivotPrintf("Pivot frame is %" PRIu32" at %s\n",curFrame,ADM_us2plain(pts));
-        if (pts >= refTime)
+        if (pts > refTime)
         {
             if (curFrame <= splitMoval + minFrame)
             {
@@ -311,18 +333,47 @@ bool ADM_Composer::getFrameNumFromPtsOrBefore(_VIDEOS *v,uint64_t refTime,int &f
             if(curFrame >= nbFrame)
                 curFrame = nbFrame - 1;
         }
-        if(splitMoval > 1)
-            splitMoval++;
+        splitMoval++;
         splitMoval >>= 1;
-        pivotPrintf("Split=%d\n",splitMoval);        
     } while(refTime != pts);
 
-    pivotPrintf("Matching frame is %" PRIu32" at %s\n",curFrame,ADM_us2plain(pts));
-    if(pts > refTime && curFrame)
-        curFrame--;
-    pivotPrintf("Our target frame is %" PRIu32"\n",curFrame);
-    frameNumber = curFrame;
-    return true;
+    // linear search with a generous safety margin
+    curFrame = (curFrame < 32)? 0 : curFrame - 32;
+    pivotPrintf("Starting linear search at frame %" PRIu32", total frames: %" PRIu32"\n",curFrame,nbFrame);
+    int candidate = -1;
+    uint64_t last = 0;
+    for(int i = 0; i < 65; i++)
+    {
+        if(curFrame >= nbFrame) break;
+        pts = ADM_NO_PTS;
+        worked = v->_aviheader->getPtsDts(curFrame,&pts,&dts);
+        if(!worked || pts == ADM_NO_PTS)
+        {
+            pivotPrintf("Linear search, skipping frame %" PRIu32"\n",curFrame);
+            curFrame++;
+            continue;
+        }
+        if(pts == refTime)
+        {
+            pivotPrintf("Perfect match for frame %" PRIu32"\n",curFrame);
+            frameNumber = curFrame;
+            return true;
+        }
+        if(pts >= rangeBegin && (last? pts > last : true) && pts < refTime)
+        {
+            candidate = curFrame;
+            last = pts;
+        }
+        curFrame++;
+    }
+    if(candidate >= 0)
+    {
+        ADM_info("Best candidate for time %s in reference is frame %" PRIu32"\n",ADM_us2plain(refTime),candidate);
+        frameNumber = candidate;
+        return true;
+    }
+    ADM_warning("Search for frame matching time %s in reference failed.\n",ADM_us2plain(refTime));
+    return false;
 }
 /**
     \fn searchPreviousKeyFrameInRef
@@ -335,9 +386,10 @@ bool ADM_Composer::getFrameNumFromPtsOrBefore(_VIDEOS *v,uint64_t refTime,int &f
 
 bool ADM_Composer::searchPreviousKeyFrameInRef(int ref,uint64_t refTime,uint64_t *nkTime)
 {
+    if(!refTime)
+        return false;
     // Search for the current frame with quick search
     _VIDEOS *v = _segments.getRefVideo(ref);
-    uint32_t nbFrame = v->_nb_video_frames;
     uint64_t pts, dts;
     int curFrame;
 
@@ -350,7 +402,6 @@ bool ADM_Composer::searchPreviousKeyFrameInRef(int ref,uint64_t refTime,uint64_t
     // rewind until we find a keyframe
     for (int i=curFrame; i>=0; i--) 
     {
-    uint64_t p;
     uint32_t flags;
             v->_aviheader->getFlags(i,&flags);
             if(!(flags & AVI_KEY_FRAME)) 
@@ -385,8 +436,7 @@ bool ADM_Composer::searchPreviousKeyFrameInRef(int ref,uint64_t refTime,uint64_t
 */
 bool        ADM_Composer::getDtsFromPts(uint64_t *time)
 {
-uint64_t refTime,nkTime,segTime;
-int lastSeg=_segments.getNbSegments();
+uint64_t segTime;
 uint32_t seg;
     // 1- Convert frameTime to segments
     if(false== _segments.convertLinearTimeToSeg(  *time, &seg, &segTime))
@@ -448,13 +498,22 @@ uint64_t    ADM_Composer::getLastKeyFramePts(void)
               pts=ADM_NO_PTS;    
               v->_aviheader->getFlags(frame,&flags);
               if(!(flags & AVI_KEY_FRAME)) continue;
+              // if B-frames are present, pts doesn't necessarily become smaller when frame is decremented,
+              // we must check whether pts is still in range and effectively go a full GOP backwards if not
+              ADM_info("frame %d is an intra, checking whether pts is in range...\n",frame);
               v->_aviheader->getPtsDts(frame,&pts,&dts);
-              if(pts==ADM_NO_PTS) continue;          
+              if(pts==ADM_NO_PTS) continue;
+              if(pts>=endTimeInRef)
+              {
+                  ADM_warning("frame %d pts in ref = %" PRIu64" is not in range, skipping\n",frame,pts);
+                  continue;
+              }
               ADM_info("found last keyframe at %d, time in reference=%s\n",frame,ADM_us2plain(pts));
               break;
           }
           if(pts!=ADM_NO_PTS)
           {
+              if(pts<s->_refStartTimeUs) continue; // no keyframe in the last segment, retry with the previous one
               pts+=s->_startTimeUs-s->_refStartTimeUs;
               ADM_info("found last keyframe at %s\n",ADM_us2plain(pts));
               return pts;
