@@ -50,20 +50,21 @@
 */
 ADM_Composer::ADM_Composer (void)
 {
-uint32_t type,value;
-
-
   _pp=NULL;
   _imageBuffer=NULL;
   _internalFlags=0;
   _currentSegment=0;
   _scratch=NULL;
+  _undo_counter=0;
   currentProjectName=std::string("");
 
   _currentPts = 0;
   markerAPts = 0;
   markerBPts = 0;
   stats.reset();
+
+    totalExtraDelay=0;
+    desyncScore=0;
 }
 /**
 	Remap 1:1 video to segments
@@ -157,11 +158,11 @@ bool ADM_Composer::setProjectName(const std::string &pj)
     \brief	Load or append a file.	The file type is determined automatically and the ad-hoc video decoder is spawned
 
     @param name: filename
-    @return 1 on success, 0 on failure
+    @return 1 on success, 0 on failure, ADM_IGN when cancelled by user.
 
 
 */
-bool ADM_Composer::addFile (const char *name)
+uint8_t ADM_Composer::addFile (const char *name)
 {
 
   uint8_t    ret =    0;
@@ -233,6 +234,12 @@ bool ADM_Composer::addFile (const char *name)
       return false;
    }
 
+    if(ret==ADM_IGN)
+    {
+        ADM_info("Cancelled by user\n");
+        return ret;
+    }
+
    /* check for resolution */
    if( _segments.getNbRefVideos())
     {
@@ -244,13 +251,22 @@ bool ADM_Composer::addFile (const char *name)
         {
          char str[512+1];
          str[0] = '\0';
+         int both=-1;
          if( info0.width != infox.width )
-            strcpy(str,"width");
+         {
+            both++;
+            strcpy(str,QT_TRANSLATE_NOOP("ADM_Composer","width"));
+         }
          if( info0.height != infox.height )
+         {
+            both++;
             snprintf(str+strlen(str),512-strlen(str),
-              "%sheight%sdifferent between first and this video stream",
-                 (strlen(str)?" and ":""),
-                 (strlen(str)?" are ":" is ") );
+                QT_TRANSLATE_NOOP("ADM_Composer","%sheight"),
+                (both? QT_TRANSLATE_NOOP("ADM_Composer"," and ") : ""));
+         }
+         snprintf(str+strlen(str),512-strlen(str),
+            QT_TRANSLATE_NOOP("ADM_Composer","%sdifferent between first and this video stream"),
+            (both? QT_TRANSLATE_NOOP("ADM_Composer"," are ") : QT_TRANSLATE_NOOP("ADM_Composer"," is ")));
          str[512] = '\0';
          GUI_Error_HIG(str,QT_TRANSLATE_NOOP("ADM_Composer","You cannot mix different video dimensions yet. Using the partial video filter later, will not work around this problem. The workaround is:\n1.) \"resize\" / \"add border\" / \"crop\" each stream to the same resolution\n2.) concatenate them together"));
          delete video._aviheader;
@@ -276,10 +292,9 @@ bool ADM_Composer::addFile (const char *name)
     if(!_segments.getNbRefVideos())
     {
         uint32_t type=0,value=0;
-#if 0
-        if(!prefs->get(DEFAULT_POSTPROC_TYPE,&type)) type=3;
-        if(!prefs->get(DEFAULT_POSTPROC_VALUE,&value)) value=3;
-#endif
+        prefs->get(DEFAULT_POSTPROC_TYPE,&type);
+        prefs->get(DEFAULT_POSTPROC_VALUE,&value);
+
         if(_pp) delete _pp;
         _pp=new ADM_PP(info.width,info.height);
         _pp->postProcType=type;
@@ -320,7 +335,6 @@ bool ADM_Composer::addFile (const char *name)
   else
     {
         // Read and construct the audio tracks for that videos
-      audioInfo *info;
       uint32_t extraLen;
       uint8_t  *extraData;
       ADM_audioStream *stream;
@@ -334,6 +348,13 @@ bool ADM_Composer::addFile (const char *name)
             ADM_audioStreamTrack *track=new ADM_audioStreamTrack;
 
             header=thisVid->_aviheader->getAudioInfo(i );
+
+            if(header->encoding==0x706d)
+            {
+                ADM_info("Mapping codec ID 0x706d to AAC.\n");
+                header->encoding=WAV_AAC;
+            }
+
             memcpy(&(track->wavheader),header,sizeof(*header));
 
             thisVid->_aviheader->getAudioStream(i,&stream);
@@ -346,6 +367,27 @@ bool ADM_Composer::addFile (const char *name)
             stream->getExtraData(&extraLen,&extraData);
             track->codec=getAudioCodec(header->encoding,header,extraLen,extraData);
 
+            // sanity check
+#define SKIP_TRACK delete track; track=NULL; continue;
+            if(!track->codec) // actually this can't happen, but better safe than sorry
+            {
+                ADM_warning("Codec is NULL, rejecting track %d\n",i);
+                SKIP_TRACK
+            }
+            if((track->wavheader).frequency < MIN_SAMPLING_RATE || (track->wavheader).frequency > MAX_SAMPLING_RATE)
+            {
+                ADM_warning("Sampling frequency %" PRIu32" for track %d out of bounds, rejecting track.\n",(track->wavheader).frequency,i);
+                SKIP_TRACK
+            }
+            if(!track->codec->getOutputChannels() || track->codec->getOutputChannels() > MAX_CHANNELS)
+            {
+                ADM_warning("Invalid number of channels %d for track %d, rejecting track.\n",track->codec->getOutputChannels(),i);
+                SKIP_TRACK
+            }
+
+            if((track->wavheader).encoding==WAV_AAC)
+                track->isbr=checkSamplingFrequency(track);
+
             thisVid->audioTracks.push_back(track);
             if(!_segments.getNbRefVideos()) // 1st video..
             {
@@ -356,7 +398,7 @@ bool ADM_Composer::addFile (const char *name)
       if(!_segments.getNbRefVideos()) // only for 1st video
       {
         activeAudioTracks.clear();
-        for(int i=0;i<nbAStream;i++)
+        for(int i=0;i<audioTrackPool.size();i++)
             activeAudioTracks.addTrack(i,audioTrackPool.at(i)); // default add 1st track of video pool
       }
     }
@@ -364,16 +406,22 @@ bool ADM_Composer::addFile (const char *name)
   printf ("[Editor] Decoder FCC: ");
   fourCC::print (info.fcc);
   printf("\n");
-  // ugly hack
-  if (info.fps1000 > 2000 * 1000)
+
+    // ugly hack
+    bool fpsTooHigh=false;
+    // we switch to 25 fps hardcoded in addReferenceVideo and update info here
+    if (info.fps1000 > 2000 * 1000)
     {
-      printf ("[Editor] FPS too high, switching to 25 fps hardcoded\n");
-      info.fps1000 = 25 * 1000;
-      updateVideoInfo (&info);
+        ADM_warning("FPS too high, switching to 25 fps hardcoded\n");
+        fpsTooHigh=true;
+        info.fps1000 = 25 * 1000;
     }
-    
+
     _segments.addReferenceVideo(&video);
-    
+
+    if(fpsTooHigh)
+        updateVideoInfo(&info);
+
     // we only try if we got everything needed...
     // Verify DTS is monotonous
     ADM_verifyDts(video._aviheader,video.timeIncrementInUs);
@@ -385,7 +433,8 @@ bool ADM_Composer::addFile (const char *name)
         if(video.decoder->bFramePossible())
         {
             printf("[Editor] B- frame possible with that codec \n");
-            if(isMpeg4Compatible(info.fcc) || isMpeg12Compatible(info.fcc) || isVC1Compatible(info.fcc))
+#define FCC_MATCHES(x) fourCC::check(info.fcc,(uint8_t *)x)
+            if(isMpeg4Compatible(info.fcc) || isMpeg12Compatible(info.fcc) || FCC_MATCHES("VC1 ") || FCC_MATCHES("WMV3"))
             {
                 ADM_info("[Editor] It is mpeg4-SP/ASP, try to guess all PTS\n");
                 uint64_t delay;
@@ -412,9 +461,9 @@ bool ADM_Composer::addFile (const char *name)
         }
      }
     int lastVideo=_segments.getNbSegments();
-    if(lastVideo && video._aviheader->unreliableBFramePts()&& isH264Compatible(info.fcc))
+    if(lastVideo && (isH264Compatible(info.fcc) || FCC_MATCHES("WVC1")))
     {
-        ADM_info("H264 in mp4 sometimes has invalid timestamps which confuse avidemux, checking\n");
+        ADM_info("%s sometimes has invalid timestamps which confuse avidemux, checking\n",fourCC::tostring(info.fcc));
         checkForValidPts(_segments.getSegment(lastVideo-1)); 
     }
     if(true==checkForDoubledFps( video._aviheader,video.timeIncrementInUs))
@@ -422,6 +471,7 @@ bool ADM_Composer::addFile (const char *name)
         ADM_info("Halving Fps...\n");
         _segments.halfFps();
     }
+    endOfStream=false;
   return 1;
 }
 #if 0
@@ -443,6 +493,50 @@ bool ADM_Composer::hasVBRAudio(void)
         return 0;
 }
 #endif
+/**
+    \fn checkSamplingFrequency
+    \brief Check AAC for implicit SBR
+*/
+bool ADM_Composer::checkSamplingFrequency(ADM_audioStreamTrack *track)
+{
+    if(!track) return false;
+    if(!track->codec) return false;
+    if(track->codec->isDummy()) return false;
+
+    uint32_t chan=track->codec->getOutputChannels();
+    if(chan<2) chan=2;
+
+    WAVHeader *hdr=&(track->wavheader);
+    if(hdr->channels>chan)
+        chan=hdr->channels;
+
+    uint32_t len=(hdr->frequency)*chan; // 1 sec max
+    uint32_t max=ADM_EDITOR_PACKET_BUFFER_SIZE;
+
+    notStackAllocator inbuf(max);
+    uint8_t *in=inbuf.data;
+
+    uint32_t inlen,samples;
+    uint64_t dts;
+    if(false==track->stream->getPacket(in,&inlen,max,&samples,&dts))
+        return false;
+
+    notStackAllocator outbuf(len*sizeof(float));
+    float *out=(float *)outbuf.data;
+
+    uint32_t nbOut;
+    if(false==track->codec->run(in,inlen,out,&nbOut))
+        return false;
+
+    uint32_t fq=track->codec->getOutputFrequency();
+    if(fq && fq!=hdr->frequency)
+    {
+        ADM_warning("Updating sampling frequency from %u to %u\n",hdr->frequency,fq);
+        hdr->frequency=fq;
+        return true; // implicit SBR
+    }
+    return false;
+}
 /**
     \fn getPARWidth
 
@@ -520,6 +614,7 @@ uint8_t ADM_Composer::cleanup (void)
   
   _segments.deleteAll();
   _currentPts = 0;
+  _currentSegment = 0;
   markerAPts = 0;
   markerBPts = 0;
   audioTrackPool.clear();
@@ -590,10 +685,6 @@ uint32_t ref;
 */
 bool ADM_Composer::changeAudioStream(uint64_t xtime,uint32_t newstream)
 {
-double     duration;
-WAVHeader *wav;
-aviInfo    info;
-uint32_t ref;
         int n=_segments.getNbRefVideos();
         for(int i=0;i<n;i++)
         {
