@@ -27,6 +27,7 @@
 #include "ADM_codec.h"
 #include "DIA_coreToolkit.h"
 #include "ADM_vidMisc.h"
+#include "prefs.h"
 
 #if 1
 #define aprintf printf
@@ -76,16 +77,19 @@ bool        ADM_EditorSegment::updateRefVideo(void)
             ADM_info("First PTS is %s\n",ADM_us2plain(pts));
         }
 
-    updateStartTime();
     //
     n=segments.size();
     if(n)
     {
     _SEGMENT *seg=getSegment(n-1);
     uint64_t dur=ref->_aviheader->getVideoDuration();
+#ifdef ADM_ZERO_OFFSET
+    seg->_refStartTimeUs=pts;
+    seg->_durationUs=dur-pts;
+#endif
     printf("Current duration %" PRIu64" ms real one %" PRIu64" ms\n",dur/1000,seg->_durationUs/1000);
     }
-
+    updateStartTime();
     return true;
 }
 /**
@@ -98,12 +102,22 @@ bool        ADM_EditorSegment::addReferenceVideo(_VIDEOS *ref)
   uint8_t 	    *d;
   aviInfo       info;
   _SEGMENT      seg;
+  uint32_t      cacheSize;
+
+  if(false==prefs->get(FEATURES_CACHE_SIZE,&cacheSize))
+      cacheSize = EDITOR_CACHE_MAX_SIZE;
+  if(cacheSize > EDITOR_CACHE_MAX_SIZE) cacheSize = EDITOR_CACHE_MAX_SIZE;
+  if(cacheSize < EDITOR_CACHE_MIN_SIZE) cacheSize = EDITOR_CACHE_MIN_SIZE;
 
   ref->dontTrustBFramePts=ref->_aviheader->unreliableBFramePts();
   ref->_aviheader->getVideoInfo (&info);
   ref->_aviheader->getExtraHeaderData (&l, &d);
   ref->decoder = ADM_getDecoder (info.fcc,  info.width, info.height, l, d,info.bpp);
-  ref->_videoCache   =   new EditorCache(8,info.width,info.height) ;
+  ref->_videoCache = new EditorCache(cacheSize,info.width,info.height);
+
+  // discard implausibly high fps, hardcode the value to 25 fps
+  if (info.fps1000 > 2000 * 1000)
+      info.fps1000 = 25 * 1000;
 
   float frameD=info.fps1000;
   frameD=frameD/1000;
@@ -114,10 +128,11 @@ bool        ADM_EditorSegment::addReferenceVideo(_VIDEOS *ref)
   // Probe the real time increment as the value determined from FPS may be incorrect due to interlace
   uint64_t firstNonZeroDts=ADM_NO_PTS,pts,dts;
   int firstNonZeroDtsFrame;
-  ADM_info("[editor] Original frame increment %s %d\n",ADM_us2plain(ref->timeIncrementInUs),(int)ref->timeIncrementInUs);
+  ADM_info("Original frame increment %s = %" PRIu64" us\n",ADM_us2plain(ref->timeIncrementInUs),ref->timeIncrementInUs);
   uint64_t minDelta=100000;
   uint64_t maxDelta=0;
-  for (int frame=0; frame<info.nb_frames; frame++)
+  uint32_t fmin=0,fmax=0;
+  for (uint32_t frame=0; frame<info.nb_frames; frame++)
   {
       if (ref->_aviheader->getPtsDts(frame,&pts,&dts) && dts!=ADM_NO_PTS && dts!=0)
       {
@@ -129,20 +144,25 @@ bool        ADM_EditorSegment::addReferenceVideo(_VIDEOS *ref)
           }
 
           uint64_t probedTimeIncrement=(dts-firstNonZeroDts)/(frame-firstNonZeroDtsFrame);
-          if(probedTimeIncrement<minDelta) minDelta=probedTimeIncrement;
-          if(probedTimeIncrement>maxDelta) maxDelta=probedTimeIncrement;
+          if(probedTimeIncrement<minDelta) { minDelta=probedTimeIncrement; fmin=frame; }
+          if(probedTimeIncrement>maxDelta) { maxDelta=probedTimeIncrement; fmax=frame; }
           firstNonZeroDts=dts;
           firstNonZeroDtsFrame=frame;
       }
   }
-  ADM_info("[Editor] min increment %s - %d\n",ADM_us2plain(minDelta),(int)minDelta);
-  ADM_info("[Editor] max increment %s - %d\n",ADM_us2plain(maxDelta),(int)maxDelta);
-  
+  if(maxDelta>=minDelta)
+  {
+      ADM_info("min increment %s = %" PRIu64" us for frame %d\n",ADM_us2plain(minDelta),minDelta,fmin);
+      ADM_info("max increment %s = %" PRIu64" us for frame %d\n",ADM_us2plain(maxDelta),maxDelta,fmax);
+  }else
+  {
+      ADM_warning("DTS missing, cannot probe time increment.\n");
+  }
   //if (minDelta==ref->timeIncrementInUs*2)
               //ref->timeIncrementInUs=minDelta;
 
   
-  ADM_info("[Editor] About %" PRIu64" microseconds per frame\n",ref->timeIncrementInUs);
+  ADM_info("About %" PRIu64" microseconds per frame\n",ref->timeIncrementInUs);
   ref->_nb_video_frames = info.nb_frames;
   //
   //  And automatically create the segment
@@ -164,10 +184,18 @@ bool        ADM_EditorSegment::addReferenceVideo(_VIDEOS *ref)
         demuxer->getPtsDts(0,&pts,&dts);
         ref->firstFramePts=0;
         if(pts==ADM_NO_PTS) ADM_warning("First frame has unknown PTS\n");
+        if(dts==ADM_NO_PTS) ADM_warning("The first frame DTS is not set\n");
+        else ADM_info("The first frame DTS = %" PRIu64" ms\n",dts/1000);
         if(pts!=ADM_NO_PTS &&pts)
         {
-            ADM_warning("The first frame has a PTS >0, adjusting to %" PRIu64" ms\n",pts/1000);
+            ADM_warning("The first frame has a PTS > 0, adjusting to %" PRIu64" %s\n",
+                (pts>=1000)? pts/1000 : pts,
+                (pts>=1000)? "ms" : "us");
             ref->firstFramePts=pts;
+#ifdef ADM_ZERO_OFFSET
+            seg._refStartTimeUs=pts;
+            seg._durationUs-=pts;
+#endif
         }
 
     segments.push_back(seg);
@@ -221,11 +249,13 @@ bool ADM_EditorSegment::deleteAll (void)
           v->_aviheader->close ();
           delete v->_aviheader;
       }
-    
+      if(v->paramCache)
+          delete [] v->paramCache;
       v->_videoCache=NULL;
       v->color=NULL;
       v->decoder=NULL;
       v->_aviheader=NULL;
+      v->paramCache=NULL;
      // Delete audio codec too
      // audioStream will be deleted by the demuxer
 
@@ -267,6 +297,15 @@ bool        ADM_EditorSegment::resetSegment(void)
         _VIDEOS  *vid=&(videos[i]);
 
         seg._durationUs=vid->_aviheader->getVideoDuration();
+#ifdef ADM_ZERO_OFFSET
+        uint64_t pts=vid->firstFramePts;
+        if(pts!=ADM_NO_PTS && pts)
+        {
+            ADM_warning("The first frame in ref video %d has PTS = %" PRIu64" us, adjusting the segment\n",i,pts);
+            seg._refStartTimeUs=pts;
+            seg._durationUs-=pts;
+        }
+#endif
         seg._reference=i;
         vid->_aviheader->getVideoInfo(&info);
         segments.push_back(seg);
@@ -372,7 +411,7 @@ bool         ADM_EditorSegment::updateStartTime(void)
         {
             uint64_t pts2,dts2;
             demuxer->getPtsDts(0,&pts2,&dts2);
-            if(pts2!=ADM_NO_PTS)
+            if(pts2 && pts2!=ADM_NO_PTS)
             {
                 ADM_info("Using pts2=%s to get dts, as this video does not start at zero\n",ADM_us2plain(pts2));
                 pts=pts2;
@@ -451,7 +490,7 @@ bool        ADM_EditorSegment::convertLinearTimeToSeg(  uint64_t frameTime, uint
 {
     if(!frameTime && segments.size()) // pick the first one
     {
-        ADM_info("Frame time=0, taking first segment \n");
+        //ADM_info("Frame time=0, taking first segment \n");
         *seg=0;
         *segTime=0; // ??
         return true;
@@ -607,8 +646,8 @@ bool        ADM_EditorSegment::removeChunk(uint64_t from, uint64_t to)
     {
         segments.erase(segments.begin()+startSeg+1);
     }
-    updateStartTime();
     removeEmptySegments();
+    updateStartTime();
     if(isEmpty())
     {
         GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Error"),QT_TRANSLATE_NOOP("adm","You cannot remove *all* the video\n"));
@@ -641,16 +680,18 @@ bool ADM_EditorSegment::truncateVideo(uint64_t from)
     ListOfSegments tmp=segments;
 
     _SEGMENT *first=getSegment(startSeg);
+    _VIDEOS *vid=getRefVideo(first->_reference);
+    uint64_t padding=vid->timeIncrementInUs/2;
     // shorten the start segment
-    first->_durationUs=startOffset;
+    first->_durationUs=startOffset+padding;
     // remove following segments
     int n=segments.size();
     for(int i=startSeg+1;i<n;i++)
     {
         segments.erase(segments.begin()+startSeg+1);
     }
-    updateStartTime();
     removeEmptySegments();
+    updateStartTime();
     if(isEmpty())
     {
         GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Error"),QT_TRANSLATE_NOOP("adm","You cannot remove *all* the video\n"));
@@ -789,7 +830,9 @@ bool        ADM_EditorSegment::removeEmptySegments(void)
                 _SEGMENT *seg=getSegment(i);
                 _VIDEOS  *vid=this->getRefVideo(seg->_reference);
                 if(seg->_durationUs==0) index=i;
-                if(seg->_refStartTimeUs==0 && seg->_durationUs==vid->firstFramePts) index=i; 
+#ifndef ADM_ZERO_OFFSET
+                if(seg->_refStartTimeUs==0 && seg->_durationUs==vid->firstFramePts) index=i;
+#endif
         }
         if(index==-1) break;
         segments.erase(segments.begin()+index);
@@ -934,6 +977,9 @@ bool        ADM_EditorSegment::pasteFromClipBoard(uint64_t currentTime)
         newSegs.push_back(s);
     }
     segments=newSegs;
+    // If a video doesn't start at zero and we paste to its first frame,
+    // we end up with an empty segment at the beginning. Remove it.
+    removeEmptySegments();
     updateStartTime();
     dump();
     return true;
