@@ -35,11 +35,15 @@ static bool updatePtsAndDts(vidHeader *hdr,uint64_t timeIncrementUs,uint64_t *de
 */
 bool ADM_verifyDts(vidHeader *hdr,uint64_t timeIncrementUs)
 {
-        int nbMissing=0;
         aviInfo info;
         hdr->getVideoInfo(&info);
         uint32_t nbFrames=0;
         nbFrames=info.nb_frames;
+        if(!nbFrames)
+        {
+            ADM_warning("The demuxer reports zero frames in this video, can't verify DTS\n");
+            return false;
+        }
         //
         ADM_info("Verifying DTS....\n");
         uint64_t got_dts=0,startDts=0;
@@ -248,6 +252,7 @@ bool ADM_setH264MissingPts(vidHeader *hdr,uint64_t timeIncrementUs,uint64_t *del
     if(!fail) return true; // Have all PTS, ok...
     ADM_info("Some PTS are missing, try to guess them...\n");
     fail=0;
+    bool interlaced=false;
     //
     for(int i=0;i<nbFrames-1;i+=1)
     {
@@ -263,6 +268,7 @@ bool ADM_setH264MissingPts(vidHeader *hdr,uint64_t timeIncrementUs,uint64_t *del
         {
             continue;
         }
+        interlaced=true;
         // TOP / BOTTOM
         if(pts==ADM_NO_PTS)
         {
@@ -271,7 +277,7 @@ bool ADM_setH264MissingPts(vidHeader *hdr,uint64_t timeIncrementUs,uint64_t *del
         }
     }
     ADM_info("H264 AVC scheme: %" PRIu32"/%" PRIu32" failures.\n",fail,nbFrames/2);
-    if(fail) goto nextScheme;
+    if(!interlaced || fail) goto nextScheme;
     {
     ADM_info("Filling 2nd field PTS\n");
     uint32_t fixed=0;
@@ -340,11 +346,11 @@ theEnd:
 bool ADM_computeMP124MissingPtsDts(vidHeader *hdr,uint64_t timeIncrementUs,uint64_t *delay)
 {
     aviInfo info;
-    uint32_t flags;
+    uint32_t flags,previousFlags=0;
     hdr->getVideoInfo(&info);
     uint32_t nDts=0,nPts=0;
-    uint32_t nbB=0;
-    uint64_t pts,dts;
+    uint32_t nbB=0,ooo=0;
+    uint64_t pts,dts,previousPts=ADM_NO_PTS;
     uint32_t nbFields=0,nbFrames=0;
     // Look how much bframes + how much valid PTS/DTS we have....
     for(int i=0;i<info.nb_frames;i++)
@@ -364,24 +370,37 @@ bool ADM_computeMP124MissingPtsDts(vidHeader *hdr,uint64_t timeIncrementUs,uint6
             nPts++;
         if(dts!=ADM_NO_PTS)
             nDts++;
+        if(pts!=ADM_NO_PTS && previousPts!=ADM_NO_PTS && (flags & AVI_B_FRAME) && !(previousFlags & AVI_B_FRAME))
+        {
+            if(pts>previousPts)
+                ooo++;
+        }
+        previousPts=pts;
+        previousFlags=flags;
     }
 next:
         ADM_info("Out of %" PRIi32" frames, we have %" PRIi32" valid DTS and %" PRIi32" valid PTS\n",info.nb_frames,nDts,nPts);
         ADM_info("We also have %" PRIi32" bframes\n",nbB);
+        if(ooo)
+            ADM_info("Unreliable PTS detected\n");
         ADM_info("We have %" PRIu32" fields and %" PRIu32" frames\n",nbFields,nbFrames);
         if(nbFields>2)
         {
             ADM_info("Cannot recompute PTS/DTS for field encoded picture.\n");
             return true;
         }
-        
         // Case 1 : We have both, nothing to do
         if(nDts>=info.nb_frames-2 && nPts>=info.nb_frames-2)
-         {
-                ADM_info("Nothing to do\n");
-                *delay=0;
-                return true;
-         }
+        {
+            if(ooo)
+            {
+                ADM_info("Trying to correct out-of-order PTS\n");
+                return setPtsFromDts(hdr,timeIncrementUs,delay);
+            }
+            ADM_info("Nothing to do\n");
+            *delay=0;
+            return true;
+        }
         // Case 2: We have PTS but not DTS
         if(nPts>=info.nb_frames-2 )
         {
@@ -536,10 +555,7 @@ bool setDtsFromPts(vidHeader *hdr,uint64_t timeIncrementUs,uint64_t *delay)
     *delay=0;
     int last=0;
     int nbFrames;
-    int nbBframe=0;
-    int maxBframe=0;
     uint32_t flags;
-    int nbB=0;
     uint64_t pts,dts;
 
     ADM_info("computing dts...\n");
@@ -557,19 +573,15 @@ bool setDtsFromPts(vidHeader *hdr,uint64_t timeIncrementUs,uint64_t *delay)
         }
         if(flags & AVI_B_FRAME)
         {
-                
-                nbB++;
                 dts=pts;
                 hdr->setPtsDts(i,pts,dts);
                 continue;
         }
    
         uint64_t oldPts,oldDts;
-        uint64_t fwdPts,fwdDts;
           hdr->getPtsDts(last,&oldPts,&oldDts);
           dts=oldPts;
           hdr->setPtsDts(i,pts,dts);
-        nbBframe=0;
         last=i;
     }
     return 1;
@@ -582,7 +594,7 @@ bool setDtsFromPts(vidHeader *hdr,uint64_t timeIncrementUs,uint64_t *delay)
 bool updatePtsAndDts(vidHeader *hdr,uint64_t timeIncrementUs,uint64_t *delay)
 {
     aviInfo info;
-    uint64_t offset,pts,dts;
+    uint64_t pts,dts;
     uint32_t nbFrames;
     uint64_t myDelay=0;
     *delay=0;
@@ -635,7 +647,7 @@ bool updatePtsAndDts(vidHeader *hdr,uint64_t timeIncrementUs,uint64_t *delay)
                     {
                         double deltaTime=dts-backDts;
                         deltaTime=deltaTime/(deltaFrame);
-                        if(deltaTime>41700 & deltaTime<41800)
+                        if(deltaTime>41700 && deltaTime<41800)
                         {
                             for(int j=backIndex+1;j<i;j++)
                             {
