@@ -251,6 +251,11 @@ int ADM_Composer::saveImageJpg(const char *filename)
 {
 	return A_saveJpg(filename);
 }
+
+int ADM_Composer::saveImagePng(const char *filename)
+{
+    return A_savePng(filename);
+}
 //----------------------------------AUDIO-------------------------
 /**
     \fn setAudioMixer
@@ -356,20 +361,20 @@ bool ADM_Composer::setAudioCodec(int dex,const char *codec, CONFcouple *c)
 /**
     \fn setAudioFilterNormalise
 */
-bool ADM_Composer::setAudioFilterNormalise(int dex,ADM_GAINMode mode, uint32_t gain)
+bool ADM_Composer::setAudioFilterNormalise(int dex, ADM_GAINMode mode, int32_t gain, int32_t maxlevel)
 {
     EditableAudioTrack *ed=getEditableAudioTrackAt(dex);
     if(!ed) return false;
-    return ed->audioEncodingConfig.audioFilterSetNormalize(mode,gain);
+    return ed->audioEncodingConfig.audioFilterSetNormalize(mode,gain,maxlevel);
 }
 /**
     \fn getAudioFilterNormalise
 */
-bool ADM_Composer::getAudioFilterNormalise(int dex,ADM_GAINMode *mode, uint32_t *gain)
+bool ADM_Composer::getAudioFilterNormalise(int dex, ADM_GAINMode *mode, int32_t *gain, int32_t *maxlevel)
 {
     EditableAudioTrack *ed=getEditableAudioTrackAt(dex);
     if(!ed) return false;
-    return ed->audioEncodingConfig.audioFilterGetNormalize(mode,gain);
+    return ed->audioEncodingConfig.audioFilterGetNormalize(mode,gain,maxlevel);
 }
 /**
     \fn getAudioFilterFrameRate
