@@ -18,6 +18,7 @@ protected:
             uint32_t currentFrame;
             ADMCompressedImage image;
             bool eofMet;
+            bool sanitizeDts;
             uint64_t  rescaleTs(uint64_t in);
             uint64_t  rewindTime;
 
@@ -44,6 +45,8 @@ protected:
         ADMBitstream    *myBitstream;
         uint8_t         *myExtra;
         uint32_t        myExtraLen;
+        bool            _init;
+        bool            h265;
         bool            compactNalus(ADMBitstream *out);
         int             convertFromAnnexB(uint8_t *inData,uint32_t inSize,
                                                       uint8_t *outData,uint32_t outMaxSize);
@@ -53,6 +56,10 @@ public:
         virtual         ~ADM_videoStreamCopyFromAnnexB();
         virtual bool    getPacket(ADMBitstream *out);
         virtual bool    getExtraData(uint32_t *extraLen, uint8_t **extraData) ;
+        bool            initOk() {return _init;}
+protected:
+        bool            extractExtraDataH264();
+        bool            extractExtraDataH265();
 };
 /**
         \fn ADM_videoStreamCopyToAnnexB
@@ -74,5 +81,23 @@ public:
         virtual         ~ADM_videoStreamCopyToAnnexB();
         virtual bool    getPacket(ADMBitstream *out);
         virtual bool    getExtraData(uint32_t *extraLen, uint8_t **extraData) ;
+protected:
+        bool extractExtraDataH264();
+        bool extractExtraDataH265();
+};
+/**
+        \fn ADM_videoStreamCopyAudRemover
+        \brief  Remove AUD Nalu units
+*/
+class ADM_videoStreamCopyAudRemover : public ADM_videoStreamCopy
+{
+protected:
+        bool            h265;
+        int             carryover;
+        uint8_t         scratchpad[ADM_COPY_FROM_ANNEX_B_SIZE];
+public:
+                        ADM_videoStreamCopyAudRemover(uint64_t startTime,uint64_t endTime);
+        virtual         ~ADM_videoStreamCopyAudRemover();
+        virtual bool    getPacket(ADMBitstream *out);
 };
 #endif
