@@ -15,6 +15,7 @@
  ***************************************************************************/
 
 #include <math.h>
+#include <errno.h>
 #include "ADM_cpp.h"
 #include "ADM_assert.h"
 #include "avi_vars.h"
@@ -48,7 +49,6 @@ static void list_audio_languages(char *p);
 static void saveCB(char*name);
 static void loadCB(char *name);
 static int set_output_format(const char *str);
-static void set_reuse_2pass_log(char *p);
 static void setVar(char *in);
 extern void UI_closeGui();
 //_________________________________________________________________________
@@ -76,27 +76,25 @@ typedef struct
 #define avs_port_change "avisynth-port"
 AUTOMATON reaction_table[]=
 {
-    
     {"append",                 1, "append video",                                                              (one_arg_type)A_appendVideo},
-    {"audio-codec",            1, "set audio codec (MP2/MP3/AC3/NONE (WAV PCM)/TWOLAME/COPY)",                 (one_arg_type)call_audiocodec},
-    {avs_port_change,          1, "set avsproxy port accordingly",                                             (one_arg_type)A_set_avisynth_port},    
-    {"help",                   0, "print this",                                                                (one_arg_type)call_help},    
-    {"info",                   0, "show information about loaded video and audio streams",                     (one_arg_type)show_info},    
-    {"list-audio-languages",   0, "list all available audio langues",                                          (one_arg_type)list_audio_languages},        
+    {"audio-codec",            1, "set audio codec (copy|Lame|FDK_AAC|LavAC3|Opus|TwoLame|...)",               (one_arg_type)call_audiocodec},
+    {avs_port_change,          1, "set avsproxy port accordingly",                                             (one_arg_type)A_set_avisynth_port},
+    {"help",                   0, "print this",                                                                (one_arg_type)call_help},
+    {"info",                   0, "show information about loaded video and audio streams",                     (one_arg_type)show_info},
+    {"list-audio-languages",   0, "list all available audio langues",                                          (one_arg_type)list_audio_languages},
     {"load",                   1, "load video or workbench",                                                   (one_arg_type)loadCB},
     {"nogui",                  0, "Run in silent mode",                                                        (one_arg_type)GUI_Quiet},
-    {"output-format",          1, "set output format (AVI|OGM|ES|PS|AVI_DUAL|AVI_UNP|...)",                    (one_arg_type)set_output_format},
+    {"output-format",          1, "set output format (MKV|MP4|ffTS|ffPS|AVI|RAW|...)",                         (one_arg_type)set_output_format},
     {"quit",                   0, "exit avidemux",                                                             (one_arg_type)call_quit},
-    {"slave",                  1, "run as slave, master is on port arg",                                       (one_arg_type)call_slave},    
-    {"reuse-2pass-log",        0, "reuse 2pass logfile if it exists",                                          (one_arg_type)set_reuse_2pass_log},        
+    {"slave",                  1, "run as slave, master is on port arg",                                       (one_arg_type)call_slave},
     {"run",                    1, "load and run a script",                                                     (one_arg_type)call_scriptEngine},
     {"save",                   1, "save video",                                                                (one_arg_type)saveCB},
     {"save-jpg",               1, "save a jpeg",                                                               (one_arg_type)A_saveJpg},
     {"save-raw-audio",         1, "save audio as-is ",                                                         (one_arg_type)A_saveAudioCopy},
     {"save-uncompressed-audio",1, "save uncompressed audio",                                                   (one_arg_type)A_saveAudioProcessed},
-    {"set-audio-language",     2, "Set language of an active audio track {track_index} {language_short_name}", (one_arg_type)A_setAudioLang},    
-    {"var",                    1, "set var (--var myvar=3)",                                                   (one_arg_type)setVar},    
-    {"video-codec",            1, "set video codec (x264/...)",                                                (one_arg_type)call_videocodec},    
+    {"set-audio-language",     2, "Set language of an active audio track {track_index} {language_short_name}", (one_arg_type)A_setAudioLang},
+    {"var",                    1, "set var (--var myvar=3)",                                                   (one_arg_type)setVar},
+    {"video-codec",            1, "set video codec (Copy|x264|x265|xvid4|ffMpeg2|ffNvEnc|...)",                (one_arg_type)call_videocodec},
 };
 #define NB_AUTO (sizeof(reaction_table)/sizeof(AUTOMATON))
 
@@ -114,6 +112,7 @@ static int myargc;
 static int index;
 static three_arg_type three;
 static two_arg_type two;
+static bool portable;
 
     argc=global_argc;
     argv = global_argv;
@@ -144,20 +143,30 @@ static two_arg_type two;
     argc-=1;
     cur=1;
     myargc=argc;
+    portable=false;
     while(myargc>0)
     {
                 if(( *argv[cur]!='-') || (*(argv[cur]+1)!='-'))
                 {
-                      if(cur==1)
+                      if(cur==1 || (cur==2 && portable))
                       {
                           loadCB(argv[cur]);
                       }
                       else
                           printf("\n Found garbage %s\n",argv[cur]);
                       cur+=1;myargc-=1;
+                      portable=false;
                       continue;
                 }
                 // else it begins with --
+                if(!strcmp(argv[cur]+2,"portable")) // portable mode switch has been already taken care of, ignore
+                {
+                    portable=true;
+                    cur++;
+                    myargc--;
+                    continue;
+                }
+                portable=false;
                 index= searchReactionTable(argv[cur]+2);
                 if(index==-1) // not found
                 {
@@ -192,7 +201,6 @@ static two_arg_type two;
                     myargc-=1+reaction_table[index].have_arg;
                 }
     } // end while
-    GUI_Verbose();
     printf("\n ********** Automation ended***********\n");
     return 0; // Do not call me anymore
 }
@@ -217,27 +225,41 @@ int searchReactionTable(char *string)
  */
 void call_scriptEngine(const char *scriptFile)
 {
-    std::vector<IScriptEngine*> engines = getScriptEngines();
-    
-    std::string root,ext;
-    ADM_PathSplit(std::string(scriptFile),root,ext);
-  
-    if (engines.size() == 1)
+    char *fullpath=ADM_PathCanonize(scriptFile);
+    FILE *fd=ADM_fopen(fullpath,"r");
+    if(!fd)
     {
-            A_parseScript(engines[0], scriptFile);
-            if(avifileinfo)
-            {
-            A_Rewind();
-                UI_setMarkers(video_body->getMarkerAPts(),video_body->getMarkerBPts());
-            }
-            return;
+        if(errno == EACCES)
+        {
+            GUI_Error_HIG(QT_TRANSLATE_NOOP("adm", "Permission Error"), QT_TRANSLATE_NOOP("adm", "Cannot open script \"%s\"."), fullpath);
+        }
+        if(errno == ENOENT)
+        {
+            GUI_Error_HIG(QT_TRANSLATE_NOOP("adm", "File Error"), QT_TRANSLATE_NOOP("adm", "Script \"%s\" does not exist."), fullpath);
+        }
+        return;
     }
-    
+
+    std::vector<IScriptEngine*> engines = getScriptEngines();
+    std::string root,ext;
+    ADM_PathSplit(std::string(fullpath),root,ext);
+  
+    if(engines.size() == 1)
+    {
+        A_parseScript(engines[0],fullpath);
+        if(avifileinfo)
+        {
+            A_Rewind();
+            UI_setMarkers(video_body->getMarkerAPts(),video_body->getMarkerBPts());
+        }
+        return;
+    }
+
     for (int i = 0; i < engines.size(); i++)
     {
         if (!engines[i]->defaultFileExtension().compare(ext))
         {
-            A_parseScript(engines[i], scriptFile);
+            A_parseScript(engines[i],fullpath);
             A_Rewind();
             UI_setMarkers(video_body->getMarkerAPts(),video_body->getMarkerBPts());
             return;
@@ -470,15 +492,6 @@ uint8_t scriptAddVar(char *var,char *value)
     return 1;
 }
 
-/**
- * 
- * @param p
- */
-void set_reuse_2pass_log(char *p)
-{
-   prefs->set(FEATURES_REUSE_2PASS_LOG,true);
-}
-
 /**
  * 
  * @param name
