@@ -41,6 +41,7 @@ extern "C" {
 
 
 static bool         libvaWorking=true;
+static bool         libvaEncoderWorking=false;
 static admMutex     imageMutex;
 static int  ADM_LIBVAgetBuffer(AVCodecContext *avctx, AVFrame *pic);
 static void ADM_LIBVAreleaseBuffer(struct AVCodecContext *avctx, AVFrame *pic);
@@ -102,7 +103,10 @@ bool libvaUsable(void)
 {    
     bool v=true;
     if(!libvaWorking) return false;
-    if(!prefs->get(FEATURES_LIBVA,&v)) v=false;
+    if(!prefs->get(FEATURES_LIBVA,&v)) 
+    {
+        v=false;
+    }
     return v;
 }
 
@@ -127,8 +131,10 @@ static ADM_vaSurface *allocateADMVaSurface(AVCodecContext *ctx)
     \fn libvaProbe
     \brief Try loading vaapi...
 */
+extern bool ADM_initLibVAEncoder(void);
 bool libvaProbe(void)
 {
+    ADM_info("Probing for libVA support...\n");
     GUI_WindowInfo xinfo;
     void *draw;
     draw=UI_getDrawWidget();
@@ -137,10 +143,19 @@ bool libvaProbe(void)
     {
         GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Error"),QT_TRANSLATE_NOOP("adm","Core has been compiled without LIBVA support, but the application has been compiled with it.\nInstallation mismatch"));
         libvaWorking=false;
+        return false;
     }
 
     if(false==admLibVA::init(&xinfo)) return false;
     libvaWorking=true;
+    { // Only check encoder if decoder is working
+       libvaEncoderWorking = ADM_initLibVAEncoder();
+        
+    }
+    if(libvaEncoderWorking) 
+        ADM_info("LIBVA HW encoder is working\n");
+    else 
+        ADM_info("LIBVA HW encoder is *NOT* working\n");
     return true;
 }
 
@@ -332,6 +347,7 @@ static enum AVPixelFormat ADM_LIBVA_getFormat(struct AVCodecContext *avctx,  con
         return AV_PIX_FMT_NONE;
     }
     // Finish intialization of LIBVA decoder
+#if 0 // The lavc functions we rely on in ADM_acceleratedDecoderFF::parseHwAccel are no more
     const AVHWAccel *accel=ADM_acceleratedDecoderFF::parseHwAccel(outPix,id,AV_PIX_FMT_VAAPI_VLD);
     if(accel)
     {
@@ -340,6 +356,8 @@ static enum AVPixelFormat ADM_LIBVA_getFormat(struct AVCodecContext *avctx,  con
         return AV_PIX_FMT_VAAPI_VLD;
     }
     return AV_PIX_FMT_NONE;
+#endif
+    return AV_PIX_FMT_VAAPI_VLD;
 }
 }
 
@@ -496,16 +514,23 @@ bool decoderFFLIBVA::uncompress (ADMCompressedImage * in, ADMImage * out)
     
     AVFrame *frame=_parent->getFramePointer();
     ADM_assert(frame);
-    int ret = avcodec_decode_video2 (_context, frame, &got_picture, &pkt);
-    
-    if(ret<0)
+    if(_parent->getDrainingState())
     {
-        char er[2048]={0};
-        av_make_error_string(er, sizeof(er)-1, ret);
-        ADM_warning("Error %d in lavcodec (%s)\n",ret,er);
-        out->refType=ADM_HW_NONE;
-        return false;
+        if(_parent->getDrainingInitiated()==false)
+        {
+            avcodec_send_packet(_context, NULL);
+            _parent->setDrainingInitiated(true);
+        }
+    }else
+    {
+        avcodec_send_packet(_context, &pkt);
     }
+
+    int ret = avcodec_receive_frame(_context, frame);
+
+    if(!_parent->decodeErrorHandler(ret))
+        return false;
+
     if(frame->pict_type==AV_PICTURE_TYPE_NONE)
     {
         out->_noPicture=true;
@@ -599,6 +624,7 @@ bool           ADM_hwAccelEntryLibVA::canSupportThis(struct AVCodecContext *avct
                return false;
                break;
            }
+           break;
 #endif       
        case AV_CODEC_ID_VC1: profile= VAProfileVC1Advanced;break;
 #ifdef LIBVA_VP9_DEC
@@ -635,7 +661,7 @@ static ADM_hwAccelEntryLibVA libvaEntry;
  */
 bool initLIBVADecoder(void)
 {
-    ADM_info("Registering LIBVA hw decocer\n");
+    ADM_info("Registering LIBVA hw decoder\n");
     ADM_hwAccelManager::registerDecoder(&libvaEntry);
     return true;
 }
