@@ -52,6 +52,7 @@
 admMutex singleThread;
 
 float currentZoom=ZOOM_1_1;
+static int cutsNotOnIntraWarned;
 #include "DIA_audioTracks.h"
 //***********************************
 //******** A Function ***************
@@ -141,9 +142,6 @@ bool getScriptName(int action, int base,getName name,const char *ext,string &out
 }
 void HandleAction (Action action)
 {
-  uint32_t nf = 0;
-  uint32_t old;
-
   admScopedMutex autolock(&singleThread); // make sure only one thread at a time calls this
   
   ADM_warning("************ %s **************\n",getActionName(action));
@@ -259,9 +257,16 @@ void HandleAction (Action action)
                     prefs->save();
                     UI_updateRecentProjectMenu();
                     UI_updateRecentMenu(); // the order matters here
+#define LAST_SESSION_FILE ADM_getBaseDir()+std::string("lastEdit.py")
+                    std::string f=LAST_SESSION_FILE;
+                    if(ADM_fileExist(f.c_str()))
+                    {
+                        if(!ADM_eraseFile(f.c_str()))
+                            ADM_warning("Could not delete last editing state %s\n",f.c_str());
+                    }
                 }
+                return;
             }
-            return;
     case ACT_VIDEO_CODEC_CONFIGURE:
             videoEncoder6Configure();
             return;
@@ -303,7 +308,11 @@ void HandleAction (Action action)
     case ACT_AUDIO_FILTERS:
         {
             EditableAudioTrack *ed=video_body->getDefaultEditableAudioTrack();
-            if(ed) ed->audioEncodingConfig.audioFilterConfigure();
+            if(ed)
+            {
+                ed->audioEncodingConfig.audioFilterConfigure();
+                UI_setTimeShift(ed->audioEncodingConfig.shiftEnabled,ed->audioEncodingConfig.shiftInMs);
+            }
         }
       return;
     case ACT_PREFERENCES:
@@ -322,13 +331,21 @@ void HandleAction (Action action)
           if(playing)
           {
               ADM_info("Stopping playback...\n");
-              GUI_PlayAvi();
+              GUI_PlayAvi(true);
           }
-          ADM_info("Closing ui\n");
-      UI_closeGui();
-          
-          return;
-      break;
+        ADM_info("Closing ui\n");
+        UI_closeGui();
+        if(video_body && video_body->canUndo())
+        {
+            A_saveSession();
+            video_body->clearUndoQueue();
+        }
+        autolock.unlock();
+        return;
+    case ACT_RESTORE_SESSION:
+        if(playing) break;
+        A_checkSavedSession(true);
+        return;
     default:
       break;
 
@@ -497,7 +514,6 @@ void HandleAction (Action action)
         video_body->addToUndoQueue();
         A_ResetMarkers();
         A_Resync();
-        GUI_setCurrentFrameAndTime();
         break;
     }
     case ACT_Copy:
@@ -544,6 +560,48 @@ void HandleAction (Action action)
               {
                   video_body->pasteFromClipBoard(currentPts);
               }
+              ADM_cutPointType chk=ADM_EDITOR_CUT_POINT_IDR;
+              if(!UI_getCurrentVCodec())
+                  chk=video_body->checkCutsAreOnIntra();
+              if(cutsNotOnIntraWarned!=(int)chk && chk!=ADM_EDITOR_CUT_POINT_IDR)
+              {
+                  const char *alert;
+                  bool ask=true;
+                  switch(chk)
+                  {
+                      case ADM_EDITOR_CUT_POINT_NON_IDR:
+                          alert=QT_TRANSLATE_NOOP("adm","The cut points of the pasted video are not on keyframes.\n"
+                              "Video saved in copy mode will be corrupted at these points.\n"
+                              "Proceed anyway?");
+                          break;
+                      case ADM_EDITOR_CUT_POINT_RECOVERY:
+                          alert=QT_TRANSLATE_NOOP("adm","This video uses non-IDR recovery points instead of IDR as keyframes. "
+                              "Picture reordering information in the video stream is not reset at non-IDR frames. "
+                              "The cut points of the pasted selection may result in playback interruption "
+                              "due to reversed display order of frames if saved in copy mode.\n"
+                              "Proceed anyway?");
+                          break;
+                      case ADM_EDITOR_CUT_POINT_MISMATCH:
+                          alert=QT_TRANSLATE_NOOP("adm","Codec or codec settings across a cut point of the pasted video do not match.\n"
+                              "Playback of the video saved in copy mode may stop at this point.\n"
+                              "Proceed anyway?");
+                          break;
+                      case ADM_EDITOR_CUT_POINT_UNCHECKED:
+                          alert=QT_TRANSLATE_NOOP("adm","Cut points of the pasted video could not be checked. "
+                              "This indicates an issue with a source video, the state of editing or a bug in the program. "
+                              "Please check the application log file or console output for details.\n"
+                              "Try anyway?");
+                      default:
+                          ask=false; break;
+                  }
+                  if(ask && !GUI_Question(alert))
+                  {
+                      video_body->undo();
+                      cutsNotOnIntraWarned=-1;
+                      break;
+                  }
+                  cutsNotOnIntraWarned=(int)chk;
+              }
               video_body->getVideoInfo (avifileinfo);
               d=video_body->getVideoDuration()-d;
               if(markA>currentPts)
@@ -561,7 +619,6 @@ void HandleAction (Action action)
               GUI_GoToTime(currentPts);
             }
             break;
-      break;
 
     case ACT_Undo:
     case ACT_Redo:
@@ -597,6 +654,7 @@ void HandleAction (Action action)
             video_body->getVideoInfo (avifileinfo);
 
             A_Resync();
+            A_Rewind();
             A_ResetMarkers();
 
             // forget last project file
@@ -610,27 +668,47 @@ void HandleAction (Action action)
             uint64_t a=video_body->getMarkerAPts();
             uint64_t b=video_body->getMarkerBPts();
             uint64_t current=video_body->getCurrentFramePts();
+            uint64_t before=video_body->getVideoDuration();
             if(a>b)
             {
                 uint64_t p=a;
                 a=b;
                 b=p;
             }
+            // Special case of B being at or beyond the last frame
+            bool lastFrame=false;
+            if(b==before)
+            {
+                lastFrame=true;
+            }else
+            {
+                uint64_t pts=video_body->getLastKeyFramePts();
+                if(pts!=ADM_NO_PTS && b>=pts) // don't waste time if B is before the last keyframe
+                {
+                    GUI_infiniteForward(pts);
+                    uint64_t lastFramePts=video_body->getCurrentFramePts();
+                    if(b>=lastFramePts) lastFrame=true; // B is at or beyond the last frame
+                }
+            }
+            if(!a && lastFrame)
+            {
+                if(action==ACT_Cut)
+                {
+                    GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Cutting"),
+                                  QT_TRANSLATE_NOOP("adm","It is impossible to cut out the entire video. Please recheck the position of markers A and B."));
+                }else
+                {
+                    GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Deleting"),
+                                  QT_TRANSLATE_NOOP("adm","It is impossible to delete the entire video. Please recheck the position of markers A and B."));
+                }
+                break;
+            }
             if(action==ACT_Cut)
             {
                 video_body->copyToClipBoard(a,b);
             }
             video_body->addToUndoQueue();
-            // Special case of B being at or beyond the last frame
-            bool lastFrame=false;
             bool result=false;
-            uint64_t pts=video_body->getLastKeyFramePts();
-            if(pts!=ADM_NO_PTS && b>=pts) // don't waste time if B is before the last keyframe
-            {
-                GUI_infiniteForward(pts);
-                uint64_t lastFramePts=video_body->getCurrentFramePts();
-                if(b>=lastFramePts) lastFrame=true; // B is at or beyond the last frame
-            }
             if(lastFrame)
                 result= video_body->truncate(a);
             else
@@ -640,20 +718,97 @@ void HandleAction (Action action)
                 GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Cutting"),QT_TRANSLATE_NOOP("adm","Error while cutting out."));
                 break;
             }
+            ADM_cutPointType chk=ADM_EDITOR_CUT_POINT_IDR;
+            if(!lastFrame && !UI_getCurrentVCodec())
+                chk=video_body->checkCutIsOnIntra(a);
+            if(cutsNotOnIntraWarned!=(int)chk && chk!=ADM_EDITOR_CUT_POINT_IDR)
+            {
+                const char *alert;
+                bool ask=true;
+                switch(chk)
+                {
+                    case ADM_EDITOR_CUT_POINT_NON_IDR:
+                        if(action==ACT_Cut)
+                            alert=QT_TRANSLATE_NOOP("adm","The end point of the cut is not on a keyframe.\n"
+                                "Video saved in copy mode will be corrupted at this point.\n"
+                                "Proceed anyway?");
+                        else
+                            alert=QT_TRANSLATE_NOOP("adm","The end point of the deletion is not on a keyframe.\n"
+                                "Video saved in copy mode will be corrupted at this point.\n"
+                                "Proceed anyway?");
+                        break;
+                    case ADM_EDITOR_CUT_POINT_RECOVERY:
+                        if(action==ACT_Cut)
+                            alert=QT_TRANSLATE_NOOP("adm","This video uses non-IDR recovery points instead of IDR as keyframes. "
+                                "Picture reordering information in the video stream is not reset at non-IDR frames. "
+                                "The chosen start and end points of the cut may result in playback interruption "
+                                "due to reversed display order of frames if saved in copy mode.\n"
+                                "Proceed anyway?");
+                        else
+                            alert=QT_TRANSLATE_NOOP("adm","This video uses non-IDR recovery points instead of IDR as keyframes. "
+                                "Picture reordering information in the video stream is not reset at non-IDR frames. "
+                                "The chosen start and end points of the deletion may result in playback interruption "
+                                "due to reversed display order of frames if saved in copy mode.\n"
+                                "Proceed anyway?");
+                        break;
+                    case ADM_EDITOR_CUT_POINT_MISMATCH:
+                        if(action==ACT_Cut)
+                            alert=QT_TRANSLATE_NOOP("adm","Codec or codec settings across the cut do not match. "
+                                "Playback of the video saved in copy mode may stop at this point.\n"
+                                "Proceed anyway?");
+                        else
+                            alert=QT_TRANSLATE_NOOP("adm","Codec or codec settings across the deletion do not match. "
+                                "Playback of the video saved in copy mode may stop at this point.\n"
+                                "Proceed anyway?");
+                        break;
+                    case ADM_EDITOR_CUT_POINT_UNCHECKED:
+                        alert=QT_TRANSLATE_NOOP("adm","Cut points could not be checked.\n"
+                            "This indicates an issue with a source video, the state of editing or a bug in the program. "
+                            "Please check the application log file or console output for details.\n"
+                            "Proceed anyway?");
+                        break;
+                    default: ask=false; break;
+                }
+                if(ask && !GUI_Question(alert))
+                {
+                    video_body->undo();
+                    cutsNotOnIntraWarned=-1;
+                    break;
+                }
+                cutsNotOnIntraWarned=(int)chk;
+            }
             A_ResetMarkers();
             A_Resync(); // total duration & stuff
-        
+
+            // If A was at the first frame of a segment with start time in ref = 0 and ref video
+            // not starting at zero, the part of the segment before the first frame got deleted too,
+            // even if we didn't ask for that explicitely. We must adjust timestamps accordingly
+            // to avoid seek errors. This is not relevant for truncating.
+            uint64_t after=video_body->getVideoDuration();
+            uint64_t c=0;
+            if(!lastFrame)
+                c=before-after+a-b;
             if(current>=a) // else current is before A, so nothing to do
             {
                 if(current<b) // current is between A & B => A
                 {
-                        current=a;
+                        current=a-c;
                 }else // current is after the removed chunk, adjust
                 {
-                        current-=(b-a);
+                        current-=b-a+c;
                 }
-            } 
-            GUI_GoToTime(current);
+            }
+            if(!video_body->goToTimeVideo(current))
+            {
+                // If seek fails, we may crash in admPreview::samePicture()
+                // due to _currentSegment and _currentPts going out of sync.
+                // Rewind to get on firm ground again.
+                A_Rewind();
+            }else
+            {
+                admPreview::samePicture();
+                GUI_setCurrentFrameAndTime();
+            }
         }
     break;
       // set decoder option (post processing ...)
@@ -699,6 +854,7 @@ int A_openVideo (const char *name)
     char *longname;
     uint32_t magic[4];
     uint32_t id = 0;
+    cutsNotOnIntraWarned = false;
 
     if (playing)
         return 0;
@@ -736,19 +892,6 @@ int A_openVideo (const char *name)
     else
         res = video_body->addFile(longname);
 
-    //  DIA_StopBusy ();
-    int ac=0; // audio codec = copy
-    bool loadDefault;
-    if(!prefs->get(RESET_ENCODER_ON_VIDEO_LOAD, &loadDefault)) loadDefault=false;
-    // if true, discard changes in output config on video load
-    if(loadDefault)
-    {
-        A_loadDefaultSettings();
-        UI_setAudioCodec(ac); // revert to copy, we don't save default audio codec config yet
-    }else
-    {
-        ac=UI_getCurrentACodec();
-    }
     // forget last project file
     video_body->setProjectName("");
 
@@ -774,6 +917,7 @@ int A_openVideo (const char *name)
 
     {
         int i;
+#if 0
         FILE *fd = NULL;
         char magic[4];
 
@@ -781,7 +925,6 @@ int A_openVideo (const char *name)
          ** by video_body->addFile (name);
          */
         //#warning FIXME
-#if 0
         if ((fd = ADM_fopen(longname, "rb")))
         {
             if (fread(magic, 4, 1, fd) == 4)
@@ -799,8 +942,21 @@ int A_openVideo (const char *name)
         prefs->set_lastfile(longname);
         updateLoaded();
         UI_updateRecentMenu();
-        if(video_body->getNumberOfActiveAudioTracks())
-            audioCodecSetByIndex(0,ac); // try to preserve audio codec
+        int ac=0; // audio codec = copy
+        bool loadDefault;
+        if(!prefs->get(RESET_ENCODER_ON_VIDEO_LOAD, &loadDefault)) loadDefault=false;
+        // if true, discard changes in output config on video load
+        if(loadDefault)
+        {
+            UI_setAudioCodec(ac); // revert to copy, we don't save default audio codec config yet
+            A_loadDefaultSettings();
+        }else
+        {
+            ac=UI_getCurrentACodec();
+            if(video_body->getNumberOfActiveAudioTracks())
+                audioCodecSetByIndex(0,ac); // try to preserve audio codec
+        }
+
         if (currentaudiostream)
         {
             uint32_t nbAudio;
@@ -885,6 +1041,8 @@ void  updateLoaded (bool resetMarker)
 //___________________________________________
 int A_appendVideo (const char *name)
 {
+    if(playing)
+        return 0;
     bool markerChanged=false;
     // Check if A or B was changed
     uint64_t beginPts=0,beginDts;
@@ -909,9 +1067,8 @@ int A_appendVideo (const char *name)
                 markerChanged=true;
     }
 
-    if (playing)
-        return 0;
     video_body->addToUndoQueue();
+    uint64_t currentPts=admPreview::getCurrentPts();
     if (!video_body->addFile (name))
     {
       GUI_Error_HIG (QT_TRANSLATE_NOOP("adm","Something failed when appending"), NULL);
@@ -929,6 +1086,7 @@ int A_appendVideo (const char *name)
         video_body->setMarkerBPts(video_body->getVideoDuration() );
         ADM_info("Extending marker B to the end (%s)\n",ADM_us2plain(video_body->getMarkerBPts()));
     }  
+    admPreview::seekToTime(currentPts);
     ReSync();
     GUI_setCurrentFrameAndTime();
     UI_setMarkers (video_body->getMarkerAPts(),video_body->getMarkerBPts());
@@ -941,8 +1099,6 @@ int A_appendVideo (const char *name)
 
 void ReSync (void)
 {
-  uint8_t isaviaud;
-
   // update audio stream
   // If we were on avi , mark it...
   GUI_setAllFrameAndTime ();
@@ -992,6 +1148,7 @@ void cleanUp (void)
  * @return 
  */
 #define DEFAULT_SETTINGS_FILE ADM_getBaseDir()+std::string("defaultSettings.py")
+#define CRASH_RECOVERY_FILE ADM_getBaseDir()+std::string("crash.py")
 bool parseScript(IScriptEngine *engine, const char *name, IScriptEngine::RunMode mode)
 {
     bool ret;
@@ -1009,7 +1166,9 @@ bool parseScript(IScriptEngine *engine, const char *name, IScriptEngine::RunMode
         video_body->setProjectName(longname);
     }
 
-    if (std::string(longname) != DEFAULT_SETTINGS_FILE)
+    if (std::string(longname) != DEFAULT_SETTINGS_FILE &&
+        std::string(longname) != LAST_SESSION_FILE &&
+        std::string(longname) != CRASH_RECOVERY_FILE)
     {
         prefs->set_lastprojectfile(longname);
         UI_updateRecentProjectMenu();
@@ -1094,8 +1253,9 @@ void A_saveDefaultSettings()
     delete writer;
     
     std::string script = stream.str();
-    ADM_info("Generated settings=%s\n",script.c_str());
-    
+    ADM_info("Generated settings:\n");
+    printf("%s\n",script.c_str());
+
     FILE *file = ADM_fopen(fileName.c_str(), "wt");
     ADM_fwrite(script.c_str(), script.length(), 1, file);
     ADM_fclose(file);
@@ -1117,7 +1277,57 @@ bool A_loadDefaultSettings(void)
     }
     return false;
 }
-/*
+/**
+ * \fn A_saveSession
+ */
+bool A_saveSession(void)
+{
+    IScriptEngine *engine=getPythonScriptEngine();
+    if(!engine)
+        return false;
+    ADM_info("Saving the current state of editing\n");
+    std::string where=LAST_SESSION_FILE;
+    if(ADM_fileExist(where.c_str()))
+    {
+        std::string tmp=ADM_getBaseDir()+std::string("lastEdit.tmp");
+        A_saveScript(engine, tmp.c_str());
+        if(!ADM_eraseFile(where.c_str()))
+        {
+            ADM_warning("Could not delete the old saved session (%s)\n",where.c_str());
+            if(!ADM_eraseFile(tmp.c_str()))
+                ADM_warning("Could not delete temporary file (%s)\n",tmp.c_str());
+            return false;
+        }
+        return !!rename(tmp.c_str(), where.c_str());
+    }
+    A_saveScript(engine, where.c_str());
+    return true;
+}
+/**
+ * \fn A_checkSavedSession
+ */
+bool A_checkSavedSession(bool load)
+{
+    std::string where=LAST_SESSION_FILE;
+    bool exists=ADM_fileExist(where.c_str());
+    if(!load)
+        return exists;
+    bool r=false;
+    if(exists)
+    {
+        IScriptEngine *engine=getPythonScriptEngine();
+        if(engine)
+        {
+            ADM_info("Restoring the last editing state from %s\n",where.c_str());
+            r=A_parseScript(engine,where.c_str());
+            A_Resync();
+            if(!ADM_eraseFile(where.c_str()))
+                ADM_warning("Could not delete %s\n",where.c_str());
+        }
+    }
+    return r;
+}
+/**
     Unpack all frames without displaying them to check for error
 
 */
@@ -1485,22 +1695,27 @@ uint8_t GUI_close(void)
   if (avifileinfo)        // already opened ?
     {                // delete everything
       // if preview is on
-      admPreview::setMainDimension(0, 0, ZOOM_1_1);
       if(getPreviewMode()!=ADM_PREVIEW_NONE)
       {
         admPreview::stop();
         setPreviewMode(ADM_PREVIEW_NONE);
       }
+      admPreview::setMainDimension(0, 0, ZOOM_1_1); // destroy preview
       UI_setNeedsResizingFlag(false);
+      uint32_t zero[6]={0};
+      UI_setVUMeter(zero);
+      if(video_body->canUndo())
+        A_saveSession();
       delete avifileinfo;
       //delete wavinfo;
-      admPreview::destroy();
       avifileinfo = NULL;
       video_body->clearUndoQueue();
       video_body->cleanup ();
-
+      UI_setAudioTrackCount(0);
+      UI_setTimeShift(false,0);
 //      filterCleanUp ();
       UI_setTitle(NULL);
+      UI_setDecoderName("XXXX");
 
     A_ResetMarkers();
     ReSync();
@@ -1544,13 +1759,14 @@ void GUI_avsProxy(void)
 
 void GUI_showCurrentFrameHex(void)
 {
+#if 0
  uint8_t *buffer;
  uint32_t fullLen,flags;
  char sType[5];
  char sSize[15];
  ADMCompressedImage image;
  uint8_t seq;
-#if 0
+
  if (!avifileinfo) return;
 
  buffer=new uint8_t [avifileinfo->width*avifileinfo->height*3];
@@ -1584,6 +1800,7 @@ void GUI_showCurrentFrameHex(void)
 #define DUMP_SIZE 30
 void GUI_showSize(void)
 {
+#if 0
 uint8_t *buffer;
  uint32_t fullLen,flags;
  ADMCompressedImage image;
@@ -1591,7 +1808,7 @@ uint8_t *buffer;
  char                text[DUMP_SIZE][100];
 
  if (!avifileinfo) return;
-#if 0
+
  buffer=new uint8_t [avifileinfo->width*avifileinfo->height*3];
  image.data=buffer;
     for(int i=0;i<DUMP_SIZE;i++)
