@@ -27,7 +27,7 @@ extern const char *shortkey(const char *);
 
 namespace ADM_qt4Factory
 {
-class diaElemInteger : public diaElem
+class diaElemInteger : public diaElem,QtFactoryUtils
 {
 
 public:
@@ -40,7 +40,7 @@ public:
   int getRequiredLayout(void);
 };
 /* Same but unsigned */
-class diaElemUInteger : public diaElem
+class diaElemUInteger : public diaElem,QtFactoryUtils
 {
 
 public:
@@ -55,10 +55,9 @@ public:
 
 //********************************************************************
 diaElemInteger::diaElemInteger(int32_t *intValue,const char *toggleTitle, int32_t min, int32_t max,const char *tip)
-  : diaElem(ELEM_TOGGLE)
+  : diaElem(ELEM_TOGGLE),QtFactoryUtils(toggleTitle)
 {
   param=(void *)intValue;
-  paramTitle=shortkey(toggleTitle);
   this->min=min;
   this->max=max;
   this->tip=tip;
@@ -66,8 +65,6 @@ diaElemInteger::diaElemInteger(int32_t *intValue,const char *toggleTitle, int32_
 
 diaElemInteger::~diaElemInteger()
 {
-  if(paramTitle)
-    ADM_dealloc( paramTitle);
 }
 void diaElemInteger::setMe(void *dialog, void *opaque,uint32_t line)
 {
@@ -80,7 +77,7 @@ void diaElemInteger::setMe(void *dialog, void *opaque,uint32_t line)
  box->setMaximum(max);
  box->setValue(*(int32_t *)param);
  
- QLabel *text=new QLabel( QString::fromUtf8(this->paramTitle),(QWidget *)dialog);
+ QLabel *text=new QLabel( myQtTitle,(QWidget *)dialog);
  text->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
  text->setBuddy(box);
 
@@ -116,10 +113,9 @@ int diaElemInteger::getRequiredLayout(void) { return FAC_QT_GRIDLAYOUT; }
 
 //******************************************************
 diaElemUInteger::diaElemUInteger(uint32_t *intValue,const char *toggleTitle, uint32_t min, uint32_t max,const char *tip)
-  : diaElem(ELEM_TOGGLE)
+  : diaElem(ELEM_TOGGLE),QtFactoryUtils(toggleTitle)
 {
   param=(void *)intValue;
-  paramTitle=shortkey(toggleTitle);
   this->min=min;
   this->max=max;
   this->tip=tip;
@@ -128,8 +124,7 @@ diaElemUInteger::diaElemUInteger(uint32_t *intValue,const char *toggleTitle, uin
 
 diaElemUInteger::~diaElemUInteger()
 {
-  if(paramTitle)
-    ADM_dealloc( paramTitle);
+  
 }
 void diaElemUInteger::setMe(void *dialog, void *opaque,uint32_t line)
 {
@@ -142,7 +137,7 @@ void diaElemUInteger::setMe(void *dialog, void *opaque,uint32_t line)
  box->setMaximum(max);
  box->setValue(*(uint32_t *)param);
  
- QLabel *text=new QLabel( QString::fromUtf8(this->paramTitle),(QWidget *)dialog);
+ QLabel *text=new QLabel(myQtTitle,(QWidget *)dialog);
  text->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
  text->setBuddy(box);
 
