@@ -26,98 +26,142 @@
 #include "ADM_dialogFactoryQt4.h"
 
 static void insertTab(uint32_t index, diaElemTabs *tab, QTabWidget *wtab);
-
 /**
     \fn qt4DiaFactoryRun(const char *title,uint32_t nb,diaElem **elems)
     \brief  Run a dialog made of nb elems, each elem being described in the **elems
     @return 0 on failure, 1 on success
 */
 
-uint8_t qt4DiaFactoryRun(const char *title,uint32_t nb,diaElem **elems)
+
+
+/**
+    \fn qt4DiaFactoryRun(const char *title,uint32_t nb,diaElem **elems)
+    \brief  Run a dialog made of nb elems, each elem being described in the **elems
+    @return 0 on failure, 1 on success
+*/
+class factoryCookie
 {
-  QDialog dialog(qtLastRegisteredDialog());
-  qtRegisterDialog(&dialog);
-  
+public:
+        factoryCookie(const char *title)
+        {
+            dialog=new QDialog(qtLastRegisteredDialog());
+            dialog->setWindowTitle(QString::fromUtf8(title));
+            vboxlayout = new QVBoxLayout(dialog);
+            tabWidget=NULL;
+            layout=NULL;
+        }
+        virtual ~factoryCookie()
+        {
+            if(vboxlayout)
+                delete vboxlayout;
+            if(dialog)
+                delete dialog;
+            dialog=NULL;
+            vboxlayout=NULL;
+        }
+public:
+        QDialog     *dialog;
+        QVBoxLayout *vboxlayout;
+        QLayout     *layout;
+        QTabWidget  *tabWidget;        
+        std::vector <diaElem *>items;
+};
+/**
+ * 
+ * @param title
+ * @param nb
+ * @param elems
+ * @return 
+ */
+void * qt4DiaFactoryPrepare(const char *title,uint32_t nb,diaElem **elems)
+{
+  factoryCookie *cookie=new factoryCookie(title);
+
   ADM_assert(title);
   ADM_assert(nb);
   ADM_assert(elems);
   
-  dialog.setWindowTitle(QString::fromUtf8(title));
-  
+    int currentLayout = 0;
+    int  v=0;
+    for(int i=0;i<nb;i++)
+    {
+       ADM_assert(elems[i]);
+       if (elems[i]->getRequiredLayout() != currentLayout)
+       {
+           if (cookie->layout)
+               cookie->vboxlayout->addLayout(cookie->layout);
+
+           switch (elems[i]->getRequiredLayout())
+           {
+               case FAC_QT_GRIDLAYOUT:
+                   cookie->layout = new QGridLayout();
+                   break;
+               case FAC_QT_VBOXLAYOUT:
+                   cookie->layout = new QVBoxLayout();
+                   break;
+           }
+
+           currentLayout = elems[i]->getRequiredLayout();
+           v = 0;
+       }
+
+       elems[i]->setMe( (void *)(cookie->dialog),cookie->layout,v);
+       v+=elems[i]->getSize();
+    }
+
+    for(int i=0;i<nb;i++)
+    {
+        ADM_assert(elems[i]);
+        elems[i]->finalize(); 
+        cookie->items.push_back(elems[i]);
+    }
+    return cookie;
+}
+/**
+ * 
+ * @param finish
+ * @return 
+ */
+bool  qt4DiaFactoryFinish(void *finish)
+{
+  bool  r=false;
+  factoryCookie *cookie=(factoryCookie *)  finish;
   QSpacerItem *spacer = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Fixed);
   QDialogButtonBox *buttonBox = new QDialogButtonBox();
-  QVBoxLayout *vboxLayout = new QVBoxLayout();
-  QLayout *layout = NULL;
-  int currentLayout = 0;
-
- int  v=0;
-
- for(int i=0;i<nb;i++)
- {
-         ADM_assert(elems[i]);
-
-         if (elems[i]->getRequiredLayout() != currentLayout)
-         {
-                 if (layout)
-                         vboxLayout->addLayout(layout);
-
-                 switch (elems[i]->getRequiredLayout())
-                 {
-                         case FAC_QT_GRIDLAYOUT:
-                                 layout = new QGridLayout();
-                                 break;
-                         case FAC_QT_VBOXLAYOUT:
-                                 layout = new QVBoxLayout();
-                                 break;
-                 }
-
-                 currentLayout = elems[i]->getRequiredLayout();
-                 v = 0;
-         }
-
-         elems[i]->setMe( (void *)&dialog,layout,v);
-         v+=elems[i]->getSize();
- }
-
-   for(int i=0;i<nb;i++)
-  {
-    ADM_assert(elems[i]);
-     elems[i]->finalize(); 
-  }
-
+  
   // Add buttons
    buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 
-   QObject::connect(buttonBox, SIGNAL(accepted()), &dialog, SLOT(accept()));
-   QObject::connect(buttonBox, SIGNAL(rejected()), &dialog, SLOT(reject()));
+   QObject::connect(buttonBox, SIGNAL(accepted()), cookie->dialog, SLOT(accept()));
+   QObject::connect(buttonBox, SIGNAL(rejected()), cookie->dialog, SLOT(reject()));
 
-   if (layout)
-           vboxLayout->addLayout(layout);
+   if (cookie->layout)
+           cookie->vboxlayout->addLayout(cookie->layout);
 
-   vboxLayout->addItem(spacer);
-   vboxLayout->addWidget(buttonBox);
+   cookie->vboxlayout->addItem(spacer);
+   cookie->vboxlayout->addWidget(buttonBox);
 
-   dialog.setLayout(vboxLayout);
+   cookie->dialog->setLayout(cookie->vboxlayout);
 
-  if(dialog.exec()==QDialog::Accepted)
+   qtRegisterDialog(cookie->dialog);
+
+  if(cookie->dialog->exec()==QDialog::Accepted)
   {
+     int nb=cookie->items.size();
      for(int i=0;i<nb;i++)
      {
-        ADM_assert(elems[i]);
-        elems[i]->getMe(); 
-    
+        ADM_assert(cookie->items[i]);
+        cookie->items[i]->getMe();     
       }
-
-         qtUnregisterDialog(&dialog);
-
-    return 1;
+     r=true;
   }
 
-  qtUnregisterDialog(&dialog);
+  qtUnregisterDialog(cookie->dialog);
 
-  return 0;
-  
+  delete cookie;
+  return r;
 }
+
 /**
     \fn shortkey(const char *in)
     \brief translate gtk like accelerator (_) to QT4 like accelerator (&)
@@ -132,74 +176,72 @@ const char *shortkey(const char *in)
         return ADM_strdup(escaped.toUtf8().constData());
 }
 
+
 /**
  *         \fn qt4DiaFactoryRunTabs
  */
-uint8_t qt4DiaFactoryRunTabs(const char *title,uint32_t nb,diaElemTabs **tabs)
+void  *qt4DiaFactoryTabsPrepare(const char *title,uint32_t nb,diaElemTabs **tabs)
 {
-    QDialog dialog(qtLastRegisteredDialog());
-
-        qtRegisterDialog(&dialog);
+    factoryCookie *cookie=new factoryCookie(title);
   
-  ADM_assert(title);
-  ADM_assert(nb);
-  ADM_assert(tabs);
+    ADM_assert(title);
+    ADM_assert(nb);
+    ADM_assert(tabs);
   
-  dialog.setWindowTitle(QString::fromUtf8(title));
-
-  QVBoxLayout *vboxLayout = new QVBoxLayout();
-  QGridLayout *layout = new QGridLayout();
-  QSpacerItem *spacer = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Fixed);
-  QTabWidget *wtabs = new QTabWidget();
-  QDialogButtonBox *buttonBox = new QDialogButtonBox();
-
-    buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-
-     QObject::connect(buttonBox, SIGNAL(accepted()), &dialog, SLOT(accept()));
-     QObject::connect(buttonBox, SIGNAL(rejected()), &dialog, SLOT(reject()));
-
-     for(int i=0;i<nb;i++)
-     {
-        ADM_assert(tabs[i]);
-        insertTab(i,tabs[i],wtabs);
-      }
-
-         vboxLayout->addLayout(layout);
-     vboxLayout->addWidget(wtabs,0,0);
-         vboxLayout->addItem(spacer);
-     vboxLayout->addWidget(buttonBox,1,0);
-
-         dialog.setLayout(vboxLayout);
-
-         // Expand to see all tabs but still allow the window to be resized smaller
-         wtabs->setUsesScrollButtons(false);
-         dialog.adjustSize();
-         wtabs->setUsesScrollButtons(true);
-
-  if(dialog.exec()==QDialog::Accepted)
-  {
-      // Read tabs
-       for(int tab=0;tab<nb;tab++)
-     {
-        ADM_assert(tabs[tab]);
-        diaElemTabs *myTab=tabs[tab];
-        for(int i=0;i<myTab->nbElems;i++)
-        {
-          myTab->dias[i]->getMe();
-        }
+    cookie->layout  = new QGridLayout();
+    cookie->tabWidget = new QTabWidget();
     
-      }
-           qtUnregisterDialog(&dialog);
 
-      return 1;
-  }
-
-  qtUnregisterDialog(&dialog);
-
-  return 0;
-  
+    for(int i=0;i<nb;i++)
+    {
+        ADM_assert(tabs[i]);
+        insertTab(i,tabs[i],cookie->tabWidget);
+         for(int j=0;j<tabs[i]->nbElems;j++)
+             cookie->items.push_back(tabs[i]->dias[j]);
+    }        
+    return cookie;
 }
+ /**
+  * 
+  * @param f
+  * @return 
+  */ 
+bool qt4DiaFactoryTabsFinish(void *f)
+{
+    bool r=false;
+    factoryCookie *cookie=(factoryCookie *)f;
+    
+    QDialogButtonBox *buttonBox = new QDialogButtonBox(); 
+    QObject::connect(buttonBox, SIGNAL(accepted()), cookie->dialog, SLOT(accept()));
+    QObject::connect(buttonBox, SIGNAL(rejected()), cookie->dialog, SLOT(reject()));    
+    buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+    QSpacerItem *spacer = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Fixed);
+    cookie->vboxlayout->addLayout(cookie->layout);
+    cookie->vboxlayout->addWidget(cookie->tabWidget,0,0);
+    cookie->vboxlayout->addItem(spacer);
+    cookie->vboxlayout->addWidget(buttonBox,1,0);
 
+     cookie->dialog->setLayout(cookie->vboxlayout);
+
+     // Expand to see all tabs but still allow the window to be resized smaller
+     cookie->tabWidget->setUsesScrollButtons(false);
+     cookie->dialog->adjustSize();
+     cookie->tabWidget->setUsesScrollButtons(true);
+
+    qtRegisterDialog(cookie->dialog);
+
+    if(cookie->dialog->exec()==QDialog::Accepted)
+    {
+        // Read tabs
+        int n=cookie->items.size();
+        for(int i=0;i<n;i++)
+            cookie->items[i]->getMe();
+        r=true;
+    }
+    qtUnregisterDialog(cookie->dialog);
+    delete cookie;
+    return r;
+}
 void insertTab(uint32_t index, diaElemTabs *tab, QTabWidget *wtab)
 {
 
@@ -217,34 +259,34 @@ void insertTab(uint32_t index, diaElemTabs *tab, QTabWidget *wtab)
      vsize+=tab->dias[i]->getSize(); 
   }
 
-        int  v=0;
+    int  v=0;
 
-        for(int i=0;i<tab->nbElems;i++)
+    for(int i=0;i<tab->nbElems;i++)
+    {
+        ADM_assert(tab->dias[i]);
+
+        if (tab->dias[i]->getRequiredLayout() != currentLayout)
         {
-                ADM_assert(tab->dias[i]);
+            if (layout)
+                vboxLayout->addLayout(layout);
 
-                if (tab->dias[i]->getRequiredLayout() != currentLayout)
-                {
-                        if (layout)
-                                vboxLayout->addLayout(layout);
+            switch (tab->dias[i]->getRequiredLayout())
+            {
+                case FAC_QT_GRIDLAYOUT:
+                    layout = new QGridLayout();
+                    break;
+                case FAC_QT_VBOXLAYOUT:
+                    layout = new QVBoxLayout();
+                    break;
+            }
 
-                        switch (tab->dias[i]->getRequiredLayout())
-                        {
-                                case FAC_QT_GRIDLAYOUT:
-                                        layout = new QGridLayout();
-                                        break;
-                                case FAC_QT_VBOXLAYOUT:
-                                        layout = new QVBoxLayout();
-                                        break;
-                        }
-
-                        currentLayout = tab->dias[i]->getRequiredLayout();
-                        v = 0;
-                }
-
-                tab->dias[i]->setMe( wid,layout,v); 
-                v+=tab->dias[i]->getSize();
+            currentLayout = tab->dias[i]->getRequiredLayout();
+            v = 0;
         }
+
+        tab->dias[i]->setMe( wid,layout,v); 
+        v+=tab->dias[i]->getSize();
+    }
   
   wtab->addTab(wid,QString::fromUtf8(tab->title));
   for(int i=0;i<tab->nbElems;i++)
@@ -253,11 +295,31 @@ void insertTab(uint32_t index, diaElemTabs *tab, QTabWidget *wtab)
   }
 
   if (layout)
-          vboxLayout->addLayout(layout);
+        vboxLayout->addLayout(layout);
 
   vboxLayout->addItem(spacerItem);
 }
+/**
+ * 
+ * @param title
+ * @param nb
+ * @param elems
+ * @return 
+ */
+uint8_t qt4DiaFactoryRun(const char *title,uint32_t nb,diaElem **elems)
+{
+    void *cookie=qt4DiaFactoryPrepare(title,nb,elems);
+    return qt4DiaFactoryFinish(cookie);
+}
 
+/**
+ *         \fn qt4DiaFactoryRunTabs
+ */
+uint8_t qt4DiaFactoryRunTabs(const char *title,uint32_t nb,diaElemTabs **tabs)
+{
+    void *cookie=qt4DiaFactoryTabsPrepare(title,nb,tabs);
+    return qt4DiaFactoryTabsFinish(cookie);
+}
 /**
  * 
  */
@@ -306,8 +368,8 @@ extern DELETE_DIA_ELEM_T   qt4DestroyMenu;
 extern DELETE_DIA_ELEM_T   qt4DestroyMenuDynamic;
 extern CREATE_USLIDER_T    qt4CreateUSlider;
 extern DELETE_DIA_ELEM_T   qt4DestroyUSlider;
-extern         CREATE_SLIDER_T     qt4CreateSlider;
-extern         DELETE_DIA_ELEM_T   qt4DestroySlider;
+extern CREATE_SLIDER_T     qt4CreateSlider;
+extern DELETE_DIA_ELEM_T   qt4DestroySlider;
 extern CREATE_THREADCOUNT_T qt4CreateThreadCount;
 extern DELETE_DIA_ELEM_T    qt4DestroyThreadCount;
 extern CREATE_TOGGLE_UINT   qt4CreateToggleUint;
@@ -324,7 +386,12 @@ static FactoryDescriptor Qt4FactoryDescriptor=
 {
         &qt4FactoryGetVersion,
         &qt4DiaFactoryRun,
-        &qt4DiaFactoryRunTabs,
+        &qt4DiaFactoryRunTabs,        
+        &qt4DiaFactoryTabsPrepare,
+        &qt4DiaFactoryTabsFinish,
+        &qt4DiaFactoryPrepare,
+        &qt4DiaFactoryFinish,
+
         // Buttons
         &qt4CreateButton,
         &qt4DestroyButton,
