@@ -17,7 +17,11 @@
 #include "ADM_coreJobs.h"
 #include "DIA_coreToolkit.h"
 #include <QtCore/QDir>
+#include <QAction>
 
+extern void loadTranslator(void);
+extern void initTranslator(void);
+extern void destroyTranslator(void);
 
 static QTableWidgetItem *fromText(const string &t,int id)
 {
@@ -164,13 +168,19 @@ jobWindow::jobWindow(void) : QDialog()
    connect(ui.pushButtonRunAll,SIGNAL(pressed()),this,SLOT(runAllJob()));
    connect(ui.pushButtonCleanup,SIGNAL(pressed()),this,SLOT(cleanup()));
 
+    QAction *bye = new QAction(QString("Quit"),this);
+    bye->setShortcut(Qt::Key_Q | Qt::CTRL);
+    connect(bye,SIGNAL(triggered()),this,SLOT(quit()));
+    this->addAction(bye);
+
     // Start our socket
-   if(false==mySocket.createBindAndAccept(&localPort))
+    localPort=0;
+    if(false==mySocket.createBindAndAccept(&localPort))
     {
         popup("Cannot bind socket");
         exit(-1);
     }
-    ADM_info("Socket bound to %d\n",(int)localPort);
+    ADM_info("Socket bound to %" PRIu32"\n",localPort);
     //ADM_socket *n=mySocket.waitForConnect(5000);
     refreshList();
     dialog=NULL;
@@ -196,13 +206,17 @@ bool        jobWindow::popup(const char *errorMessage)
 */
 bool jobRun(int ac,char **av)
 {
-
+    initTranslator();
+#if defined(_WIN32) && QT_VERSION >= QT_VERSION_CHECK(5,11,0)
+    QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
     QApplication *app=new QApplication(ac,av,0);
     Q_INIT_RESOURCE(jobs);
+    loadTranslator();
     jobWindow *jWindow=new jobWindow();
 
-
     jWindow->exec();
+    destroyTranslator();
     delete jWindow;
     delete app;
     return true;
