@@ -16,6 +16,8 @@
 
 #include "Q_props.h"
 #include "avi_vars.h"
+#include "ADM_edAudioTrackExternal.h"
+#include "ADM_edAudioTrackFromVideo.h"
 #include "ADM_vidMisc.h"
 #include "ADM_toolkitQt.h"
 #include "ADM_coreUtils.h"
@@ -36,11 +38,11 @@ propWindow::propWindow(QWidget *parent) : QDialog(parent)
     if (!avifileinfo)
         return;
 
-#define FILLTEXT(a,b,c) {snprintf(text,79,b,c); listOfValues.push_back(text); ui.a->setText(text);}
-#define FILLTEXT4(a,b,c,d) {snprintf(text,79,b,c,d); listOfValues.push_back(text); ui.a->setText(text);}
-#define FILLTEXT5(a,b,c,d,e) {snprintf(text,79,b,c,d,e); listOfValues.push_back(text); ui.a->setText(text);}
-#define SET_YES(a,b) ui.a->setText(yesno[b])
-#define FILLQT_TRANSLATE_NOOP(a,q) listOfValues.push_back(text); ui.q->setText(text);
+#define FILLTEXT(a,b,c) {snprintf(text,79,b,c); listOfValues.push_back(QString::fromUtf8(text)); ui.a->setText(QString::fromUtf8(text));}
+#define FILLTEXT4(a,b,c,d) {snprintf(text,79,b,c,d); listOfValues.push_back(QString::fromUtf8(text)); ui.a->setText(QString::fromUtf8(text));}
+#define FILLTEXT5(a,b,c,d,e) {snprintf(text,79,b,c,d,e); listOfValues.push_back(QString::fromUtf8(text)); ui.a->setText(QString::fromUtf8(text));}
+#define SET_YES(a,b) ui.a->setText(QString::fromUtf8(yesno[b])
+#define FILLQT_TRANSLATE_NOOP(a,q) listOfValues.push_back(QString::fromUtf8(text)); ui.q->setText(QString::fromUtf8(text));
 
     //------------------------------------
 
@@ -93,7 +95,30 @@ propWindow::propWindow(QWidget *parent) : QDialog(parent)
         sprintf(text, "%s", getStrFromAudioCodec(wavinfo->encoding));
         FILLQT_TRANSLATE_NOOP("qprops",labelACodec);
 
-        switch (wavinfo->channels)
+        uint32_t channels=wavinfo->channels;
+        uint32_t frequency=wavinfo->frequency;
+        duration=0;
+        EditableAudioTrack *ed=video_body->getDefaultEditableAudioTrack();
+        if(ed)
+        {
+            ADM_EDAUDIO_TRACK_TYPE type=ed->edTrack->getTrackType();
+            switch (type)
+            {
+                case ADM_EDAUDIO_FROM_VIDEO:
+                    duration=ed->edTrack->castToTrackFromVideo()->getDurationInUs();
+                    channels=ed->edTrack->castToTrackFromVideo()->getOutputChannels();
+                    frequency=ed->edTrack->castToTrackFromVideo()->getOutputFrequency();
+                    break;
+                case ADM_EDAUDIO_EXTERNAL:
+                    duration=ed->edTrack->castToExternal()->getDurationInUs();
+                    channels=ed->edTrack->castToExternal()->getOutputChannels();
+                    frequency=ed->edTrack->castToExternal()->getOutputFrequency();
+                    break;
+                default:break;
+            }
+        }
+
+        switch (channels)
         {
             case 1:
                 sprintf(text,"%s", QT_TRANSLATE_NOOP("qprops","Mono"));
@@ -102,7 +127,7 @@ propWindow::propWindow(QWidget *parent) : QDialog(parent)
                 sprintf(text,"%s", QT_TRANSLATE_NOOP("qprops","Stereo"));
                 break;
             default:
-                sprintf(text, "%d",wavinfo->channels);
+                sprintf(text, "%d",channels);
                 break;
         }
 
@@ -112,12 +137,12 @@ propWindow::propWindow(QWidget *parent) : QDialog(parent)
 
         FILLTEXT(labelVBR,"%s","n/a");
 
-        FILLTEXT(labelFrequency, QT_TRANSLATE_NOOP("qprops","%" PRIu32" Hz"), wavinfo->frequency);
+        FILLTEXT(labelFrequency, QT_TRANSLATE_NOOP("qprops","%" PRIu32" Hz"), frequency);
 
-        duration=st->getDurationInUs();
         ms2time(duration/1000,&hh,&mm,&ss,&ms);
         sprintf(text, QT_TRANSLATE_NOOP("qprops","%02d:%02d:%02d.%03d"), hh, mm, ss, ms);
         FILLQT_TRANSLATE_NOOP("qprops",labelAudioDuration);
+        ui.labelAudioDuration->setEnabled(!!duration);
 
 //                SET_YES(labelVBR,currentaudiostream->isVBR());
     } else
@@ -129,9 +154,9 @@ propWindow::propWindow(QWidget *parent) : QDialog(parent)
 }
 
 #define ADDCATEGORY(a) props += QString("\n=====================================================\n")\
-                               +QString(a)\
+                               +QString::fromUtf8(a)\
                                +QString("\n=====================================================\n");
-#define ADDNAMEVALUE(a,b) props += QString(a)+QString("\t")+QString(b)+QString("\n");
+#define ADDNAMEVALUE(a,b) props += QString::fromUtf8(a)+QString("\t")+QString(b)+QString("\n");
 
 /**
     \fn propsCopyToClipboard
