@@ -26,9 +26,14 @@ workWindow::workWindow(QWidget *parent) : QDialog(parent)
      ui->setupUi(this);
      active=true;
      setWindowModality(Qt::ApplicationModal);
-     connect( ui->buttonCancel,SIGNAL(clicked(bool)),this,SLOT(stop(bool)));
+#if QT_VERSION < QT_VERSION_CHECK(5,0,0)
+     connect(ui->buttonBox,SIGNAL(rejected()),this,SLOT(reject()));
+#else
+     connect(ui->buttonBox,&QDialogButtonBox::rejected,this,&QDialog::reject);
+#endif
  }
- void workWindow::stop(bool a) 
+
+void workWindow::reject(void)
 {
     ADM_info("Stop Request\n");
     active=false;
