@@ -77,6 +77,7 @@ extern const char *encoderGetIndexedName (uint32_t i);
 uint32_t audioEncoderGetNumberOfEncoders(void);
 const char  *audioEncoderGetDisplayName(int i);
 extern void checkCrashFile(void);
+extern bool A_checkSavedSession(bool load);
 extern void UI_QT4VideoWidget(QFrame *frame);
 extern void loadTranslator(void);
 extern void initTranslator(void);
@@ -114,7 +115,7 @@ extern void ADM_ExitCleanup(void);
 static bool uiRunning=false;
 static bool uiIsMaximized=false;
 
-bool needsResizing=false;
+static bool needsResizing=false;
 
 #define WIDGET(x)  (((MainWindow *)QuiMainWindows)->ui.x)
 
@@ -255,9 +256,11 @@ void MainWindow::sliderReleased(void)
  * \fn sliderPressed
  */
 void MainWindow::sliderPressed(void)
-{ 
-  dragTimer.stop();
-  dragState=dragState_Active;
+{
+    if(playing)
+        sendAction(ACT_PlayAvi); // stop playback
+    dragTimer.stop();
+    dragState=dragState_Active;
 //  ADM_info("Pressed\n");
 }
 /**
@@ -304,8 +307,15 @@ void MainWindow::audioToggled(bool checked)
         AVDM_setVolume(0);
 }
 
-void MainWindow::previewModeChanged(int  flop)
+void MainWindow::previewModeChangedFromMenu(bool flop)
 {
+    ui.toolBar->actions().at(5)->setChecked(flop);
+    sendAction(ACT_PreviewChanged);
+}
+
+void MainWindow::previewModeChangedFromToolbar(bool flop)
+{
+    ui.menuVideo->actions().at(3)->setChecked(flop);
     sendAction(ACT_PreviewChanged);
 }
 
@@ -340,6 +350,17 @@ void MainWindow::currentTimeToClipboard(void)
     clipboard->setText(ct);
 }
 
+/**
+    \fn setRefreshCap
+*/
+void MainWindow::setRefreshCap(void)
+{
+    refreshCapEnabled=false;
+    refreshCapValue=0;
+    prefs->get(FEATURES_CAP_REFRESH_ENABLED,&refreshCapEnabled);
+    prefs->get(FEATURES_CAP_REFRESH_VALUE,&refreshCapValue);
+}
+
 /**
     \fn ctor
 */
@@ -349,9 +370,6 @@ MainWindow::MainWindow(const vector<IScriptEngine*>& scriptEngines) : _scriptEng
     qtRegisterDialog(this);
     ui.setupUi(this);
     dragState=dragState_Normal;
-    refreshCapEnabled=false;
-    prefs->get(FEATURES_CAP_REFRESH_ENABLED,&refreshCapEnabled);
-    prefs->get(FEATURES_CAP_REFRESH_VALUE,&refreshCapValue);
     recentFiles = NULL;
     recentProjects = NULL;
 
@@ -363,8 +381,6 @@ MainWindow::MainWindow(const vector<IScriptEngine*>& scriptEngines) : _scriptEng
         //
         connect( this,SIGNAL(actionSignal(Action )),this,SLOT(actionSlot(Action )));
         //
-        connect( ui.checkDisplayOut,SIGNAL(stateChanged(int)),this,SLOT(previewModeChanged(int)));
-        //
         connect(this, SIGNAL(updateAvailable(int,std::string,std::string)),this,SLOT(updateAvailableSlot(int,std::string,std::string)));
 
     /*
@@ -383,7 +399,7 @@ MainWindow::MainWindow(const vector<IScriptEngine*>& scriptEngines) : _scriptEng
         ADM_mwNavSlider *qslider=(ADM_mwNavSlider *)slider;
     slider->setMinimum(0);
     slider->setMaximum(ADM_LARGE_SCALE);
-#if 1
+#if !(defined(__APPLE__) && QT_VERSION >= QT_VERSION_CHECK(5,10,0))
         slider->setTickInterval(ADM_SCALE_INCREMENT);
         slider->setTickPosition(QSlider::TicksBothSides);
 #endif
@@ -448,10 +464,21 @@ MainWindow::MainWindow(const vector<IScriptEngine*>& scriptEngines) : _scriptEng
     connect(this->recentFiles, SIGNAL(triggered(QAction*)), this, SLOT(searchRecentFiles(QAction*)));
     connect(this->recentProjects, SIGNAL(triggered(QAction*)), this, SLOT(searchRecentProjects(QAction*)));
 
+    addSessionRestoreToRecentMenu(myMenuRecent);
+
     buildMyMenu();
     buildCustomMenu(); // action lists are populated (i.e. buildActionLists() called) within buildCustomMenu()
     buildButtonLists();
+
+#define AUTOREPEAT_TOOLBUTTON(x) ui.x->setAutoRepeat(true); ui.x->setAutoRepeatDelay(500); ui.x->setAutoRepeatInterval(100);
+    AUTOREPEAT_TOOLBUTTON(toolButtonPreviousFrame)
+    AUTOREPEAT_TOOLBUTTON(toolButtonNextFrame)
+    AUTOREPEAT_TOOLBUTTON(toolButtonPreviousIntraFrame)
+    AUTOREPEAT_TOOLBUTTON(toolButtonNextIntraFrame)
+
     // Crash in some cases addScriptReferencesToHelpMenu();
+    connect(ui.menuVideo->actions().at(3),SIGNAL(toggled(bool)),this,SLOT(previewModeChangedFromMenu(bool)));
+    connect(ui.toolBar->actions().at(5),SIGNAL(toggled(bool)),this,SLOT(previewModeChangedFromToolbar(bool)));
 
     this->installEventFilter(this);
     slider->installEventFilter(this);
@@ -461,7 +488,11 @@ MainWindow::MainWindow(const vector<IScriptEngine*>& scriptEngines) : _scriptEng
     this->setFocus(Qt::OtherFocusReason);
 
     setAcceptDrops(true);
-        setWindowIcon(QIcon(MKICON(avidemux_icon_small)));
+#ifndef __APPLE__
+    setWindowIcon(QIcon(MKICON(avidemux-icon)));
+#else
+    setWindowIcon(QIcon(MKOSXICON(avidemux)));
+#endif
 
     // Hook also the toolbar
     connect(ui.toolBar,  SIGNAL(actionTriggered ( QAction *)),this,SLOT(searchToolBar(QAction *)));
@@ -485,7 +516,7 @@ MainWindow::MainWindow(const vector<IScriptEngine*>& scriptEngines) : _scriptEng
 
     threshold = RESIZE_THRESHOLD;
     actZoomCalled = false;
-    ignoreResizeEvent = true;
+    ignoreResizeEvent = false;
     blockResizing = false;
     blockZoomChanges = true;
 
@@ -507,6 +538,7 @@ toolBarTranslate toolbar[]=
 {"actionProperties",        ACT_VIDEO_PROPERTIES},
 {"actionLoad_run_project",  ACT_RUN_SCRIPT},
 {"actionSave_project",      ACT_SAVE_PY_SCRIPT},
+{"actionPlayFiltered",      ACT_PreviewChanged},
 
 {NULL,ACT_DUMMY}
 };
@@ -543,7 +575,11 @@ bool MainWindow::buildMenu(QMenu *root,MenuEntry *menu, int nb)
     for(int i=0;i<nb;i++)
     {
         MenuEntry *m=menu+i;
-        QString qs=QString::fromUtf8(QT_TRANSLATE_NOOP("adm",m->text.c_str()));
+        QString qs;
+        if(m->translated)
+            qs=QString::fromUtf8(m->text.c_str());
+        else
+            qs=QString::fromUtf8(QT_TRANSLATE_NOOP("adm",m->text.c_str()));
         switch(m->type)
         {
             case MENU_SEPARATOR:
@@ -671,6 +707,8 @@ bool MainWindow::buildMyMenu(void)
     connect( ui.menuView,SIGNAL(triggered(QAction*)),this,SLOT(searchViewMenu(QAction*)));
     buildMenu(ui.menuView, &myMenuView[0], myMenuView.size());
 
+    ui.menuVideo->actions().at(3)->setCheckable(true);
+
     return true;
 }
 
@@ -686,13 +724,13 @@ void MainWindow::buildActionLists(void)
     // Make a list of the items that are enabled/disabled depending if video is loaded or  not
     //-----------------------------------------------------------------------------------
     for(int i=1;i<6;i++)
-        ActionsAvailableWhenFileLoaded.push_back(ui.menuFile->actions().at(i));        
+        ActionsAvailableWhenFileLoaded.push_back(ui.menuFile->actions().at(i));
 
-    ActionsAvailableWhenFileLoaded.push_back(ui.menuFile->actions().at(9)); // "Information"
+    ActionsAvailableWhenFileLoaded.push_back(ui.menuFile->actions().at(this->_scriptEngines.size()*2+7)); // "Information"
 
     for(int i=3;i<11;i++)
     {
-        if(i==5 || i==7 || i==11 || i==13) continue;
+        if(i==5 || i==7) continue;
         ActionsAvailableWhenFileLoaded.push_back(ui.menuEdit->actions().at(i));
     }
 
@@ -701,8 +739,6 @@ void MainWindow::buildActionLists(void)
         ActionsAvailableWhenFileLoaded.push_back(ui.menuView->actions().at(i));
     }
 
-    ActionsAvailableWhenFileLoaded.push_back(ui.menuVideo->actions().at(1)); // post-processing
-
     bool canSave=!!ADM_mx_getNbMuxers();
     for(int i=3-canSave;i<ui.toolBar->actions().size();i++)
     { // disable "Save" and "Information" buttons in the toolbar if no video is loaded, "Save" also if we've got zero muxers
@@ -855,10 +891,14 @@ void MainWindow::setMenuItemsEnabledState(void)
         ui.checkBox_TimeShift->setEnabled(false);
         ui.spinBox_TimeValue->setEnabled(false);
 
+        if(getPreviewMode()!=ADM_PREVIEW_NONE)
+            slider->setEnabled(false);
+
         return;
     }
 
-    bool vid=false, undo=false, redo=false, paste=false;
+    bool vid, undo, redo, paste, restore;
+    vid=undo=redo=paste=restore=false;
     if(avifileinfo)
         vid=true; // a video is loaded
 
@@ -885,6 +925,7 @@ void MainWindow::setMenuItemsEnabledState(void)
     ui.menuEdit->actions().at(1)->setEnabled(redo); // menu item "Redo"
     if(!vid || (!undo && !redo)) // if no edits have been performed, disable "Reset Edit" menu item
     {
+        restore=A_checkSavedSession(false);
         ui.menuEdit->actions().at(2)->setEnabled(false);
     }else
     {
@@ -906,6 +947,17 @@ void MainWindow::setMenuItemsEnabledState(void)
         haveRecentItems=true;
     ui.menuRecent->actions().back()->setEnabled(haveRecentItems);
 
+    n=0;
+    if(recentFiles)
+        n++;
+    if(recentProjects)
+        n++;
+    if(ui.menuRecent->actions().size()>(n+2)) // "Restore session" entry exists
+        ui.menuRecent->actions().at(n+1)->setEnabled(restore);
+
+    ui.selectionDuration->setEnabled(vid);
+    slider->setEnabled(vid);
+
     updateCodecWidgetControlsState();
     // actions performed by the code above may result in a window resize event,
     // which in turn may initiate unwanted zoom changes e.g. when stopping playback
@@ -925,6 +977,10 @@ void MainWindow::updateCodecWidgetControlsState(void)
     ui.pushButtonDecoderConf->setEnabled(b);
     // take care of the "Decoder Options" item in the menu "Video"
     ui.menuVideo->actions().at(0)->setEnabled(b);
+    // post-processing is available only for software decoding and old codecs
+    if(b && !(isMpeg12Compatible(avifileinfo->fcc) || isMpeg4Compatible(avifileinfo->fcc)))
+        b=false;
+    ui.menuVideo->actions().at(1)->setEnabled(b);
 
     b=false;
     if(ui.comboBoxVideo->currentIndex())
@@ -935,10 +991,14 @@ void MainWindow::updateCodecWidgetControlsState(void)
         ui.pushButtonVideoFilter->setEnabled(b);
         // take care of the "Filter" item in the menu "Video" as well
         ui.menuVideo->actions().at(2)->setEnabled(b);
+        ui.menuVideo->actions().at(3)->setEnabled(b);
+        ui.toolBar->actions().at(5)->setEnabled(b);
     }else
     {
         ui.pushButtonVideoFilter->setEnabled(false);
         ui.menuVideo->actions().at(2)->setEnabled(false);
+        ui.menuVideo->actions().at(3)->setEnabled(false);
+        ui.toolBar->actions().at(5)->setEnabled(false);
     }
 
     b=false;
@@ -1261,9 +1321,9 @@ bool MainWindow::eventFilter(QObject* watched, QEvent* event)
                         }else
                         {
                             if(!swpud)
-                                sendAction(ACT_NextKFrame);
-                            else
                                 sendAction(ACT_PreviousKFrame);
+                            else
+                                sendAction(ACT_NextKFrame);
                         }
                         return true;
                     case Qt::Key_C:
@@ -1349,11 +1409,14 @@ bool MainWindow::eventFilter(QObject* watched, QEvent* event)
                         threshold = 0;
                 }
 
-                blockResizing = true;
-                admPreview::setMainDimension(w,h,zoom);
-                actZoomCalled = false;
-                admPreview::samePicture(); // required at least for VDPAU
-                blockResizing = false;
+                if(zoom > oldzoom + .001 || zoom < oldzoom - .001)
+                {
+                    blockResizing = true;
+                    admPreview::setMainDimension(w,h,zoom);
+                    actZoomCalled = false;
+                    admPreview::samePicture(); // required at least for VDPAU
+                    blockResizing = false;
+                }
             }
             if (watched == ui.sliderPlaceHolder)
             {
@@ -1416,6 +1479,9 @@ void MainWindow::changeEvent(QEvent *event)
             UI_resize(w,h);
             UI_setNeedsResizingFlag(false);
         }
+        // Always adjust zoom on maximize
+        if (ev->oldState() != Qt::WindowMaximized && QuiMainWindows->isMaximized())
+            ignoreResizeEvent = false;
     }
     QWidget::changeEvent(event);
 }
@@ -1582,17 +1648,31 @@ bool      UI_reset(void)
 */
 int UI_Init(int nargc, char **nargv)
 {
-        ADM_info("Starting QT4 GUI...\n");
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+    ADM_info("Starting Qt5 GUI...\n");
+#else
+    ADM_info("Starting Qt4 GUI...\n");
+#endif
     initTranslator();
 
     global_argc=nargc;
     global_argv=nargv;
     ADM_renderLibInit(&UI_Hooks);
-
+#if defined(_WIN32) && QT_VERSION >= QT_VERSION_CHECK(5,11,0)
+    // Despite HiDPI scaling being supported from Qt 5.6 on, important aspects
+    // like OpenGL support were fixed only in much later versions.
+    QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
     myApplication=new myQApplication (global_argc, global_argv);
     myApplication->connect(myApplication, SIGNAL(lastWindowClosed()), myApplication, SLOT(quit()));
     myApplication->connect(myApplication, SIGNAL(aboutToQuit()), myApplication, SLOT(cleanup()));
+#ifdef __APPLE__
+    Q_INIT_RESOURCE(avidemux_osx);
+#elif defined(_WIN32)
+    Q_INIT_RESOURCE(avidemux_win32);
+#else
     Q_INIT_RESOURCE(avidemux);
+#endif
     Q_INIT_RESOURCE(filter);
 
     loadTranslator();
@@ -1612,10 +1692,13 @@ uint8_t initGUI(const vector<IScriptEngine*>& scriptEngines)
     UI_getPhysicalScreenSize(QuiMainWindows, &w,&h);
     printf("The screen seems to be %u x %u px\n",w,h);
     mw->ui.frame_video->setAttribute(Qt::WA_OpaquePaintEvent);
-    
 
     UI_QT4VideoWidget(mw->ui.frame_video);  // Add the widget that will handle video display
+#if QT_VERSION < QT_VERSION_CHECK(5,11,0) // not sure about the version
     mw->ui.frame_video->setAcceptDrops(true); // needed for drag and drop to work on windows
+#endif
+    admPreview::setMainDimension(0,0,ZOOM_1_1);
+
     UI_updateRecentMenu();
     UI_updateRecentProjectMenu();
 
@@ -1640,9 +1723,7 @@ uint8_t initGUI(const vector<IScriptEngine*>& scriptEngines)
 #else
         ADM_info("OpenGL: Not enabled at built time.\n");
 #endif
-    
-    
-    
+
     return 1;
 }
 /**
@@ -1686,14 +1767,15 @@ void UI_applySettings(void)
 {
     ((MainWindow *)QuiMainWindows)->updateActionShortcuts();
     ((MainWindow *)QuiMainWindows)->volumeWidgetOperational();
+    ((MainWindow *)QuiMainWindows)->setRefreshCap();
 }
 /**
-    \fn UI_getCurrentPreview(void)
-    \brief Read previewmode from comboxbox
+    \fn UI_getCurrentPreview
+    \brief Read previewmode from checkable menu actions
 */
 int UI_getCurrentPreview(void)
 {
-    if(WIDGET(checkDisplayOut)->isChecked()) 
+    if(WIDGET(menuVideo)->actions().at(3)->isChecked() || WIDGET(toolBar)->actions().at(5)->isChecked())
     {
         printf("Output is ON\n");
         return 1;
@@ -1703,12 +1785,13 @@ int UI_getCurrentPreview(void)
 }
 
 /**
-    \fn UI_setCurrentPreview(int ne)
-    \brief Update comboxbox with previewmode
+    \fn UI_setCurrentPreview
+    \brief Update "Play filtered" checkable menu actions with previewmode
 */
 void UI_setCurrentPreview(int ne)
 {
-        WIDGET(checkDisplayOut)->setChecked(!!ne);
+    WIDGET(menuVideo)->actions().at(3)->setChecked(!!ne);
+    WIDGET(toolBar)->actions().at(5)->setChecked(!!ne);
 }
 /**
         \fn FatalFunctionQt
@@ -2084,6 +2167,12 @@ void UI_setMarkers(uint64_t a, uint64_t b)
     snprintf(text,79,"%02" PRIu32":%02" PRIu32":%02" PRIu32".%03" PRIu32,hh,mm,ss,ms);
     WIDGET(pushButtonJumpToMarkerB)->setText(text);
 
+    timems=(uint32_t)(b-a);
+    ms2time(timems,&hh,&mm,&ss,&ms);
+    snprintf(text,79,"%02" PRIu32":%02" PRIu32":%02" PRIu32".%03" PRIu32,hh,mm,ss,ms);
+    QString duration=QString("Selection: ")+QString(text);
+    WIDGET(selectionDuration)->setText(duration);
+
     slider->setMarkers(absoluteA, absoluteB);
 }
 
@@ -2187,6 +2276,19 @@ bool UI_setVUMeter( uint32_t volume[6])
     UI_vuUpdate( volume);
     return true;
 }
+
+/**
+    \fn UI_setVolume
+*/
+bool UI_setVolume(void)
+{
+    if(WIDGET(toolButtonAudioToggle)->isChecked())
+        ((MainWindow *)QuiMainWindows)->volumeChange(0);
+    else
+        AVDM_setVolume(0);
+    return true;
+}
+
 /**
     \fn UI_setDecoderName
 */
@@ -2300,8 +2402,7 @@ void UI_setAudioTrackCount( int nb )
 {
     char txt[50];
     sprintf(txt,QT_TRANSLATE_NOOP("qgui2"," (%d track(s))"),nb);
-     WIDGET(TrackCountLabel)->setText(QString(txt));
-
+    WIDGET(TrackCountLabel)->setText(QString::fromUtf8(txt));
 }
 /**
  * \fn dtor
