@@ -2,6 +2,7 @@
 #define Q_gui2_h
 
 #define MKICON(x) ":/new/prefix1/pics/"#x".png"
+#define MKOSXICON(x) ":/new/prefix1/pics/"#x".icns"
 
 #include <vector>
 #include <QtCore/QUrl>
@@ -130,6 +131,7 @@ protected:
 public slots:
         void updateAvailableSlot(int version, std::string date, std::string url);
         void dragTimerTimeout(void);
+        void setRefreshCap(void);
         void actionSlot(Action a)
         {
             if(a==ACT_PlayAvi) // ugly
@@ -160,7 +162,8 @@ public slots:
         void sliderWheel(int way);
 	void volumeChange( int u );
 	void audioToggled(bool checked);
-	void previewModeChanged(int status);
+	void previewModeChangedFromMenu(bool status);
+	void previewModeChangedFromToolbar(bool status);
 	void previousIntraFrame(void);
 	void nextIntraFrame(void);
 	void timeChangeFinished(void);
@@ -204,6 +207,7 @@ protected:
 	void addScriptEnginesToFileMenu(std::vector<MenuEntry>& fileMenu);
 	void addScriptShellsToToolsMenu(vector<MenuEntry>& toolMenu);
 	void addScriptReferencesToHelpMenu();
+        void addSessionRestoreToRecentMenu(vector<MenuEntry>& menu);
         bool buildMyMenu(void);
         bool buildMenu(QMenu *root,MenuEntry *menu, int nb);
 	void buildRecentMenu(QMenu *menu,std::vector<std::string>files, QAction **actions);
