@@ -24,6 +24,8 @@
 #include <math.h>
 #include <string.h>
 #include <ctype.h>
+#include <algorithm>
+
 
 #include "ADM_default.h"
 #include "ADM_assert.h"
@@ -218,7 +220,7 @@ uint32_t ms=(uint32_t)(ams/1000);
     else    
     {
         ms2time(ms,&hh,&mm,&ss,&mms);
-        sprintf(buffer,"%02"PRIu32":%02"PRIu32":%02"PRIu32",%03"PRIu32,hh,mm,ss,mms);
+        sprintf(buffer,"%02" PRIu32":%02" PRIu32":%02" PRIu32",%03" PRIu32,hh,mm,ss,mms);
     }
     return buffer;
 
@@ -228,16 +230,17 @@ uint32_t ms=(uint32_t)(ams/1000);
         \fn ADM_LowerCase
         \brief change to lower case in place the string
 */
-void  ADM_LowerCase(char *string)
+
+
+void  ADM_lowerCase(std::string &st)
 {
-  int l=strlen(string)-1;
-        for(int i=l;i>=0;i--)
-        {
-                string[i]=tolower(string[i]);
-        }
-
+    std::transform(st.begin(), st.end(), st.begin(), ::tolower);
 }
-
+/**
+ * 
+ * @param name
+ * @return 
+ */
 uint8_t         ADM_fileExist(const char *name)
 {
 FILE *file;
@@ -259,7 +262,7 @@ extern char *ADM_slashToBackSlash(const char *in)
 {
     char *out,*cout;
     int n;
-    n=strlen(in);
+    n=(int)strlen(in);
     cout=out=(char *)ADM_alloc(n+1);   
     for(int i=0;i<n+1;i++)
     {
@@ -271,24 +274,29 @@ extern char *ADM_slashToBackSlash(const char *in)
     
 }
 /*
-    
+ * \fn    ADM_getCurrentDate 
 */
-void TLK_getDate(ADM_date *date)
+ const ADM_date        ADM_getCurrentDate()
 {
   time_t timez;
   tm *t;
   time(&timez);
   t=localtime(&timez);
+  ADM_date date;
   if(t)
   {
-    date->hours=t->tm_hour;
-    date->minutes=t->tm_min;
-    date->seconds=t->tm_sec;
+    date.hours=t->tm_hour;
+    date.minutes=t->tm_min;
+    date.seconds=t->tm_sec;
   }
-  
+  return date;
 }
 
-bool shutdown(void)
+ /**
+  * \fn ADM_shutdown
+  * @return 
+  */
+bool ADM_shutdown(void)
 {
 #ifdef _WIN32
 	return (shutdown_win32() == 0);
@@ -296,4 +304,27 @@ bool shutdown(void)
 	return (system("shutdown -P 0") == 0);
 #endif
 }
+
+
+/**
+ * 
+ * @return 
+ */
+const std::string &ADM_getTimeDateAsString(const char *format)
+{    
+    static std::string text; // not thread safe...
+    char timestamp[32];
+    uint64_t ss=ADM_getSecondsSinceEpoch();
+    time_t tme=(time_t)ss;
+    struct tm *ct;
+    ct=localtime(&tme);
+    size_t l=strftime(timestamp,32,format? format : "%F %H%M%S",ct);
+    if(l)
+        text=std::string(timestamp);
+    else
+        text=std::string("UNKNOWN");
+    return text;
+}
+
+
 //EOF
