@@ -38,23 +38,28 @@ extern "C"
 {
 void gettimeofday(struct timeval *p, TIMZ *tz);
 }
-
-void gettimeofday(struct timeval *p, TIMZ *tz)
+// https://stackoverflow.com/questions/10905892/equivalent-of-gettimeday-for-windows
+void gettimeofday(struct timeval * tp, TIMZ * tzp)
 {
-    unsigned long int sec;
-    SYSTEMTIME  tme;
+    // Note: some broken versions only have 8 trailing zero's, the correct epoch has 9 trailing zero's
+    // This magic number is the number of 100 nanosecond intervals since January 1, 1601 (UTC)
+    // until 00:00:00 January 1, 1970 
+    static const uint64_t EPOCH = ((uint64_t)116444736000000000ULL);
 
-    GetSystemTime(&tme);
+    SYSTEMTIME  system_time;
+    FILETIME    file_time;
+    uint64_t    time;
 
-    sec=tme.wSecond;
-    sec+=60*tme.wMinute;
-    sec+=60*60*tme.wHour;
+    GetSystemTime(&system_time);
+    SystemTimeToFileTime(&system_time, &file_time);
+    time = ((uint64_t)file_time.dwLowDateTime);
+    time += ((uint64_t)file_time.dwHighDateTime) << 32;
 
-    p->tv_sec=sec;
-    p->tv_usec=tme.wMilliseconds*1000;
-
-    return;
+    tp->tv_sec = (long)((time - EPOCH) / 10000000L);
+    tp->tv_usec = (long)(system_time.wMilliseconds * 1000);
+    return ;
 }
+
 #endif    // HAVE_GETTIMEOFDAY
 
 int getpriority(int which, int who)
