@@ -9,7 +9,7 @@
 #include "ADM_win32.h"
 #include "ADM_misc.h"
 #include <algorithm>
-void redirectStdoutToFile(void)
+void redirectStdoutToFile(const char *logFile)
 {
     // Don't redirect stdout and stderr if SDL hasn't already hijacked it.
     // This allows us to optionally compile all EXEs as console applications
@@ -39,10 +39,21 @@ void redirectStdoutToFile(void)
     remove(stderrPath);
 
     // Redirect output to log file in the user's profile directory
-    const char* logFile = "admlog.txt";
-
+ #if 1
     std::string filePath=std::string(ADM_getLogDir())+std::string(logFile);
+#else
+    std::string filePath=std::string("f:\\tmp\\lm\\")+std::string(logFile);
+#endif
 
-    freopen(filePath.c_str(),"w",stdout);
- 
+    int length = utf8StringToWideChar(filePath.c_str(), -1, NULL);
+    wchar_t *filePath2 = new wchar_t[length+1];
+    wchar_t mode[] = L"w";
+    utf8StringToWideChar(filePath.c_str(), -1, filePath2);
+    FILE *newFD=_wfreopen(filePath2, mode,stdout);
+    delete[] filePath2;
+    if (!newFD)
+    {
+        assert(0);
+        ADM_assert(0);
+    }
 }
