@@ -33,13 +33,9 @@ ADM_audioStream::ADM_audioStream(WAVHeader *header,ADM_audioAccess *access)
     lastDts=ADM_AUDIO_NO_DTS;
     lastDtsBase=0;
     sampleElapsed=0;
-    if(access)
-    {
-        if(access->canGetDuration()==true)
-                durationInUs=access->getDurationInUs();
-        else
-                durationInUs=0;
-    }
+    durationInUs=0;
+    if(access && access->canGetDuration())
+        durationInUs=access->getDurationInUs();
     language=ADM_UNKNOWN_LANGUAGE;
 }
 /**
@@ -179,9 +175,10 @@ bool    ADM_audioStream::advanceDtsByCustomSample(uint32_t samples,uint32_t fq)
 */
 ADM_audioStream  *ADM_audioCreateStream(WAVHeader *wavheader, ADM_audioAccess *access,bool makeTimeMap)
 {
-uint32_t size;
     switch(wavheader->encoding)
     {
+        case WAV_PCM_FLOAT:
+            return new ADM_audioStreamFloatPCM(wavheader,access);            
         case WAV_EAC3:
             return new ADM_audioStreamEAC3(wavheader,access);
         case WAV_AC3:
@@ -223,6 +220,7 @@ const char *getStrFromAudioCodec( uint32_t codec)
               case WAV_OPUS:  return QT_TRANSLATE_NOOP("adm","OPUS");
               case WAV_FLAC:  return QT_TRANSLATE_NOOP("adm","FLAC");
               case WAV_EAC3:  return QT_TRANSLATE_NOOP("adm","E-AC3");
+              case WAV_PCM_FLOAT:  return QT_TRANSLATE_NOOP("adm","Float PCM");
               case WAV_OGG_VORBIS: return QT_TRANSLATE_NOOP("adm","Ogg Vorbis");
               case WAV_MP4: return QT_TRANSLATE_NOOP("adm","MP4");
               case WAV_AAC: return QT_TRANSLATE_NOOP("adm","AAC");
