@@ -23,6 +23,10 @@ GPL-v2
 */
 ADM_audioStreamMP3::ADM_audioStreamMP3(WAVHeader *header,ADM_audioAccess *access,bool createMap) : ADM_audioStreamBuffered(header,access)
 {
+    // Suppress repeated debug messages
+    _msg_counter=0;
+    _msg_ratelimit=new ADMCountdown(200);
+    _msg_ratelimit->reset();
     // If hinted..., compute the duration ourselves
     if(access->isCBR()==true && access->canSeekOffset()==true)
     {
@@ -43,13 +47,14 @@ ADM_audioStreamMP3::ADM_audioStreamMP3(WAVHeader *header,ADM_audioAccess *access
         if(true==createMap)
         {
             buildTimeMap();
+            uint32_t nb=seekPoints.size();
+            if(nb)
+                durationInUs=seekPoints[nb-1]->timeStamp;
             return;
         }
     }
     // Time based
     durationInUs=access->getDurationInUs();
-    
-
 }
 
 /**
@@ -64,6 +69,9 @@ ADM_audioStreamMP3::~ADM_audioStreamMP3()
         delete seekPoints[i];
         seekPoints[i]=NULL;
     }
+    if(_msg_ratelimit)
+        delete _msg_ratelimit;
+    _msg_ratelimit = NULL;
 }
 /**
     \fn goToTime
@@ -88,7 +96,7 @@ bool         ADM_audioStreamMP3::goToTime(uint64_t nbUs)
     // if VBR use our time map
     if(!seekPoints.size())
     {
-        ADM_error("VBR MP2/MP3 stream with no time map, cannot seek");
+        ADM_error("VBR MP2/MP3 stream with no time map, cannot seek\n");
         return false;
     }
     if(nbUs<=seekPoints[0]->timeStamp) // too early
@@ -112,7 +120,7 @@ bool         ADM_audioStreamMP3::goToTime(uint64_t nbUs)
             return true;
         }
     }
-    ADM_error("VBR MP2/MP3 request for time outside of time map, cannot seek");
+    ADM_error("VBR MP2/MP3 request for time outside of time map, cannot seek\n");
     return false;
 }
 /**
@@ -130,8 +138,22 @@ int nbSyncBytes=0;
         // Do we have enough ? Refill if needed ?
         if(needBytes(ADM_LOOK_AHEAD)==false) 
         {
-            ADM_warning("MP3: Not enough data to lookup header\n");
-                return 0;
+            if(_msg_ratelimit->done())
+            {
+                if(_msg_counter)
+                {
+                    ADM_warning("MP3: Not enough data to lookup header (message repeated %" PRIu32" times)\n",_msg_counter);
+                    _msg_counter=0;
+                }else
+                {
+                    ADM_warning("MP3: Not enough data to lookup header\n");
+                }
+                _msg_ratelimit->reset();
+            }else
+            {
+                _msg_counter++;
+            }
+            return 0;
         }
         // Peek
         peek(ADM_LOOK_AHEAD,data);
@@ -150,6 +172,7 @@ int nbSyncBytes=0;
                 //printf("%"PRId64" , size=%d\n",*dts,*size);
                 if(nbSyncBytes)
                         ADM_info("[MP3 Stream] Sync found after %d bytes...\n",nbSyncBytes);
+                _msg_counter=0;
                 return 1;
             }
             
@@ -177,7 +200,7 @@ DIA_workingBase *work=createWorking(QT_TRANSLATE_NOOP("adm","Building time map")
     
     ADM_assert(access->canSeekOffset()==true);
     access->setPos(0);
-    printf("[audioStreamMP3] Starting time map\n");
+    ADM_info("Starting MP3 time map\n");
     rewind();
     Clock *clk=new Clock();
     clk->reset();
@@ -248,7 +271,7 @@ DIA_workingBase *work=createWorking(QT_TRANSLATE_NOOP("adm","Building time map")
     delete work;
     delete clk;
     access->setPos(0);
-    printf("[audioStreamMP3] Ending time map\n");
+    ADM_info("Finishing MP3 time map\n");
     return true;
 }
   
