@@ -29,7 +29,7 @@ const char *i2state(int a)
     return "?????";
 
 }
-#define CHANGE_STATE(x) { printf("%s -> %s\n",i2state(stopRequest),i2state(x));stopRequest=x;}
+#define CHANGE_STATE(x) { ADM_info("%s -> %s\n",i2state(stopRequest),i2state(x));stopRequest=x;}
 /**
     \fn audioDeviceThreaded
     \brief constructor
@@ -81,6 +81,25 @@ uint8_t audioDeviceThreaded::init(uint32_t channel, uint32_t fq ,CHANNEL_TYPE *c
     // Allocate buffer
     memcpy(incomingMapping,channelMapping,sizeof(CHANNEL_TYPE)*MAX_CHANNELS);
     _channels=channel;
+
+    char incoming[200];
+    char outgoing[200];
+    char chnl[20];
+    char *str1=incoming;
+    char *str2=outgoing;
+    *str1=*str2=0;
+    const CHANNEL_TYPE *outgoingMapping=getWantedChannelMapping(_channels);
+    for(int i=0;i<_channels;i++)
+    {
+        sprintf(chnl,"    %s\n",ADM_printChannel(incomingMapping[i]));
+        strcat(str1,chnl);
+        sprintf(chnl,"    %s\n",ADM_printChannel(*outgoingMapping));
+        strcat(str2,chnl);
+        outgoingMapping++;
+    }
+    ADM_info("Incoming channel map:\n%s",str1);
+    ADM_info("Outgoing channel map:\n%s",str2);
+
     _frequency=fq;
     sizeOf10ms=(_channels*_frequency*2)/100;
     sizeOf10ms&=~15; // make sure it is a multiple of 16
@@ -248,7 +267,12 @@ bool        audioDeviceThreaded::getVolumeStats(uint32_t *vol)
         if(d>255) d=255;
         raw[i]=(uint32_t)d;
     }
-    
+    // Assign mono to front center
+    if(_channels==1)
+    {
+        vol[1]=raw[0];
+        return true;
+    }
     // Move channels around so that they fit Left/Right/Center/Rear Left, Rear Right,LFE
     const CHANNEL_TYPE *chans=this->getWantedChannelMapping(_channels);
     static const CHANNEL_TYPE output[6]={ADM_CH_FRONT_LEFT,ADM_CH_FRONT_CENTER,ADM_CH_FRONT_RIGHT,
