@@ -124,7 +124,11 @@ uint8_t fileParser::open( const char *filename,FP_TYPE *multi )
         uint32_t current=base;
         _curFd = 0;
         uint64_t total=0;
-        
+        uint64_t threshold,tolerance;
+        threshold=tolerance=1;
+        threshold<<=28;
+        tolerance<<=20;
+
         // build match string
         char match[16];
         match[0]='%';
@@ -159,6 +163,13 @@ uint8_t fileParser::open( const char *filename,FP_TYPE *multi )
                 fdIo myFd;
                 myFd.file=f;
                 myFd.fileSize=ADM_fileSize(outName.c_str());
+                // check whether the file likely belongs to a different stream
+                if(count && myFd.fileSize > threshold+tolerance)
+                {
+                    ADM_fclose(myFd.file);
+                    break;
+                }
+
                 myFd.fileSizeCumul = total;
                 total+=  myFd.fileSize;
 
@@ -166,6 +177,21 @@ uint8_t fileParser::open( const char *filename,FP_TYPE *multi )
                                             myFd.fileSize );
 
                 listOfFd.append(myFd);
+                // append only if all files but the last one are of ~ equal size: 256 or 512 MiB,
+                // 1, 2 or 4 GiB, the usual threshold for automatically split streams
+                if(myFd.fileSize < threshold-tolerance)
+                    break;
+                if(!count)
+                {
+                    for(int i=0;i<5;i++)
+                    {
+                        if(myFd.fileSize >= threshold-tolerance && myFd.fileSize <= threshold+tolerance)
+                            break;
+                        threshold<<=1;
+                        if(i==1)
+                            tolerance<<=3; // 8 MiB starting with 1 GiB fragment size
+                    }
+                }
                 count++;
                 current++;
         } 
@@ -334,14 +360,16 @@ uint8_t fileParser::getpos(uint64_t *o)
 ----------------------------------------------------*/
 uint32_t fileParser::read32(uint32_t len, uint8_t *buffer)
 {
-uint32_t r;
-
 uint64_t remain,begin,mx,last;
 
         ADM_assert(_off>=_head);
         ADM_assert(_off<=_tail);
 
-        if(_head>=_size-1) return 0;
+        if(_head>=_size-1)
+        {
+            memset(buffer,0,len);
+            return 0;
+        }
 
 // Check we do not go out of bound
         if(_off+len>=_size)
@@ -472,8 +500,22 @@ uint8_t r;
     }
     else    
     {
-        read32(1,&r);
-        _off--;
+        uint64_t mx=listOfFd[_curFd].fileSize+listOfFd[_curFd].fileSizeCumul-_off;
+        // If nothing is left, switch to the next file
+        if(!mx)
+        {
+            _head=_tail=_off;
+            _curFd++;
+            if(_curFd>=listOfFd.size()) return 0;
+            fseeko(listOfFd[_curFd].file,0,SEEK_SET);
+            mx=listOfFd[_curFd].fileSize;
+        }
+        if(mx>DMX_BUFFER) mx=DMX_BUFFER;
+        // Fill the buffer
+        fread(_buffer,mx,1,listOfFd[_curFd].file);
+        _head=_off;
+        _tail=_head+mx;
+        r=_buffer[0];
     }
     return r;
 
