@@ -18,6 +18,7 @@
  ***************************************************************************/
 
 #include "ADM_default.h"
+#include "ADM_vidMisc.h"
 #include "fourcc.h"
 #include "ADM_coreMuxerFfmpeg.h"
 #include "ADM_muxerUtils.h"
@@ -181,7 +182,7 @@ bool muxerFFmpeg::initVideo(ADM_videoStream *stream)
         c->rc_buffer_size=8*1024*224;
         c->rc_max_rate=9500*1000;
         c->rc_min_rate=0;
-        c->flags=CODEC_FLAG_QSCALE;
+        c->flags=AV_CODEC_FLAG_QSCALE;
 
         uint32_t fcc=stream->getFCC();
 
@@ -281,7 +282,7 @@ bool muxerFFmpeg::initVideo(ADM_videoStream *stream)
             if(videoExtraDataSize)
             {
                 ADM_info("Video has extradata and muxer requires globalHeader, assuming it is done so.\n");
-                c->flags|=CODEC_FLAG_GLOBAL_HEADER;
+                c->flags|=AV_CODEC_FLAG_GLOBAL_HEADER;
             }else
             {
                 ADM_warning("Video has no extradata but muxer requires globalHeader.\n");
@@ -353,10 +354,20 @@ bool muxerFFmpeg::initAudio(uint32_t nbAudioTrack,ADM_audioStream **audio)
                               par->frame_size=1152;
                               par->codec_id = AV_CODEC_ID_MP3;
                               break;
+                  case WAV_LPCM:
                   case WAV_PCM:
+                                if(audioheader->encoding==WAV_LPCM)
+                                {
+                                  // One chunk is 10 ms (1/100 of fq)
+                                  par->frame_size=4;
+                                  par->codec_id = AV_CODEC_ID_PCM_S16BE;break;
+                                }
+                                else
+                                {
                                   // One chunk is 10 ms (1/100 of fq)
                                   par->frame_size=4;
                                   par->codec_id = AV_CODEC_ID_PCM_S16LE;break;
+                                }
                   case WAV_AAC:
                                   ffmpuxerSetExtradata(par,audioextraSize,audioextraData);
                                   par->codec_id = AV_CODEC_ID_AAC;
@@ -376,7 +387,7 @@ bool muxerFFmpeg::initAudio(uint32_t nbAudioTrack,ADM_audioStream **audio)
             if(audioextraSize)
             {
                 ADM_info("Audio has extradata and muxer requires globalHeader, assuming it is done so.\n");
-                c->flags|=CODEC_FLAG_GLOBAL_HEADER;
+                c->flags|=AV_CODEC_FLAG_GLOBAL_HEADER;
             }else
             {
                 ADM_warning("Audio has no extradata but muxer requires globalHeader.\n");
@@ -433,9 +444,10 @@ bool muxerFFmpeg::saveLoop(const char *title)
     uint8_t *buffer=new uint8_t[bufSize];
     uint64_t rawDts;
     uint64_t lastVideoDts=0;
+    uint64_t lastVideoDtsLav=0;
     uint64_t videoIncrement;
-    int ret;
-    int written=0;
+    bool ret=true;
+    uint32_t written=0;
     bool result=true;
     int missingPts=0;
     
@@ -478,22 +490,44 @@ bool muxerFFmpeg::saveLoop(const char *title)
             rawDts=out.dts;
             if(rawDts==ADM_NO_PTS)
             {
+                ADM_warning("No DTS information for frame %" PRIu32"\n",written);
                 lastVideoDts+=videoIncrement;
+            }else if(lastVideoDts && rawDts<=lastVideoDts)
+            {
+                ADM_warning("Duplicated or going back DTS for frame %" PRIu32", adding %" PRIu64" ms\n",written,(videoIncrement/2)/1000);
+                lastVideoDts+=videoIncrement/2;
+                out.dts=lastVideoDts;
             }else
             {
                 lastVideoDts=out.dts;
             }
+
             if(out.pts==ADM_NO_PTS)
             {
                 ADM_warning("No PTS information for frame %" PRIu32"\n",written);
                 missingPts++;
                 out.pts=lastVideoDts;
             }
-
+            if(out.pts<lastVideoDts)
+            {
+                ADM_warning("Bumping PTS to keep PTS >= DTS for frame %" PRIu32"\n",written);
+                out.pts=lastVideoDts;
+            }
 
             encoding->pushVideoFrame(out.len,out.out_quantizer,lastVideoDts);
             muxerRescaleVideoTimeDts(&(out.dts),lastVideoDts);
+            if(!roundup && lastVideoDtsLav && out.dts==lastVideoDtsLav)
+            {
+                ADM_warning("Bumping lav DTS to avoid collision for frame %" PRIu32"\n",written);
+                out.dts++;
+            }
+            lastVideoDtsLav=out.dts;
             muxerRescaleVideoTime(&(out.pts));
+            if(out.dts>out.pts)
+            {
+                ADM_warning("Bumping lav PTS to keep PTS >= DTS for frame %" PRIu32"\n",written);
+                out.pts=out.dts;
+            }
             aprintf("[FF:V] RawDts:%lu Scaled Dts:%lu\n",rawDts,out.dts);
             aprintf("[FF:V] Rescaled: Len : %d flags:%x Pts:%" PRIu64" Dts:%" PRIu64"\n",out.len,out.flags,out.pts,out.dts);
 
@@ -527,7 +561,6 @@ bool muxerFFmpeg::saveLoop(const char *title)
                 WAVHeader *info=a->getInfo();
                 if(!info) // no more track
                     continue;
-                uint32_t fq=info->frequency;
 
                 while(1)
                 {
@@ -552,7 +585,7 @@ bool muxerFFmpeg::saveLoop(const char *title)
                     if(audioTrack->dts!=ADM_NO_PTS)
                     {
                         //printf("Audio PTS:%"PRId64", limit=%"PRId64"\n",audioTrack->dts,lastVideoDts+videoIncrement);
-                        if(audioTrack->dts>lastVideoDts+videoIncrement) break; // This packet is in the future
+                        if(audioTrack->dts>lastVideoDts+videoIncrement+audioDelay) break; // This packet is in the future
                     }
                     // Write...
                     AVPacket pkt;
@@ -589,9 +622,16 @@ bool muxerFFmpeg::saveLoop(const char *title)
 
     }
     delete [] buffer;
-    if((videoDuration *4)/5 > lastVideoDts)
+    if(false==ret)
     {
-        GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Too short"), QT_TRANSLATE_NOOP("adm","The video has been saved but seems to be incomplete."));
+        char msg[512+1];
+        snprintf(msg,512,QT_TRANSLATE_NOOP("adm",
+            "The saved video is incomplete. "
+            "The error occured at %s (%d\%). "
+            "This may happen as result of invalid time stamps in the video."),
+            ADM_us2plain(lastVideoDts),
+            (int)(lastVideoDts*100/videoDuration));
+        GUI_Error_HIG(QT_TRANSLATE_NOOP("adm","Too short"),msg);
         result=false;
     }
     ADM_info("[FF] Wrote %d frames, nb audio streams %d\n",written,nbAStreams);
