@@ -58,6 +58,16 @@ uint8_t diaFactoryRunTabs(const char *title,uint32_t nb,diaElemTabs **tabs)
 	ADM_assert(Factory); 
 	return Factory->FactoryRunTab(title,nb,tabs);
 }
+void * diaFactoryRunTabsPrepare(const char *title,uint32_t nb,diaElemTabs **tabs)
+{
+	ADM_assert(Factory); 
+	return Factory->FactoryTabPrepare(title,nb,tabs);
+}
+bool  diaFactoryRunTabsFinish(void *f)
+{
+        ADM_assert(Factory); 
+	return Factory->FactoryTabFinish(f);
+}
 
 // ****************** Buttons ********************
 diaElemButton ::diaElemButton(const char *toggleTitle, ADM_FAC_CALLBACK *cb,void *cookie,const char *tip) :diaElem(ELEM_BUTTON)
@@ -361,7 +371,7 @@ void diaElemBitrate::setMaxQz(uint32_t qz)
 }
 DIA_MKSTUBS(diaElemBitrate)
 // ****************** diaElemFile ********************
-diaElemFile ::diaElemFile(uint32_t writeMode,char **filename,const char *toggleTitle,
+diaElemFile ::diaElemFile(uint32_t writeMode,std::string &filename,const char *toggleTitle,
         const char *defaultSuffix ,const char *tip)
 {
 	ADM_assert(Factory); 
@@ -386,7 +396,7 @@ void      diaElemFile::changeFile(void)
 DIA_MKSTUBS(diaElemFile)
 
 // ****************** diaElemDirSelect ********************
-diaElemDirSelect ::diaElemDirSelect(char **filename,const char *toggleTitle,const char *tip):
+diaElemDirSelect ::diaElemDirSelect(std::string &filename,const char *toggleTitle,const char *tip):
 	diaElemDirSelectBase()
 {
 	ADM_assert(Factory); 
@@ -465,6 +475,31 @@ void      diaElemToggleUint::enable(uint32_t onoff)
 		internalPointer->enable(onoff); 
 	}
 DIA_MKSTUBS(diaElemToggleUint)
+// ****************** diaElemAspectRatio ********************
+diaElemAspectRatio ::diaElemAspectRatio(uint32_t *num,uint32_t *den,const char *toggleTitle,const char *tip):
+	diaElem(ELEM_ASPECT_RATIO)
+{
+	ADM_assert(Factory); 
+	internalPointer=Factory->CreateAspectRatio(num,den,toggleTitle,tip);
+}
+diaElemAspectRatio ::~diaElemAspectRatio()
+	{
+	
+	ADM_assert(Factory); 
+	Factory->DestroyAspectRatio(internalPointer);
+	internalPointer=NULL;
+}
+void      diaElemAspectRatio::finalize(void)
+	{ 
+		ADM_assert(internalPointer); 
+		internalPointer->finalize(); 
+	}
+void      diaElemAspectRatio::enable(uint32_t onoff)
+	{ 
+		ADM_assert(internalPointer); 
+		internalPointer->enable(onoff); 
+	}
+DIA_MKSTUBS(diaElemAspectRatio)
 // ****************** diaElemToggleInt ********************
 diaElemToggleInt ::diaElemToggleInt(uint32_t *toggleValue,const char *toggleTitle, int32_t *uintval, 
 									const char *name,int32_t min,int32_t max,const char *tip):
