@@ -39,6 +39,8 @@ const ffVideoCodec ffCodec[]=
   {"MJPB",  AV_CODEC_ID_MJPEGB,    false, false,    false},
   {"FPS1",  AV_CODEC_ID_FRAPS,     false, false,    false},
   {"cvid",  AV_CODEC_ID_CINEPAK,   false, false,    false},
+  {"FICV",  AV_CODEC_ID_FIC,       false, false,    false},
+  {"AVdn",  AV_CODEC_ID_DNXHD,     false, false,    false},
 // Need extradata
   {"WMV2", AV_CODEC_ID_WMV2,       true, false,    false},
   {"WMV1", AV_CODEC_ID_WMV1,       true, false,    false},
@@ -112,6 +114,10 @@ AVCodecID ADM_codecIdFindByFourcc(const char *fcc)
     {
       return AV_CODEC_ID_MPEG4;
     }
+if (isVP9Compatible (fid) == 1)
+    {
+      return AV_CODEC_ID_VP9;
+    }
 
     uint32_t nb=sizeof(ffCodec)/sizeof(ffVideoCodec);
     for(int i=0;i<nb;i++)
