@@ -55,32 +55,37 @@ void mixDump(uint8_t * ptr, uint32_t len)
     *str = *str2 = 0;
 
     for (i = 0; i < len; i++)
-      {
-	  			if (*ptr < 32)
-			    {
-						strcat(str, ".");
-				  } else
-			    {
-						sprintf(tiny, "%c", *ptr);
-						strcat(str, tiny);
-			    }
+    {
+        if (*ptr < 32 || *ptr > 126)
+        {
+            strcat(str, ".");
+        } else
+        {
+            sprintf(tiny, "%c", *ptr);
+            strcat(str, tiny);
+        }
 
-	  	sprintf(tiny, " %02x", *ptr);
-		  strcat(str2, tiny);
-		  ptr++;
+        sprintf(tiny, " %02x", *ptr);
+        strcat(str2, tiny);
+        ptr++;
 
-		  if ((i % 16) == 15)
-		    {
-					printf("\n %04" PRIx32" : %s %s", (i >> 4) << 4, str, str2);
-					*str = 0;
-					*str2 = 0;
-		    }
-      }
-	// left over
-	if(len%16!=0)
-	{
-		 printf("\n %04" PRIx32" : %s %s", (len >> 4) << 4, str, str2);
-	}
+        if ((i % 16) == 15)
+        {
+            printf("\n %04" PRIx32" : %s  %s", (i >> 4) << 4, str, str2);
+            *str = 0;
+            *str2 = 0;
+        }
+    }
+    // left over
+    uint32_t leftover = len % 16;
+    if (leftover)
+    {
+        for (i = 0; i < 16 - leftover; i++)
+        {
+            strcat(str, " ");
+        }
+        printf("\n %04" PRIx32" : %s  %s", (len >> 4) << 4, str, str2);
+    }
     printf("\n");
 }
 /*
@@ -189,57 +194,41 @@ void memcpyswap(uint8_t *dest, uint8_t *src, uint32_t size)
 
 }
 /**
-    \file ADM_findMpegStartCode
-    \brief    Find mpeg1/2/4 video startcode
-    00 00 01 xx yy
-    return xx + offset to yy
-
+    \fn ADM_findMpegStartCode
+    \brief Compatibility wrapper
 */
 bool ADM_findMpegStartCode(uint8_t *start, uint8_t *end,uint8_t *outstartcode,uint32_t *offset)
 {
-    uint32_t startcode=0xffffffff;
-    uint8_t  *ptr=start;
-
-
-    while(ptr<end)
-	{
-		startcode=(startcode<<8)+*ptr;
-		if((startcode&0xffffff00)==0x100)
-		{
-			*outstartcode=*ptr;
-			*offset=ptr-start+1;
-			return true;
-		}
-		ptr++;
-	}
-	return false; // startcode not found
+    bool zero=false;
+    return ADM_findAnnexBStartCode(start,end,outstartcode,offset,&zero);
 }
 /**
-    \file ADM_findH264StartCode
-    \brief    Find mpeg1/2/4 video startcode
-    00 00 00 01 xx yy
-    return xx + offset to yy
-
+    \fn     ADM_findAnnexBStartCode
+    \brief  Find Annex B video stream startcode 00 00 01 xx yy
+            and check whether it is preceded by zero byte,
+            return xx + offset to payload (yy).
 */
-
-bool ADM_findH264StartCode(uint8_t *start, uint8_t *end,uint8_t *outstartcode,uint32_t *offset)
+bool ADM_findAnnexBStartCode(uint8_t *start, uint8_t *end, uint8_t *outstartcode, uint32_t *offset, bool *zero)
 {
     uint32_t startcode=0xffffffff;
     uint8_t  *ptr=start;
+    *zero=false;
     end--;
 
     while(ptr<end)
-	{
-		startcode=(startcode<<8)+*ptr;
-		if(1==startcode)
-		{
-			*outstartcode=ptr[1];
-			*offset=ptr-start+2;
-			return true;
-		}
-		ptr++;
-	}
-	return false; // startcode not found
+    {
+        startcode=(startcode<<8)+*ptr;
+        if(1==(startcode&0xffffff))
+        {
+            if(!(startcode>>24))
+                *zero=true;
+            *outstartcode=ptr[1];
+            *offset=ptr-start+2;
+            return true;
+        }
+        ptr++;
+    }
+    return false; // startcode not found
 }
 
 //**********************************************************
@@ -502,6 +491,31 @@ bool ADM_probeSequencedFile(const char *fileName)
      if(false==ADM_splitSequencedFile(fileName, &left, &right,&nbDigit,&base))
             return false;
 
+    // check whether the filesize approx. matches 2^n GiB, the usual
+    // threshold for automatically split streams
+    bool success=false;
+    uint64_t fileSize,threshold,tolerance;
+    threshold=tolerance=1;
+    threshold<<=28; // we start at 256 MiB, this value is hardcoded in some devices
+    tolerance<<=20; // 1 MiB
+    int64_t sz=ADM_fileSize(fileName);
+    if(sz<0)
+        return false;
+    fileSize=sz;
+    for(int i=0;i<5;i++)
+    {
+        if(fileSize >= threshold-tolerance && fileSize <= threshold+tolerance)
+        {
+            success=true;
+            break;
+        }
+        threshold<<=1;
+        if(i==1)
+            tolerance<<=3; // 8 MiB starting with 1 GiB fragment size
+    }
+    if(!success)
+        return false;
+
     // check if at least one sequence exists...
     std::string aLeft(left);
     std::string aRight(right);
@@ -520,14 +534,9 @@ bool ADM_probeSequencedFile(const char *fileName)
     sprintf(names,match,base+1);
     std::string middle(names);
     std::string target=aLeft+middle+aRight;
-    bool r=false;
-    FILE *f=ADM_fopen(target.c_str(),"rb");
-    if(f)
-    {
-        fclose(f);
-        r=true;
-    }
-
-    return r;
+    sz=ADM_fileSize(target.c_str());
+    if(sz<0 || sz > threshold+tolerance)
+        return false;
+    return true;
 }
 //EOF
