@@ -275,12 +275,21 @@ static bool checkProfile(const VAProfile &profile,VAConfigID *cid,const char *na
     *cid=-1;
     VAConfigAttrib attrib;
     attrib.type = VAConfigAttribRTFormat;
+    ADM_info("--Probing %s ...\n",name);
     CHECK_ERROR(vaGetConfigAttributes(ADM_coreLibVA::display, profile, VAEntrypointVLD, &attrib, 1));
     if(xError)
     {
-         ADM_warning("Cannot get attribute for %s \n",name);
+         ADM_warning("Cannot get attribute  for VAEntrypointVLD %s \n",name);
          return false;
     }
+    ADM_info("RT Format =0x%x\n",attrib.value);
+#define CHECK_RT(x)    if(attrib.value & VA_RT_FORMAT_##x ) ADM_info("\t" #x " supported\n");
+    CHECK_RT(YUV420);
+    CHECK_RT(YUV422);
+    CHECK_RT(YUV444);
+    CHECK_RT(YUV420_10BPP);
+    CHECK_RT(RGB32);
+    
     CHECK_ERROR(vaCreateConfig( ADM_coreLibVA::display, profile, VAEntrypointVLD,&attrib, 1,cid));
     if(xError)
     {
@@ -446,7 +455,6 @@ bool admLibVA::init(GUI_WindowInfo *x)
 {
     Display *dis=(Display *)x->display;
     ADM_coreLibVA::display=vaGetDisplay(dis);
-    int maj=0,min=0,patch=0;
     ADM_info("[LIBVA] Initializing LibVA library ...\n");
 
 
@@ -454,6 +462,11 @@ bool admLibVA::init(GUI_WindowInfo *x)
     ADM_coreLibVA::decoders::h264=false;
     ADM_coreLibVA::directOperation=true;
     ADM_coreLibVA::transferMode=ADM_LIBVA_NONE;
+    
+    ADM_coreLibVA::configH265=-1;
+    ADM_coreLibVA::configH26510Bits=-1;
+    ADM_coreLibVA::configVP9=-1;
+    
 
     myWindowInfo=*x;
     VAStatus xError;
@@ -565,7 +578,11 @@ VAContextID        admLibVA::createDecoder(VAProfile profile,int width, int heig
                 break;
 
     }
-
+    if(cid==-1)
+    {
+        ADM_warning("No VA support for that\n");
+        return VA_INVALID;
+    }
     CHECK_ERROR(vaCreateContext ( ADM_coreLibVA::display, cid,
                 width,    height,
                 VA_PROGRESSIVE, // ?? NOT SURE ??
@@ -896,7 +913,6 @@ dropIt:
 bool        admLibVA::putX11Surface(ADM_vaSurface *img,int widget,int displayWidth,int displayHeight)
 {
     int xError;
-    VASurfaceStatus status;
     CHECK_WORKING(false);
     CHECK_ERROR(vaPutSurface ( ADM_coreLibVA::display, img->surface,(Drawable)widget,0,0,img->w, img->h,0,0,displayWidth,displayHeight,
                               NULL,0 // clip & and num clip
@@ -916,7 +932,6 @@ bool   admLibVA::imageToSurface(VAImage *src, ADM_vaSurface *dst)
 {
 
     int xError;
-    VASurfaceStatus status;
     CHECK_WORKING(false);
     CHECK_ERROR(vaPutImage(ADM_coreLibVA::display,
                            dst->surface,
@@ -940,7 +955,6 @@ bool   admLibVA::surfaceToImage(ADM_vaSurface *dst,VAImage *src )
 {
 
     int xError;
-    VASurfaceStatus status;
     CHECK_WORKING(false);
     CHECK_ERROR(vaGetImage(ADM_coreLibVA::display,
                            dst->surface,
@@ -965,7 +979,6 @@ bool   admLibVA::surfaceToImage(ADM_vaSurface *dst,VAImage *src )
 bool   admLibVA::uploadToImage( ADMImage *src,VAImage *dest)
 {
     int xError;
-    VASurfaceStatus status;
     CHECK_WORKING(false);
     uint8_t *ptr=NULL;
     CHECK_ERROR(vaMapBuffer(ADM_coreLibVA::display, dest->buf, (void**)&ptr))
@@ -1003,7 +1016,6 @@ bool   admLibVA::uploadToImage( ADMImage *src,VAImage *dest)
 bool   admLibVA::downloadFromImage( ADMImage *src,VAImage *dest,ADMColorScalerSimple *color)
 {
     int xError;
-    VASurfaceStatus status;
     CHECK_WORKING(false);
     uint8_t *ptr=NULL;
     CHECK_ERROR(vaMapBuffer(ADM_coreLibVA::display, dest->buf, (void**)&ptr))
@@ -1106,7 +1118,6 @@ bool   admLibVA:: admImageToSurface( ADMImage *src,ADM_vaSurface *dest)
 {
     int xError;
     bool r=false;
-    VASurfaceStatus status;
     CHECK_WORKING(false);
     uint8_t *ptr=NULL;
 
@@ -1361,5 +1372,4 @@ bool        admLibVA::destroyFilterConfig(VAConfigID &id)
 }
 
 
-#include "ADM_coreLibVA_encoder.cpp"
 #endif
