@@ -20,22 +20,24 @@
 #include "DIA_coreToolkit.h"
 #include "ADM_hwAccel.h"
 
-extern "C"
-{
-    static void ADM_releaseBuffer(struct AVCodecContext *avctx, AVFrame *pic);
-    static int  ADM_getBuffer(AVCodecContext *avctx, AVFrame *pic);
-}
+#ifdef ADM_DEBUG
+    #define LAV_VERBOSITY_LEVEL AV_LOG_DEBUG
+#else
+    #define LAV_VERBOSITY_LEVEL AV_LOG_INFO
+#endif
 
 #define aprintf(...) {}
 
 
 //****************************
-extern uint8_t DIA_lavDecoder (bool  * swapUv, bool * showU);
+extern uint8_t DIA_lavDecoder (bool *swapUv);
+#if 0
 extern "C"
 {
   int av_is_voppacked (AVCodecContext * avctx, int *vop_packed, int *gmc,
 		       int *qpel);
 };
+#endif
 /**
     \fn clonePic
     \brief Convert AvFrame to ADMImage
@@ -126,17 +128,15 @@ uint8_t decoderFF::getPARHeight (void)
 //________________________________________________
 bool  decoderFF::setParam(void)
 {
-	DIA_lavDecoder(&decoderFF_params.swapUv, &decoderFF_params.showMv);
-
-	return true;			// no param for ffmpeg
+    DIA_lavDecoder(&decoderFF_params.swapUv);
+    return true;
 }
 
-const decoderFF::decoderFF_param_t decoderFF::defaultConfig = {false, false};
+const decoderFF::decoderFF_param_t decoderFF::defaultConfig = {false};
 
 const ADM_paramList decoderFF::decoderFF_param_template[] =
 {
 	{"swapUv", offsetof(decoderFF_param_t, swapUv), "bool", ADM_param_bool},
-	{"showMv", offsetof(decoderFF_param_t, showMv), "bool", ADM_param_bool},
 	{NULL, 0, NULL}
 };
 
@@ -164,6 +164,10 @@ decoderFF::decoderFF (uint32_t w, uint32_t h,uint32_t fcc, uint32_t extraDataLen
 	resetConfiguration();
 
   hurryUp=false;
+  _drain=false;
+  _done=false;
+  _keepFeeding=false;
+  _endOfStream=false;
   _setBpp=false;
   _setFcc=false;
   codecId = 0;
@@ -189,8 +193,8 @@ decoderFF::decoderFF (uint32_t w, uint32_t h,uint32_t fcc, uint32_t extraDataLen
   if(extraDataLen)
     {
             _extraDataLen=(int)extraDataLen;
-            _extraDataCopy=new uint8_t[extraDataLen+FF_INPUT_BUFFER_PADDING_SIZE];
-            memset(_extraDataCopy,0,extraDataLen+FF_INPUT_BUFFER_PADDING_SIZE);
+            _extraDataCopy=new uint8_t[extraDataLen+AV_INPUT_BUFFER_PADDING_SIZE];
+            memset(_extraDataCopy,0,extraDataLen+AV_INPUT_BUFFER_PADDING_SIZE);
             memcpy(_extraDataCopy,extraData,extraDataLen);
     }
    hwDecoder=NULL;
@@ -261,6 +265,9 @@ uint32_t decoderFF::admFrameTypeFromLav (AVFrame *pic)
                 if (pic->key_frame)
                         aprintf ("\n But keyframe is set\n");
                 break;
+        case AV_PICTURE_TYPE_NONE:
+                if(codecId == AV_CODEC_ID_HUFFYUV || codecId == AV_CODEC_ID_FFVHUFF)
+                    SET(AVI_KEY_FRAME)
         default:
                 break;
     }
@@ -309,152 +316,170 @@ bool    decoderFF::flush(void)
 {
     if(_context)
         avcodec_flush_buffers(_context);
+    _drain=false;
+    _done=false;
     return true;
 }
+
+/**
+    \fn decodeErrorHandler
+    \brief Evaluate return value of avcodec_receive_frame
+*/
+bool decoderFF::decodeErrorHandler(int code)
+{
+    if(code<0)
+    {
+        switch(code)
+        {
+            case AVERROR_EOF:
+                ADM_warning("[lavc] End of video stream reached\n");
+                _keepFeeding=false;
+                _endOfStream=true;
+                flush();
+                return false;
+            case AVERROR(EAGAIN):
+#ifdef ADM_DEBUG
+                ADM_info("[lavc] The decoder expects more input before output can be produced\n");
+#endif
+                _keepFeeding=true;
+                return false;
+            case AVERROR(EINVAL):
+                ADM_error("[lavc] Codec not opened\n");
+                return false;
+            default:
+            {
+                char er[2048]={0};
+                av_make_error_string(er, sizeof(er)-1, code);
+                ADM_warning("Error %d in lavcodec (%s)\n",code,er);
+                return false;
+            }
+        }
+    }
+    _keepFeeding=false;
+    _endOfStream=false;
+    return true;
+}
+
 /**
     \fn uncompress
     \brief Actually decode an image
 */
 bool   decoderFF::uncompress (ADMCompressedImage * in, ADMImage * out)
 {
-  int got_picture = 0;
-  uint8_t *oBuff[3];
   int ret = 0;
   out->_noPicture = 0;
   if(hwDecoder)
         return hwDecoder->uncompress(in,out);
  
-  
-  if (decoderFF_params.showMv)
-    {
-      _context->debug_mv |= FF_SHOW;
-      _context->debug |= 0;	//FF_DEBUG_VIS_MB_TYPE;
-    }
-  else
-    {
-      _context->debug_mv &= ~FF_SHOW;
-      _context->debug &= ~(FF_DEBUG_VIS_MB_TYPE + FF_DEBUG_VIS_QP);
-    }
-
   //printf("Frame size : %d\n",in->dataLength);
 
-  if (in->dataLength == 0 && !_allowNull)	// Null frame, silently skipped
+    if (in->dataLength == 0 && !_allowNull) // Null frame, silently skipped
     {
-
-      printf ("[Codec] null frame\n");
-        // search the last image
-        if (_context->coded_frame &&
-            _context->coded_frame->data &&
-            _context->coded_frame->data[0]
-            )
-          {
-            printf("[Codec] Cloning older pic\n");
-            clonePic (_context->coded_frame, out);
-            out->Pts=ADM_COMPRESSED_NO_PTS;
-          }
-        else
-            {
-                out->_noPicture = 1;
-                out->Pts=ADM_COMPRESSED_NO_PTS;
-                printf("[Codec] No Picture\n");
-            }
-          return 1;
+        printf ("[Codec] null frame\n");
+        out->_noPicture = 1;
+        out->Pts=ADM_COMPRESSED_NO_PTS;
+        printf("[Codec] No Picture\n");
+        return 1;
     }
-   // Put a safe value....
-   out->Pts=in->demuxerPts;
+    // Put a safe value....
+    out->Pts=in->demuxerPts;
     _context->reordered_opaque=in->demuxerPts;
   //_frame.opaque=(void *)out->Pts;
   //printf("Incoming Pts :%"PRId64"\n",out->Pts);
-  AVPacket pkt;
-  av_init_packet(&pkt);
-  pkt.data=in->data;
-  pkt.size=in->dataLength;
-  if(in->flags&AVI_KEY_FRAME)
-    pkt.flags=AV_PKT_FLAG_KEY;
-  else
-    pkt.flags=0;
+    if(_drain)
+    {
+        if(!_done)
+        {
+            avcodec_send_packet(_context, NULL);
+            _done=true;
+        }
+    }else
+    {
+        AVPacket pkt;
+        av_init_packet(&pkt);
+        pkt.data=in->data;
+        pkt.size=in->dataLength;
+        if(in->flags&AVI_KEY_FRAME)
+            pkt.flags=AV_PKT_FLAG_KEY;
+        else
+            pkt.flags=0;
 
-  ret = avcodec_decode_video2 (_context, _frame, &got_picture, &pkt);
-  if(!bFramePossible())
-  {
-    // No delay, the value is sure, no need to hide it in opaque
-    _context->reordered_opaque=(int64_t)in->demuxerPts;
-  }
-  out->_qStride = 0;		//Default = no quant
-  if (0 > ret && !hurryUp)
-    {
-      printf ("\n[lavc] error in lavcodec decoder!\n");
-      printf ("[lavc] Err: %d, size :%d\n", ret, in->dataLength);
-      return 0;
+        avcodec_send_packet(_context, &pkt);
     }
-  if (!got_picture && !hurryUp)
-    {
-      // Some encoder code a vop header with the
-      // vop flag set to 0
-      // it is meant to mean frame skipped but very dubious
-      if (in->dataLength <= 8)
-        if(codecId == AV_CODEC_ID_MPEG4||codecId==AV_CODEC_ID_FRAPS)
-	{
-	  printf ("[lavc] Probably pseudo black frame...\n");
-	  out->_Qp = 2;
-	  out->flags = 0;	// assume P ?
-          if(_context->coded_frame)
-            clonePic (_context->coded_frame, out);
-          else
-             out->_noPicture = 1;
-          out->Pts=ADM_NO_PTS; // not sure
-	  return 1;
-	}
-      // allow null means we allow null frame in and so potentially
-      // have no frame out for a time
-      // in that case silently fill with black and returns it as KF
-      if (_allowNull)
-	{
-	  out->flags = AVI_KEY_FRAME;
-	  if (!_refCopy)
-	    {
-            out->blacken();
-	    }
-	  else
-	    {
-	      out->_noPicture = 1;
-	    }
-	  printf ("\n[lavc] ignoring got pict ==0\n");
-	  return 1;
 
-	}
-#if 0
-      printf ("[lavc] Err: %d, size: %d\n", ret, in->dataLength);
-      printf ("\n[lavc] error in FFMP43/mpeg4!: got picture\n");
-#endif
-      //GUI_Alert("Please retry with misc->Turbo off");
-      //return 1;
-      return 0;
-    }
-  if (hurryUp)
+    ret = avcodec_receive_frame(_context, _frame);
+
+    out->_qStride = 0; // Default = no quant
+
+    if (ret && !hurryUp)
     {
-      out->flags = frameType ();
-      return 1;
+        // Some encoder code a vop header with the
+        // vop flag set to 0
+        // it is meant to mean frame skipped but very dubious
+#define MPEG4_EMPTY_FRAME_THRESHOLD 19 // MAX_NVOP_SIZE
+#define FRAPS_EMPTY_FRAME_THRESHOLD  8 // assumption
+        if ((in->dataLength <= MPEG4_EMPTY_FRAME_THRESHOLD && codecId == AV_CODEC_ID_MPEG4) ||
+            (in->dataLength <= FRAPS_EMPTY_FRAME_THRESHOLD && codecId == AV_CODEC_ID_MPEG4))
+        {
+            printf ("[lavc] Probably placeholder frame (data length: %u)\n",in->dataLength);
+            out->_Qp = 2;
+            out->Pts=ADM_NO_PTS; // not sure
+            out->_noPicture = 1;
+            return true;
+        }
+        // allow null means we allow null frame in and so potentially
+        // have no frame out for a time
+        // in that case silently fill with black and returns it as KF
+        if (_allowNull)
+        {
+            out->flags = AVI_KEY_FRAME;
+	    if (!_refCopy)
+                out->blacken();
+	    else
+	        out->_noPicture = 1;
+            printf ("\n[lavc] ignoring that we got no picture\n");
+	    return 1;
+        }
     }
+    if (hurryUp)
+    {
+        out->flags = frameType ();
+        return 1;
+    }
+
+    if(!decodeErrorHandler(ret))
+        return false;
+
+    if(!bFramePossible())
+    {
+        // No delay, the value is sure, no need to hide it in opaque
+        _frame->reordered_opaque=(int64_t)in->demuxerPts;
+    }
+
   // We have an image....
   switch (_context->pix_fmt)
     {
     case AV_PIX_FMT_YUV411P:
       out->_colorspace = ADM_COLOR_YUV411;
+      aprintf("colorspace is AV_PIX_FMT_YUV411P --> ADM_COLOR_YUV411\n");
       break;
     case AV_PIX_FMT_YUYV422:
       out->_colorspace = ADM_COLOR_YUV422;
+      aprintf("colorspace is AV_PIX_FMT_YUYV422 --> ADM_COLOR_YUV422\n");
       break;
     case AV_PIX_FMT_YUV422P:
     case AV_PIX_FMT_YUVJ422P:
       out->_colorspace = ADM_COLOR_YUV422P;
+      aprintf("colorspace is AV_PIX_FMT_YUV422P or AV_PIX_FMT_YUVJ422P --> ADM_COLOR_YUV422P\n");
       break;
     case AV_PIX_FMT_GRAY8:
-       out->_colorspace = ADM_COLOR_Y8;
-       break;
+      out->_colorspace = ADM_COLOR_Y8;
+      aprintf("colorspace is AV_PIX_FMT_GRAY8 --> ADM_COLOR_Y8\n");
+      break;
     case AV_PIX_FMT_YUV444P:
     case AV_PIX_FMT_YUVJ444P:
       out->_colorspace = ADM_COLOR_YUV444;
+      aprintf("colorspace is AV_PIX_FMT_YUV444P or AV_PIX_FMT_YUVJ444P --> ADM_COLOR_YUV444\n");
       break;
     case AV_PIX_FMT_YUV420P:
     case AV_PIX_FMT_YUVJ420P:
@@ -463,20 +488,28 @@ bool   decoderFF::uncompress (ADMCompressedImage * in, ADMImage * out)
       // In that case depending on swap u/v
       // we do it or not
       out->_colorspace = ADM_COLOR_YV12;
+      aprintf("colorspace is AV_PIX_FMT_YUV420P, AV_PIX_FMT_YUVJ420P or AV_PIX_FMT_YUVA420P --> ADM_COLOR_YV12\n");
+      break;
+    case AV_PIX_FMT_BGR24:
+      out->_colorspace = ADM_COLOR_BGR24;
+      aprintf("colorspace is AV_PIX_FMT_BGR24 --> ADM_COLOR_BGR24\n");
+      break;
+    case AV_PIX_FMT_RGB24:
+      out->_colorspace = ADM_COLOR_RGB24;
+      aprintf("colorspace is AV_PIX_FMT_RGB24 --> ADM_COLOR_RGB24\n");
       break;
-   case AV_PIX_FMT_BGR24:
-   case AV_PIX_FMT_RGB24:
-	  out->_colorspace = ADM_COLOR_BGR24;
-	  break;
     case AV_PIX_FMT_BGR0:
     case AV_PIX_FMT_BGRA:
       out->_colorspace = ADM_COLOR_BGR32A;
+      aprintf("colorspace is AV_PIX_FMT_BGR0 or AV_PIX_FMT_BGRA --> ADM_COLOR_BGR32A\n");
       break;
     case AV_PIX_FMT_RGBA: // ???PIX_FMT_RGBA32:
       out->_colorspace = ADM_COLOR_RGB32A;
+      aprintf("colorspace is AV_PIX_FMT_RGBA --> ADM_COLOR_RGB32A\n");
       break;
     case AV_PIX_FMT_RGB555:
       out->_colorspace = ADM_COLOR_RGB555;
+      aprintf("colorspace is AV_PIX_FMT_RGB555 --> ADM_COLOR_RGB555\n");
       break;
 #if 0
     case AV_PIX_FMT_VDPAU_MPEG1:
@@ -500,6 +533,9 @@ bool   decoderFF::uncompress (ADMCompressedImage * in, ADMImage * out)
   case AV_PIX_FMT_YUV444P10LE:
         out->_colorspace=ADM_COLOR_YUV444_10BITS;
         break;      
+  case AV_PIX_FMT_YUV422P10LE:
+        out->_colorspace=ADM_COLOR_YUV422_10BITS;
+        break;
   case   AV_PIX_FMT_P010LE:      
         out->_colorspace= ADM_COLOR_NV12_10BITS;
         break;
@@ -538,16 +574,12 @@ decoderFF (w, h,fcc,extraDataLen,extraData,bpp)
   WRAP_Open (AV_CODEC_ID_MPEG4);
   
 }
-bool decoderFFMpeg4::uncompress (ADMCompressedImage * in, ADMImage * out)
+bool decoderFFMpeg4::uncompress(ADMCompressedImage *in, ADMImage *out)
 {
     // For pseudo startcode
-    if(in->dataLength)
-    {
-        in->data[in->dataLength]=0;
-        in->data[in->dataLength+1]=0;
-    }
+    if(!_drain && in->dataLength && in->dataLength < ADM_COMPRESSED_MAX_DATA_LENGTH - 2)
+        memset(in->data+in->dataLength,0,2);
     return decoderFF::uncompress(in,out);
-
 }
 //************************************
 decoderFFDV::decoderFFDV (uint32_t w, uint32_t h,uint32_t fcc, uint32_t extraDataLen, uint8_t *extraData,uint32_t bpp):
@@ -558,6 +590,25 @@ decoderFF (w, h,fcc,extraDataLen,extraData,bpp)
   
 
 }
+//************************************
+decoderFFficv::decoderFFficv(uint32_t w, uint32_t h, uint32_t fcc, uint32_t extraDataLen, uint8_t *extraData, uint32_t bpp) : decoderFF(w,h,fcc,extraDataLen,extraData,bpp)
+{
+    WRAP_Open (AV_CODEC_ID_FIC)
+}
+/**
+    \fn uncompress
+*/
+bool decoderFFficv::uncompress(ADMCompressedImage *in, ADMImage *out)
+{
+    if(decoderFF::uncompress(in,out))
+    {
+        if(in->flags&AVI_KEY_FRAME)
+            out->flags=AVI_KEY_FRAME;
+        return true;
+    }
+    return false;
+}
+//************************************
 decoderFFMpeg12::decoderFFMpeg12 (uint32_t w, uint32_t h,uint32_t fcc, uint32_t extraDataLen, uint8_t *extraData,uint32_t bpp):
                 decoderFF (w, h,fcc,extraDataLen,extraData,bpp)
 {
@@ -587,7 +638,10 @@ decoderFFH264::decoderFFH264 (uint32_t w, uint32_t h,uint32_t fcc, uint32_t extr
   decoderMultiThread ();
   ADM_info ("[lavc] Initializing H264 decoder with %d extradata\n", (int)extraDataLen);
   WRAP_Open(AV_CODEC_ID_H264);
-  
+#ifdef ADM_DEBUG
+  //_context->debug |= FF_DEBUG_MMCO;
+  //_context->debug |= FF_DEBUG_PICT_INFO;
+#endif
 }
 //*********************
 extern "C" {int av_getAVCStreamInfo(AVCodecContext *avctx, uint32_t  *nalSize, uint32_t *isAvc);}
@@ -654,10 +708,7 @@ void ADM_lavInit(void)
     avcodec_register_all();
     av_log_set_callback(adm_lavLogCallback);
     av_setFatalHandler(ffFatalError);
-#ifdef ADM_DEBUG
-    av_log_set_level(AV_LOG_DEBUG);
-#endif
-
+    av_log_set_level(LAV_VERBOSITY_LEVEL);
 }
 void adm_lavLogCallback(void  *instance, int level, const char* fmt, va_list list)
 {
@@ -665,7 +716,7 @@ void adm_lavLogCallback(void  *instance, int level, const char* fmt, va_list lis
     char buf[256];
 
     vsnprintf(buf, sizeof(buf), fmt, list);
-    if(level<=AV_LOG_INFO)
+    if(buf[0] != '\0' && level<=LAV_VERBOSITY_LEVEL)
         ADM_info("[lavc] %s",buf);
 }
 
