@@ -142,9 +142,9 @@ bool ADM_coreVideoEncoder::getRealPtsFromInternal(uint64_t val,uint64_t *dts,uin
             return true;
         }
     }
-    ADM_warning("Cannot find PTS : %s \n",ADM_us2plain(val));
+    ADM_warning("Cannot find PTS : %" PRIu64" \n",val);
     for(int i=0;i<n;i++) 
-        ADM_warning("%d : %s\n",i,ADM_us2plain(mapper[i].internalTS));
+        ADM_warning("%d : %" PRIu64", %s\n",i,mapper[i].internalTS,ADM_us2plain(mapper[i].realTS));
     ADM_assert(0);
     return false;
 
@@ -196,11 +196,11 @@ static bool getFileNameAndExt(const std::string &input, std::string &output)
     size_t lastSlash;
 
      lastSlash=s.find_last_of("/");
-     if(lastSlash!=-1)
+     if(lastSlash!=std::string::npos)
         s.replace(0,lastSlash+1,std::string(""));
 #ifdef _WIN32
      size_t lastBackSlash=s.find_last_of("\\");
-     if(lastBackSlash!=-1)
+     if(lastBackSlash!=std::string::npos)
         s.replace(0,lastBackSlash+1,std::string(""));
 #endif
     ADM_info("Stripping : %s => %s\n",input.c_str(),s.c_str());
@@ -265,7 +265,7 @@ bool ADM_listFile(const std::string& path,const std::string& extension,vector <s
         getFileNameAndExt(s,file);
         lastDot=file.find_last_of('.');
         // Remove extension
-        if(lastDot!=-1)
+        if(lastDot!=std::string::npos)
             file.replace(lastDot,file.size(),std::string(""));
         listOut.push_back(file);
     }
