@@ -16,6 +16,7 @@
  *                                                                         *
  ***************************************************************************/
 #include "ADM_default.h"
+#include "ADM_vidMisc.h"
 #include "ADM_coreVideoEncoderFFmpeg.h"
 #include "prefs.h"
 #include "ADM_coreUtils.h"
@@ -79,7 +80,7 @@ _hasSettings=false;
      else
             encoderDelay=0;
     ADM_info("[Lavcodec] Using a video encoder delay of %d ms\n",(int)(encoderDelay/1000));
-
+    lastLavPts=0;
 }
 /**
     \fn ADM_coreVideoEncoderFFmpeg
@@ -146,27 +147,14 @@ bool             ADM_coreVideoEncoderFFmpeg::prolog(ADMImage *img)
                                 _frame->linesize[2] = 0;//w >> 1;
                                 _frame->format=AV_PIX_FMT_RGB32;
                                 _context->pix_fmt =AV_PIX_FMT_RGB32;break;
+        case ADM_COLOR_RGB24:   _frame->linesize[0] = w*3;
+                                _frame->linesize[1] = 0;
+                                _frame->linesize[2] = 0;
+                                _frame->format = AV_PIX_FMT_RGB24;
+                                _context->pix_fmt = AV_PIX_FMT_RGB24;break;
         default: ADM_assert(0);
 
     }
-
-    // Eval fps
-    uint64_t f=source->getInfo()->frameIncrement;
-    // Let's put 100 us as time  base
-
-#ifdef TIME_TENTH_MILLISEC
-    _context->time_base.num=1;
-    _context->time_base.den=10000LL;
-#else
-
-    int n,d;
-
-    usSecondsToFrac(f,&n,&d);
- //   printf("[ff] Converted a time increment of %d ms to %d /%d seconds\n",f/1000,n,d);
-    _context->time_base.num=n;
-    _context->time_base.den=d;
-#endif
-    timeScaler=1000000.*av_q2d(_context->time_base); // Optimize, can be computed once
     return true;
 }
 /**
@@ -214,15 +202,23 @@ bool             ADM_coreVideoEncoderFFmpeg::preEncode(void)
     p+=getEncoderDelay();
     _frame->pts= timingToLav(p);    //
     if(!_frame->pts) _frame->pts=AV_NOPTS_VALUE;
+    if(_frame->pts!=AV_NOPTS_VALUE && lastLavPts!=AV_NOPTS_VALUE && _frame->pts==lastLavPts)
+    {
+        ADM_warning("Lav PTS collision at frame %" PRIu32", lav PTS=%" PRId64", time %s\n",nb,_frame->pts,ADM_us2plain(p));
+        _frame->pts++;
+    }
+    lastLavPts=_frame->pts;
 
     ADM_timeMapping map; // Store real PTS <->lav value mapping
     map.realTS=p;
     map.internalTS=_frame->pts;
     mapper.push_back(map);
 
-    aprintf("Codec> incoming pts=%" PRIu64"\n",image->Pts);
+    aprintf("Codec> real pts=%" PRIu64", internal pts=%" PRId64"\n",p,_frame->pts);
     //printf("--->>[PTS] :%"PRIu64", raw %"PRIu64" num:%"PRIu32" den:%"PRIu32"\n",_frame->pts,image->Pts,_context->time_base.num,_context->time_base.den);
     //
+    int w=getWidth();
+    int h=getHeight();
     switch(targetColorSpace)
     {
         case ADM_COLOR_YV12:
@@ -233,9 +229,6 @@ bool             ADM_coreVideoEncoderFFmpeg::preEncode(void)
 
         case ADM_COLOR_YUV422P:
         {
-              int w=getWidth();
-              int h=getHeight();
-
                 if(!colorSpace->convertImage(image,rgbByteBuffer.at(0)))
                 {
                     printf("[ADM_jpegEncoder::encode] Colorconversion failed\n");
@@ -247,7 +240,25 @@ bool             ADM_coreVideoEncoderFFmpeg::preEncode(void)
                 break;
         }
         case ADM_COLOR_RGB32A:
-                if(!colorSpace->convertImage(image,rgbByteBuffer.at(0)))
+        case ADM_COLOR_RGB24:
+        {
+                ADMImageRefWrittable swap(w,h);
+                int strides[3];
+                uint8_t *ptr[3];
+
+                image->GetPitches(strides);
+                image->GetReadPlanes(ptr);
+
+                swap._planes[0] = ptr[0];
+                swap._planeStride[0] = strides[0];
+
+                swap._planes[1] = ptr[2];
+                swap._planeStride[1] = strides[2];
+
+                swap._planes[2] = ptr[1];
+                swap._planeStride[2] = strides[1];
+
+                if(!colorSpace->convertImage(&swap,rgbByteBuffer.at(0)))
                 {
                     printf("[ADM_jpegEncoder::encode] Colorconversion failed\n");
                     return false;
@@ -256,6 +267,7 @@ bool             ADM_coreVideoEncoderFFmpeg::preEncode(void)
                 _frame->data[2] = NULL;
                 _frame->data[1] = NULL;
                 break;
+        }
         default:
                 ADM_assert(0);
     }
@@ -294,6 +306,8 @@ int              ADM_coreVideoEncoderFFmpeg::encodeWrapper(AVFrame *in,ADMBitstr
             ADM_warning("Encoder produced no data\n");
             pkt.size=0;
         }
+        lavPtsFromPacket=pkt.pts; // some encoders don't set pts in coded_frame
+        packetFlags=pkt.flags;
         return pkt.size;            
 }
 /**
@@ -313,9 +327,20 @@ bool ADM_coreVideoEncoderFFmpeg::setupInternal(AVCodec *codec)
     if(_globalHeader)
     {
                 ADM_info("Codec configured to use global header\n");
-                _context->flags|=CODEC_FLAG_GLOBAL_HEADER;
+                _context->flags|=AV_CODEC_FLAG_GLOBAL_HEADER;
     }
    prolog(image);
+    uint64_t inc=source->getInfo()->frameIncrement;
+#ifdef TIME_TENTH_MILLISEC
+    _context->time_base.num=1;
+    _context->time_base.den=10000LL;
+#else
+    int n,d;
+    usSecondsToFrac(inc,&n,&d);
+    _context->time_base.num=n;
+    _context->time_base.den=d;
+#endif
+    timeScaler=1000000.*av_q2d(_context->time_base);
    printf("[ff] Time base %d/%d\n", _context->time_base.num,_context->time_base.den);
    if(_hasSettings && LAVS(MultiThreaded))
     {
@@ -397,32 +422,42 @@ bool             ADM_coreVideoEncoderFFmpeg::getExtraData(uint32_t *l,uint8_t **
 */
 bool ADM_coreVideoEncoderFFmpeg::loadStatFile(const char *file)
 {
-  printf("[FFmpeg] Loading stat file :%s\n",file);
-  FILE *_statfile = ADM_fopen (file, "rb");
-  int statSize;
+    ADM_info("Loading stat file %s\n",file);
+    FILE *_statfile = ADM_fopen (file, "rb");
 
-  if (!_statfile)
+    if (!_statfile)
     {
-      printf ("[ffmpeg] internal file does not exists ?\n");
-      return false;
+        ADM_error ("Cannot open stat file. Does it exist?\n");
+        return false;
     }
 
-  fseek (_statfile, 0, SEEK_END);
-  statSize = ftello (_statfile);
-  fseek (_statfile, 0, SEEK_SET);
-  _context->stats_in = (char *) av_malloc(statSize+1);
-  _context->stats_in[statSize] = 0;
-  fread (_context->stats_in, statSize, 1, _statfile);
-  fclose(_statfile);
-
+    fseek (_statfile, 0, SEEK_END);
+    uint64_t statSize = ftello (_statfile);
+    if (statSize >= INT_MAX - 32) // enforce the av_malloc limit
+    {
+        ADM_error ("Stat file too large.\n");
+        fclose(_statfile);
+        return false;
+    }
+    fseek (_statfile, 0, SEEK_SET);
+    _context->stats_in = (char *) av_malloc(statSize+1);
+    _context->stats_in[statSize] = 0;
+    if (!fread (_context->stats_in, statSize, 1, _statfile))
+    {
+        ADM_error ("Cannot read stat file.\n");
+        fclose(_statfile);
+        return false;
+    }
+    fclose(_statfile);
 
     int i;
     char *p=_context->stats_in;
-   for(i=-1; p; i++){
-            p= strchr(p+1, ';');
-        }
-  printf("[FFmpeg] stat file loaded ok, %d frames found\n",i);
-  return true;
+    for(i=-1; p; i++)
+    {
+        p= strchr(p+1, ';');
+    }
+    ADM_info ("Stat file loaded ok, %d frames found.\n",i);
+    return true;
 }
 /**
         \fn postEncode
@@ -435,13 +470,14 @@ bool ADM_coreVideoEncoderFFmpeg::postEncode(ADMBitstream *out, uint32_t size)
     if(_context->coded_frame)
     {
         pict_type=_context->coded_frame->pict_type;
-        keyframe=_context->coded_frame->key_frame;
     }else
     {
         out->len=0;
         ADM_warning("No picture...\n");
         return false;
     }
+    if(packetFlags & AV_PKT_FLAG_KEY)
+        keyframe=true;
     aprintf("[ffMpeg4] Out Quant :%d, pic type %d keyf %d %p\n",out->out_quantizer,pict_type,keyframe,_context->coded_frame);
     out->len=size;
     out->flags=0;
@@ -467,8 +503,10 @@ bool ADM_coreVideoEncoderFFmpeg::postEncode(ADMBitstream *out, uint32_t size)
 
     } else
     {
-        if(!getRealPtsFromInternal(_context->coded_frame->pts,&(out->dts),&(out->pts)))
-                return false;
+        if(lavPtsFromPacket==AV_NOPTS_VALUE)
+            return false;
+        if(!getRealPtsFromInternal(lavPtsFromPacket,&(out->dts),&(out->pts)))
+            return false;
     }
     // update lastDts
     lastDts=out->dts;
@@ -503,7 +541,6 @@ bool ADM_coreVideoEncoderFFmpeg::presetContext(FFcodecSettings *set)
 
 #define SETX(x) _context->x=set->lavcSettings.x; printf("[LAVCODEC]"#x" : %d\n",set->lavcSettings.x);
 
-      SETX (me_method);
       SETX (qmin);
       SETX (qmax);
       SETX (max_b_frames);
@@ -526,10 +563,6 @@ bool ADM_coreVideoEncoderFFmpeg::presetContext(FFcodecSettings *set)
 #undef SETX
 #undef SETX_COND
 
-#define SETX(x) if(set->lavcSettings.x){ _context->flags|=CODEC_FLAG##x;printf("[LAVCODEC]"#x" is set\n");}
-      SETX (_GMC);
-
-
     switch (set->lavcSettings.mb_eval)
 	{
         case 0:
@@ -545,6 +578,7 @@ bool ADM_coreVideoEncoderFFmpeg::presetContext(FFcodecSettings *set)
           ADM_assert (0);
 	}
 
+#define SETX(x) if(set->lavcSettings.x){ _context->flags|=AV_CODEC_FLAG##x;printf("[LAVCODEC]"#x" is set\n");}
       SETX (_4MV);
       SETX (_QPEL);
       if(set->lavcSettings._TRELLIS_QUANT) _context->trellis=1;
@@ -561,7 +595,6 @@ bool ADM_coreVideoEncoderFFmpeg::presetContext(FFcodecSettings *set)
 #undef SETX
   _context->bit_rate_tolerance = 8000000;
   _context->b_quant_factor = 1.25;
-  _context->rc_strategy = 2;
   _context->b_frame_strategy = 0;
   _context->b_quant_offset = 1.25;
   _context->rtp_payload_size = 0;
@@ -582,6 +615,16 @@ bool ADM_coreVideoEncoderFFmpeg::presetContext(FFcodecSettings *set)
   _context->p_masking = 0.0;
 
   // Set frame rate den/num
+  uint64_t inc=source->getInfo()->frameIncrement;
+#ifdef TIME_TENTH_MILLISEC
+  _context->time_base.num=1;
+  _context->time_base.den=10000LL;
+#else
+  int n,d;
+  usSecondsToFrac(inc,&n,&d);
+  _context->time_base.num=n;
+  _context->time_base.den=d;
+#endif
   prolog(image);
   return true;
 }
@@ -628,7 +671,7 @@ bool ADM_coreVideoEncoderFFmpeg::setupPass(void)
         {
                 case 1:
                     printf("[ffMpeg4] Setup-ing Pass 1\n");
-                    _context->flags |= CODEC_FLAG_PASS1;
+                    _context->flags |= AV_CODEC_FLAG_PASS1;
                     // Open stat file
                     statFile=ADM_fopen(statFileName,"wt");
                     if(!statFile)
@@ -639,7 +682,7 @@ bool ADM_coreVideoEncoderFFmpeg::setupPass(void)
                     break;
                 case 2:
                     printf("[ffMpeg4] Setup-ing Pass 2\n");
-                    _context->flags |= CODEC_FLAG_PASS2;
+                    _context->flags |= AV_CODEC_FLAG_PASS2;
                     if(false==loadStatFile(statFileName))
                     {
                         printf("[ffmpeg4] Cannot load stat file\n");
