@@ -18,14 +18,12 @@
 
 extern "C" {
 #include "libavcodec/avcodec.h"
-#include "libavcodec/audioconvert.h"
 }
 
 #include "ADM_ad_plugin.h"
 #include "fourcc.h"
 #include "ADM_audioXiphUtils.h"
 
-#define SCRATCH_PAD_SIZE (100*1000*2)
 #define ADMWA_BUF (4*1024*16) // 64 kB internal
 /**
  * \class ADM_AudiocoderLavcodec
@@ -41,10 +39,10 @@ protected:
                 ADM_outputFlavor        outputFlavor;
                 AVCodecContext          *_context;
                 AVFrame                 *_frame;
+                uint8_t                 *_paddedExtraData;
                 uint8_t    _buffer[ ADMWA_BUF];
                 uint32_t   _tail,_head;
                 uint32_t   _blockalign;
-                uint8_t scratchPad[SCRATCH_PAD_SIZE];
     uint32_t    channels;
     bool        decodeToS16(float **outptr,uint32_t *nbOut);
     bool        decodeToFloat(float **outptr,uint32_t *nbOut);
@@ -53,6 +51,7 @@ protected:
     bool        decodeToS32(float **outptr,uint32_t *nbOut);
     bool        decodeToFloatPlanarStereo(float **outptr,uint32_t *nbOut);
     uint32_t    outputFrequency;
+    bool        sbrChecked;
 public:
                     ADM_AudiocoderLavcodec(uint32_t fourcc, WAVHeader *info, uint32_t l, uint8_t *d);
     virtual         ~ADM_AudiocoderLavcodec() ;
@@ -77,8 +76,8 @@ static  ad_supportedFormat Formats[]={
         {WAV_MP2,AD_MEDIUM_QUAL},
         {WAV_FLAC,AD_MEDIUM_QUAL},
         {WAV_AC3,AD_LOW_QUAL},   // liba52 preferred ???
-        {WAV_AAC,AD_LOW_QUAL},   // libfaad preferred ???
-        {0x706D,AD_LOW_QUAL},
+        {WAV_AAC,AD_HIGH_QUAL},
+        {0x706D,AD_HIGH_QUAL},
         {WAV_EAC3,AD_MEDIUM_QUAL},
         {WAV_OGG_VORBIS,AD_HIGH_QUAL},
 };
@@ -95,6 +94,7 @@ DECLARE_AUDIO_DECODER(ADM_AudiocoderLavcodec,						// Class
 */
    bool ADM_AudiocoderLavcodec::resetAfterSeek( void )
    {
+            avcodec_flush_buffers(_context);
             _tail=_head=0;
             return 1;
    };
@@ -108,11 +108,13 @@ DECLARE_AUDIO_DECODER(ADM_AudiocoderLavcodec,						// Class
     ADM_info(" [ADM_AD_LAV] Using decoder for type 0x%x\n",info->encoding);
     ADM_info(" [ADM_AD_LAV] #of channels %d\n",info->channels);
     _tail=_head=0;
+    _paddedExtraData=NULL;
     channels=info->channels;
     _blockalign=0;
     _frame=av_frame_alloc();
     AVCodecID codecID = AV_CODEC_ID_NONE;
-    outputFrequency=info->frequency;    
+    outputFrequency=info->frequency;
+    sbrChecked=false;
     switch(fourcc)
     {
       case WAV_WMAPRO:
@@ -183,20 +185,24 @@ DECLARE_AUDIO_DECODER(ADM_AudiocoderLavcodec,						// Class
     _context->bit_rate = info->byterate*8;
     _context->sample_fmt=AV_SAMPLE_FMT_FLT;
     _context->request_sample_fmt=AV_SAMPLE_FMT_FLT;
-    
+
     if(fourcc==WAV_OGG_VORBIS)
     {
         // Need to translate from adm to xiph
         int xiphLen=(int)l+(l/255)+4+5;
-        uint8_t *xiph=new uint8_t[xiphLen];
+        uint8_t *xiph=new uint8_t[xiphLen+AV_INPUT_BUFFER_PADDING_SIZE];
+        memset(xiph,0,xiphLen+AV_INPUT_BUFFER_PADDING_SIZE);
         xiphLen=ADMXiph::admExtraData2xiph(l,d,xiph);
-        _context->extradata=xiph;
-        _context->extradata_size=xiphLen;
-    }else
+        _paddedExtraData=xiph;
+        l=xiphLen;
+    }else if(l)
     {
-        _context->extradata=(uint8_t *)d;
-        _context->extradata_size=(int)l;    
+        _paddedExtraData=new uint8_t[l+AV_INPUT_BUFFER_PADDING_SIZE];
+        memset(_paddedExtraData,0,l+AV_INPUT_BUFFER_PADDING_SIZE);
+        memcpy(_paddedExtraData,d,l);
     }
+    _context->extradata=_paddedExtraData;
+    _context->extradata_size=l;
 
     if (!_blockalign) 
     {
@@ -257,23 +263,34 @@ DECLARE_AUDIO_DECODER(ADM_AudiocoderLavcodec,						// Class
         _blockalign=378;
       }
     }
-    ADM_info("[ADM_ad_lav] init successful (blockalign %d), channels=%d\n",info->blockalign,_context->channels);
+    ADM_info("[ADM_ad_lav] init successful (blockalign %d), channels=%d\n",_blockalign,_context->channels);
     if(_context->sample_rate!=outputFrequency)
     {
         ADM_warning("Output frequency does not match input frequency (SBR ?) : %d / %d\n",
             _context->sample_rate,outputFrequency);
         outputFrequency=_context->sample_rate;
     }
+    if(_context->channels!=info->channels)
+    {
+        ADM_warning("Decoder and demuxer disagree about # of channels: %d / %d\n",_context->channels,info->channels);
+        if(updateChannels(_context->channels))
+            channels=_context->channels;
+    }
 }
 /**
     \fn dtor
 */
  ADM_AudiocoderLavcodec::~ADM_AudiocoderLavcodec()
- {
-        avcodec_close(_context);
-        av_free(_context);
-        _context=NULL;
-        av_frame_free(&_frame);
+{
+    avcodec_close(_context);
+    av_free(_context);
+    _context=NULL;
+    av_frame_free(&_frame);
+    if(_paddedExtraData)
+    {
+        delete [] _paddedExtraData;
+       _paddedExtraData=NULL;
+    }
 }
 /**
     \fn decodeToS16
@@ -418,64 +435,107 @@ bool ADM_AudiocoderLavcodec::decodeToFloatPlanar(float **outptr,uint32_t *nbOut)
 */
 uint8_t ADM_AudiocoderLavcodec::run(uint8_t *inptr, uint32_t nbIn, float *outptr, uint32_t *nbOut)
 {
+    *nbOut=0;
+    // Shrink
+    if(_head && (_tail+nbIn)*3>ADMWA_BUF*2)
+    {
+        memmove(_buffer,_buffer+_head,_tail-_head);
+        _tail-=_head;
+        _head=0;
+    }
+    //
+    ADM_assert(nbIn+_tail<ADMWA_BUF);
+    memcpy(_buffer+_tail,inptr,nbIn);
+    _tail+=nbIn;
 
-        *nbOut=0;
-        // Shrink
-        if(_head && (_tail+nbIn)*3>ADMWA_BUF*2)
-        {
-            memmove(_buffer,_buffer+_head,_tail-_head);
-            _tail-=_head;
-            _head=0;
-        }
-        //
-        ADM_assert(nbIn+_tail<ADMWA_BUF);
-        memcpy(_buffer+_tail,inptr,nbIn);
-        _tail+=nbIn;
+    AVPacket pkt;
+    av_init_packet(&pkt);
+    int nbChunk,res=0;
+    bool eof=false;
+    while(_tail-_head>=_blockalign && !eof)
+    {
+        nbChunk=(_tail-_head)/_blockalign;
+        pkt.size=nbChunk*_blockalign;
+        pkt.data=_buffer+_head;
 
-        AVPacket pkt;
-        av_init_packet(&pkt);
-        int nbChunk,out,gotData;
-        while(_tail-_head>=_blockalign)
+        avcodec_send_packet(_context, &pkt);
+        // Regardless of the outcome, always consume the data.
+        _head+=nbChunk*_blockalign;
+
+        while(!res)
         {
-            nbChunk=(_tail-_head)/_blockalign;
-            pkt.size=nbChunk*_blockalign;
-            pkt.data=_buffer+_head;
-      
-            out=avcodec_decode_audio4(_context, _frame, &gotData, &pkt);
-            if(out<0)
+            res=avcodec_receive_frame(_context, _frame);
+            if(res==AVERROR(EAGAIN)) break; // we need to send more input
+            if(res==AVERROR_EOF)
             {
-                ADM_warning( "[ADM_ad_lav] *** decoding error (%u)***\n",_blockalign);
-                _head+=1; // Try skipping some bytes
-                continue;
+                eof=true;
+                break;
             }
-            _head+=out; // consumed bytes
-            if(!gotData)
-                continue;
+            if(res<0)
+            {
+                char er[2048]={0};
+                av_make_error_string(er, sizeof(er)-1, res);
+                ADM_warning("[ADM_ad_lav] decoding error: %s\n",er);
+                break;
+            }
+
+            bool invalid=false;
+            int  toCheck=1;
+            if(_context->sample_fmt==AV_SAMPLE_FMT_FLTP || _context->sample_fmt==AV_SAMPLE_FMT_S32P)
+                toCheck=channels;
+
+            for(int i=0;i<toCheck;i++)
+            {
+                if(!_frame->data[i]) // no data inside, abort
+                {
+                    invalid=true;
+                    break;
+                }
+            }
+            if(invalid) 
+                break;
             switch(_context->sample_fmt)
             {
-              case AV_SAMPLE_FMT_FLT:     decodeToFloat(&outptr,nbOut);break;
-              case AV_SAMPLE_FMT_FLTP:    decodeToFloatPlanar(&outptr,nbOut);break;             
-              case AV_SAMPLE_FMT_S32P:    decodeToS32Planar(&outptr,nbOut);break;              
-              case AV_SAMPLE_FMT_S32:    decodeToS32(&outptr,nbOut);break;              
-              
-              default:
-                  ADM_info("Decoder created using ??? %d...\n",_context->sample_fmt);
-                  ADM_assert(0);
-                  break;
+                case AV_SAMPLE_FMT_FLT:     decodeToFloat(&outptr,nbOut);break;
+                case AV_SAMPLE_FMT_FLTP:    decodeToFloatPlanar(&outptr,nbOut);break;
+                case AV_SAMPLE_FMT_S32P:    decodeToS32Planar(&outptr,nbOut);break;
+                case AV_SAMPLE_FMT_S32:     decodeToS32(&outptr,nbOut);break;
+                default:
+                    ADM_info("Decoder created using ??? %d...\n",_context->sample_fmt);
+                    ADM_assert(0);
+                    break;
             }
         }
+    }
+
+    if(!sbrChecked)
+    {
+        if(_context->sample_rate!=outputFrequency)
+        {
+            ADM_warning("Output frequency %d does not match input frequency %d. Implicit SBR?\n",
+                _context->sample_rate,outputFrequency);
+            outputFrequency=_context->sample_rate;
+        }
+        sbrChecked=true;
+    }
+
         //------------------
           if(channels>=5 )
             {
             CHANNEL_TYPE *p_ch_type = channelMapping;
 #define DOIT(x,y) if(_context->channel_layout & AV_CH_##x) *(p_ch_type++)=ADM_CH_##y;
+            if(!_context->channel_layout)
+                _context->channel_layout=av_get_default_channel_layout(channels);
 
                     DOIT(FRONT_LEFT,FRONT_LEFT);
                     DOIT(FRONT_RIGHT,FRONT_RIGHT);
                     DOIT(FRONT_CENTER,FRONT_CENTER);
                     DOIT(LOW_FREQUENCY,LFE);
-                    DOIT(SIDE_LEFT,REAR_LEFT);
-                    DOIT(SIDE_RIGHT,REAR_RIGHT); // AV_CH_SIDE_LEFT
+                    DOIT(SIDE_LEFT,REAR_LEFT) // see https://trac.ffmpeg.org/ticket/3160
+                    DOIT(SIDE_RIGHT,REAR_RIGHT)
+                    // as long as we don't support 7.1, we map both side and back channels to ADM_CH_REAR_*
+                    DOIT(BACK_LEFT,REAR_LEFT)
+                    DOIT(BACK_RIGHT,REAR_RIGHT)
             }
 
         return 1;
