@@ -40,11 +40,9 @@ static ADM_audioEncoder encoderDesc = {
   NULL   // Defined by macro automatically
 
 };
-//ADM_DECLARE_AUDIO_ENCODER_CONFIG(NULL);
-extern "C" ADM_audioEncoder *getInfo (void)
-{
-  return &encoderDesc;
-}
+
+ADM_DECLARE_AUDIO_ENCODER_CONFIG();
+
 
 /******************* / Declare plugin*******************************************************/
 
@@ -58,6 +56,8 @@ AUDMEncoder_PCM::AUDMEncoder_PCM(AUDMAudioFilter * instream,bool globalHeader, C
 {
   printf("[PCM] Creating PCM\n");
   wavheader.encoding=WAV_PCM;
+  wavheader.bitspersample=16;
+  
 
 };
 
