@@ -74,10 +74,8 @@ uint8_t mkvHeader::open(const char *name)
 {
 
   ADM_ebml_file ebml;
-  uint64_t id,len;
+  uint64_t len;
   uint64_t alen;
-  ADM_MKV_TYPE type;
-  const char *ss;
 
   _timeBase=1000; // default value is 1 ms timebase (unit is in us)
   _isvideopresent=0;
@@ -120,6 +118,16 @@ uint8_t mkvHeader::open(const char *name)
   {
        ADM_ebml_file seekHead( &ebml,alen);
        readSeekHead(&seekHead);
+  }else      
+  { // No valid seek_head, try to find MKV_TRACKS
+      ADM_info("Searching for MKV_TRACKS\n");
+      int headerLen;
+      uint64_t bodyLen;
+      if(!ebml.findContainerOfSecondary(MKV_SEGMENT,MKV_TRACKS,true, &_trackPosition, &headerLen,&bodyLen))
+      {
+          ADM_warning("Cannot locate MKV_TRACKS\n");
+          return false;
+      }
   }
   /* Now find tracks */
   /* And analyze them */
@@ -135,17 +143,21 @@ uint8_t mkvHeader::open(const char *name)
   }
   readCue(&ebml);
   printf("[MKV] Indexing clusters\n");
-  if(!indexClusters(&ebml))
+  uint8_t result=indexClusters(&ebml);
+  if(result!=1)
   {
-    printf("[MKV] Cluster indexing failed\n");
-    return 0;
+    if(!result)
+        printf("[MKV] Cluster indexing failed\n");
+    return result;
   }
   printf("[MKV]Found %u clusters\n",_clusters.size());
   printf("[MKV] Indexing video\n");
-    if(!videoIndexer(&ebml))
+    result=videoIndexer(&ebml);
+    if(result!=1)
     {
-      printf("[MKV] Video indexing failed\n");
-      return 0;
+        if(!result)
+            printf("[MKV] Video indexing failed\n");
+        return result;
     }
   // update some infos
   _videostream.dwLength= _mainaviheader.dwTotalFrames=_tracks[0].index.size();;
@@ -170,33 +182,33 @@ uint8_t mkvHeader::open(const char *name)
     for(int i=0;i<1+_nbAudioTrack;i++)
         ADM_info("Track %" PRIu32" has an index size of %d entries\n",i,_tracks[i].index.size());
 
+    int last=_tracks[0].index.size();
+    if(isVC1Compatible(_videostream.fccHandler))
+    {
+        mkvTrak *vid=_tracks;
 
-  if(isVC1Compatible(_videostream.fccHandler))
-  {
-      int       nb=_tracks[0].index.size();
-      mkvTrak   *vid=_tracks;
-
-      ADM_warning("Deleting  timestamps. For VC1, they are often wrong\n");
-      for(int i=1;i<nb-1;i++)
-          vid->index[i].Pts=ADM_NO_PTS;
-  }
+        ADM_warning("Deleting timestamps. For VC1, they are often wrong\n");
+        for(int i=1;i<last-1;i++)
+            vid->index[i].Pts=ADM_NO_PTS;
+    }
   // Delay frames + recompute frame duration
 // now that we have a good frameduration and max pts dts difference, we can set a proper DTS for all video frame
     uint32_t ptsdtsdelta, mindelta;
-    bool hasBframe;
-  ComputeDeltaAndCheckBFrames(&mindelta, &ptsdtsdelta,&hasBframe);
+    bool hasBframe=false;
+    if(last>1)
+        ComputeDeltaAndCheckBFrames(&mindelta, &ptsdtsdelta, &hasBframe);
 
-
-  int last=_tracks[0].index.size();
-  uint64_t increment=_tracks[0]._defaultFrameDuration;
-  uint64_t lastDts=0;
-  _tracks[0].index[0].Dts=0;
-  mkvTrak                 *vid=_tracks;
-  if(hasBframe==true) // Try to compute a sane DTS knowing the PTS and the DTS/PTS delay
+    uint64_t increment=_tracks[0]._defaultFrameDuration;
+    uint64_t lastDts=0;
+    _tracks[0].index[0].Dts=0;
+    mkvTrak *vid=_tracks;
+    if(hasBframe==true) // Try to compute a sane DTS knowing the PTS and the DTS/PTS delay
     {
-      for(int i=1;i<last;i++)
-      {
-            uint64_t pts,dts;
+        bool useSortedPtsToSetDts=false;
+        int bumped=0;
+        for(int i=1;i<last;i++)
+        {
+            uint64_t pts;
             pts=vid->index[i].Pts;
             lastDts+=increment; // This frame dts with no correction
             if(pts==ADM_NO_PTS)
@@ -204,28 +216,73 @@ uint8_t mkvHeader::open(const char *name)
                 vid->index[i].Dts=lastDts;
                 continue;
             }
-            uint64_t limitDts=vid->index[i].Pts-ptsdtsdelta;
-            if(  lastDts<limitDts)
+            uint64_t limitDts=0;
+            if(vid->index[i].Pts>ptsdtsdelta)
+                limitDts=vid->index[i].Pts-ptsdtsdelta;
+
+#define DTS_BUMPED_THRESHOLD 10
+
+            if(lastDts<limitDts)
             {
-                lastDts=limitDts;
+                if(i==1)
+                {
+                    lastDts = (vid->index[0].Pts-ptsdtsdelta>0)? vid->index[0].Pts-ptsdtsdelta : 0;
+                    vid->index[0].Dts = lastDts;
+                    lastDts+=increment;
+                }
+                if(limitDts-lastDts>1000) // not just a rounding error
+                {
+                    ADM_warning("Bumping DTS by %" PRIu64" ms to keep PTS/DTS delta within limits for frame %d\n",(limitDts-lastDts)/1000,i);
+                    lastDts=limitDts;
+                    bumped++;
+                    if(bumped>DTS_BUMPED_THRESHOLD)
+                    {
+                        ADM_warning("Default time increment is probably too low, resorting to direct mapping.\n");
+                        useSortedPtsToSetDts=true;
+                        break;
+                    }
+                }
             }
             vid->index[i].Dts=lastDts;
-      }
+        }
+        if(useSortedPtsToSetDts)
+        {
+            setDtsFromListOfSortedPts();
+        }
         // Check that we have PTS>=DTS also
-        uint64_t enforePtsGreaterThanDts=0;
+        uint64_t enforcePtsGreaterThanDts;
+        bool secondTry=false;
+_again:
+        enforcePtsGreaterThanDts=0;
+        int frmeMaxDlta=-1;
         for(int i=0;i<last;i++)
         {
-                if(vid->index[i].Pts<vid->index[i].Dts)
+            if(vid->index[i].Pts<vid->index[i].Dts)
+            {
+                uint64_t delta=vid->index[i].Dts-vid->index[i].Pts;
+                if(delta>enforcePtsGreaterThanDts)
                 {
-                    uint64_t delta=vid->index[i].Dts-vid->index[i].Pts;
-                    if(delta>enforePtsGreaterThanDts) enforePtsGreaterThanDts=delta;
+                    enforcePtsGreaterThanDts=delta;
+                    frmeMaxDlta=i;
                 }
+            }
         }
-        if(enforePtsGreaterThanDts)
+        if(enforcePtsGreaterThanDts)
         {
-                ADM_info("Have to delay by %" PRIu64" us so that PTS>DTS\n",enforePtsGreaterThanDts);
-                for(int i=0;i<_nbAudioTrack+1;i++)
-                delayTrack(i,&(_tracks[i]),enforePtsGreaterThanDts);
+            ADM_info("Have to delay by %" PRIu64" us detected for frame %d so that PTS>DTS\n",enforcePtsGreaterThanDts,frmeMaxDlta);
+            if(enforcePtsGreaterThanDts>ptsdtsdelta)
+            {
+                ADM_warning("The delay is implausibly high. The calculated frame rate is probably too low.\n");
+                if(!secondTry)
+                {
+                    setDtsFromListOfSortedPts();
+                    ADM_warning("Retrying with direct mapping.\n");
+                    secondTry=true;
+                    goto _again;
+                }
+            }
+            for(int i=0;i<_nbAudioTrack+1;i++)
+                delayTrack(i,&(_tracks[i]),enforcePtsGreaterThanDts);
         }
     }else
     {       // No bframe, DTS=PTS
@@ -238,29 +295,53 @@ uint8_t mkvHeader::open(const char *name)
 
   if(last)
   {
-          float duration=_tracks[0].index[last-1].Pts;
-          duration/=1000;
-          uint32_t duration32=(uint32_t)duration;
-          printf("[MKV] Video Track duration for %u ms\n",duration32);
-          // Useless.....
-
-
+      for(int i=last-32;i<last;i++)
+      {
+          if(i<0) continue;
+          if(_tracks[0].index[i].Pts==ADM_NO_PTS) continue;
+          if(_tracks[0].duration<_tracks[0].index[i].Pts)
+              _tracks[0].duration=_tracks[0].index[i].Pts;
+      }
+      _tracks[0].duration+=increment;
+      uint32_t duration32=(uint32_t)((_tracks[0].duration+500)/1000);
+      printf("[MKV] Video track duration: %s (%u ms)\n",ADM_us2plain(_tracks[0].duration),duration32);
+      // Useless.....
 
           for(int i=0;i<_nbAudioTrack;i++)
           {
             rescaleTrack(&(_tracks[1+i]),duration32);
-            if(_tracks[1+i].wavHeader.encoding==WAV_OGG_VORBIS)
+            switch(_tracks[1+i].wavHeader.encoding)
             {
-                printf("[MKV] Reformatting vorbis header for track %u\n",i);
-                reformatVorbisHeader(&(_tracks[1+i]));
+                case WAV_OGG_VORBIS:            
+                    ADM_info("[MKV] Reformatting vorbis header for track %u\n",i);
+                    reformatVorbisHeader(&(_tracks[1+i]));
+                    break;
+                case WAV_PCM:
+                case WAV_LPCM:
+                    ADM_info("[MKV] Checking PCM block size for track %i\n",i);
+                    _tracks[i+1]._needBuffering=isBufferingNeeded(&(_tracks[i+1]));
+                    break;
+                default:
+                    break;
+                
             }
+            
+            
           }
     }
-    _access=new mkvAccess *[_nbAudioTrack];
+    _access=new ADM_audioAccess *[_nbAudioTrack];
     _audioStreams=new ADM_audioStream *[_nbAudioTrack];
     for(int i=0;i<_nbAudioTrack;i++)
     {
-        _access[i]=new mkvAccess(_filename,&(_tracks[i+1]));
+        if(_tracks[i+1]._needBuffering)
+        {
+            mkvAccess *access=new mkvAccess(_filename,&(_tracks[i+1]));
+            _access[i]=new mkvAccessBuffered(access,_tracks[i+1]._needBuffering);
+        }
+        else
+        {
+               _access[i]=new mkvAccess(_filename,&(_tracks[i+1]));
+        }
         _audioStreams[i]=ADM_audioCreateStream(&(_tracks[1+i].wavHeader), _access[i]);;
         _audioStreams[i]->setLanguage(_tracks[1+i].language);
     }
@@ -283,6 +364,86 @@ bool mkvHeader::delayTrack(int index,mkvTrak *track, uint64_t value)
     }
     return true;
 }
+/**
+    \fn setDtsFromListOfSortedPts
+    \brief Handle cases of mixed content and wrong default time increment
+*/
+bool mkvHeader::setDtsFromListOfSortedPts(void)
+{
+    mkvTrak *vid=_tracks;
+    int last=_tracks[0].index.size();
+    int nbValidPts=_sortedPts.size();
+    int nbPtsUnset=_framesNoPts.size();
+    if(!nbPtsUnset)
+    {
+        ADM_assert(last==nbValidPts);
+        for(int i=0;i<last;i++)
+            vid->index[i].Dts=_sortedPts.at(i);
+    }else
+    {
+        int start=0;
+        while(start<nbPtsUnset && _framesNoPts.at(start)==start) start++;
+        // The first entry in the sorted list is frame "start", how many consecutive frames with pts do we have?
+        int span=last-start;
+        if(start<nbPtsUnset)
+            span=_framesNoPts.at(start)-start;
+        // Calculate current average frame increment from the first 8 frames at best.
+        uint64_t increment=_tracks[0]._defaultFrameDuration;;
+        int psize=(span<8)? span : 8;
+        if(psize)
+        {
+            double d=_sortedPts.at(psize);
+            d/=psize;
+            increment=d;
+        }
+        // Calculate the offset we must add to all timestamps when the first frames don't have valid pts.
+        uint64_t shift=increment*start;
+        // Go!
+        int curValid=0;
+        int curNoPts=0;
+        int backfill=0;
+        uint64_t lastFilledDts=0;
+        for(int i=0;i<last;i++)
+        {
+            if(curNoPts==nbPtsUnset || _framesNoPts.at(curNoPts)>i)
+            {
+                uint64_t dts=_sortedPts.at(curValid)+shift;
+                if(backfill)
+                {
+                    if(curValid>1) // We don't get meaningful time increment at the start of the video, use what we alredy have.
+                    {
+                        double d=dts-lastFilledDts;
+                        d/=backfill+1;
+                        increment=d;
+                    }
+                    uint64_t shift2=0;
+                    for(int j=0;j<backfill;j++)
+                    {
+                        int k=i-backfill+j;
+                        shift2=increment*(j+1);
+                        vid->index[k].Dts=lastFilledDts+shift2;
+                    }
+                    if(!shift) // We are at the frame right after early B-frames.
+                    {
+                        dts+=shift2;
+                        if(curValid<2)
+                            shift=shift2;
+                    }
+                    backfill=0;
+                }
+                vid->index[i].Dts=dts;
+                lastFilledDts=dts;
+                curValid++;
+                if(curValid==nbValidPts) break;
+            }else
+            {
+                backfill++;
+                curNoPts++;
+            }
+        }
+    }
+    return true;
+}
 /**
     \fn checkDeviation
     \brief Bypass the 1ms accuracy by making sure all the frames are in the form PTS=offset + N*frameInterval
@@ -294,8 +455,6 @@ bool mkvHeader::enforceFixedFrameRate(int num, int den)
   double dHalf=(500000.*(double)num)/((double)den);
   int half=dHalf-1; // half interval in us
   int first=0;
-  int bad=0;
-  int good=0;
   while(  track->index[first].Pts==ADM_NO_PTS && first<nb) first++; // we should have some at least
   uint64_t zero= track->index[first].Pts;
   ADM_info("Num=%d Den=%d half=%d zero=%d first=%d\n",num,den,half,(int)zero,first);
@@ -309,7 +468,13 @@ bool mkvHeader::enforceFixedFrameRate(int num, int den)
     dmultiple/=(1000000.*(double)num);
     uint64_t multiple=(uint64_t)dmultiple;
     int64_t reconstructed=(multiple*1000000*num)/den+zero;
-    //printf("frame %d multiple = %d, pts=%d, reconstructed=%d,delta = %d\n",i,(int)multiple,(int)pts,(int)reconstructed,(int)(pts-reconstructed));
+#if 0
+    if(i<100)
+    {
+        pts+=zero;
+        printf("frame %d multiple = %d, pts=%d, reconstructed=%d,delta = %d\n",i,(int)multiple,(int)pts,(int)reconstructed,(int)(pts-reconstructed));
+    }
+#endif
     track->index[i].Pts=reconstructed;
   }
   return true;
@@ -329,7 +494,6 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
     int64_t delta,maxDelta=0;
     int64_t minDelta=100000000;
     *bFramePresent=false;
-    int nbValidDts=0;
 
 
 
@@ -362,8 +526,6 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
         // the maximum will give us the max PTS-DTS delta so that we can compute DTS
         for(int i=0;i<nb-1;i++)
         {
-            if(track->index[i].Dts!=ADM_NO_PTS)
-                nbValidDts++;
             if(track->index[i].flags==AVI_B_FRAME) nbBFrame++;
             if(track->index[i+1].Pts==ADM_NO_PTS || track->index[i].Pts==ADM_NO_PTS)
                 continue;
@@ -377,11 +539,12 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
             }
             if(delta<minDelta) minDelta=delta;
             if(delta>maxDelta) maxDelta=delta;
-            //printf("\/=%" PRId64" Min %" PRId64" MAX %" PRId64"\n",delta,minDelta,maxDelta);
+           // ADM_warning("Delta=%d,  MinDelta=%d MaxDelta=%d\n",delta,minDelta,maxDelta);
         }
     }
     if(nbBFrame) *bFramePresent=true;
 
+    ADM_warning(">>> MinDelta=%d MaxDelta=%d\n",minDelta,maxDelta);
     int stdFrameRate=getStdFrameRate(track->_defaultFrameDuration);
 
     int num= _videostream.dwScale;
@@ -395,18 +558,35 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
         mkvDeviation devEngine(nb);      
         int first=0;
         while(  track->index[first].Pts==ADM_NO_PTS && first<nb) 
-              first++; // we should have some at least
+        {
+            _framesNoPts.push_back(first);
+            first++; // we should have some at least
+        }
         uint64_t zero= track->index[first].Pts;
         ADM_info("Num=%d Den=%d zero=%d first=%d\n",num,den,(int)zero,first);
+        int valid=0;
         for(int i=first;i<nb;i++)
         {
-          uint64_t pts=track->index[i].Pts;
-          if(pts==ADM_NO_PTS) continue;
-          if(pts<zero) continue;
-            
-          devEngine.add(pts-zero);
+            uint64_t pts=track->index[i].Pts;
+            if(pts==ADM_NO_PTS)
+            {
+                _framesNoPts.push_back(i);
+                continue;
+            }
+            if(pts<zero) // early B-frames
+            {
+                _framesNoPts.push_back(i);
+                continue;
+            }
+            devEngine.add(pts-zero);
+            valid++;
         }
         devEngine.sort();
+        uint64_t *s = devEngine.getSorted();
+        for(int i=0;i<valid;i++)
+        {
+            _sortedPts.push_back(s[i]-500); // subtract 500 us added by add()
+        }
         ADM_info("Checking deviation for native %d %d\n", _videostream.dwScale,   _videostream.dwRate);
         deviation=devEngine.computeDeviation(  _videostream.dwScale,   _videostream.dwRate,skipped);
         
@@ -420,13 +600,18 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
         ADM_info("Deviation        = %d\n",deviation);
         ADM_info("DeviationMinDelta = %d\n",deviationMinDelta);
         ADM_info("Deviation skip    = %d\n",minDeltaSkip);
-    
-        if(minDeltaSkip>skipped*3) // we are skipping a lot more frame, fps too high
+        bool skipValid=true;
+        if(track->_defaultFrameDuration)
         {
-            ADM_info("Too many skipped frames, dropping candidates\n");
+            ADM_info("MinDelta=%d, defaultFrameDuation=%d\n",minDelta,track->_defaultFrameDuration);
+            if((minDelta*2)<=track->_defaultFrameDuration)
+                skipValid=false;
+        }
+        if(minDeltaSkip>skipped*3  && skipValid) // we are skipping a lot more frame, fps too high
+        {
+            ADM_info("Too many skipped frames, dropping candidates (skipped=%d, min delta skip=%d)\n",skipped,minDeltaSkip);
             deviationMinDelta=deviation*2;
         }
-                
         if(minDelta)
         {
             bool preferMinDelta=false;
@@ -481,9 +666,6 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
         minDelta=100000000;
         for(int i=0;i<nb-1;i++)
         {
-            if(track->index[i].Dts!=ADM_NO_PTS)
-                nbValidDts++;
-            if(track->index[i].flags==AVI_B_FRAME) nbBFrame++;
             if(track->index[i+1].Pts==ADM_NO_PTS || track->index[i].Pts==ADM_NO_PTS)
                 continue;
 
@@ -520,34 +702,6 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
 
     ADM_info("First frame pts     %" PRId64" us\n",track->index[0].Pts);
 
-    if(nbValidDts<3)
-    {
-            ADM_warning("Not enough valid DTS (%d)\n",(int)nbValidDts);
-            *minDeltaX=minDelta;
-            *maxDeltaX=0;
-            return false;
-    }
-
-
-    uint64_t adj=0;
-    int limit=32;
-    if(limit>nb) limit=nb;
-    // Pts must be >= maxDelta for all frames, the first 32 will do
-    for(int i=0;i<limit;i++)
-    {
-        if(maxDelta>track->index[i].Pts)
-        {
-            uint64_t newAdj=maxDelta-track->index[i].Pts;
-            if(newAdj>adj) adj=newAdj;
-        }
-    }
-    if(adj) // need to correct
-    {
-        ADM_info("Delaying video by %" PRIu64" us\n",adj);
-        ADM_info("[mkv] Delaying audio by %" PRIu64" us\n",adj);
-        for(int i=0;i<_nbAudioTrack+1;i++)
-            delayTrack(i,&(_tracks[i]),adj);
-    }
     *maxDeltaX=maxDelta;
     *minDeltaX=minDelta;
     return true;
@@ -817,7 +971,6 @@ uint8_t mkvHeader::close(void)
   _nbAudioTrack=0;
   _filename=NULL;
  // memset(_tracks,0,sizeof(_tracks));
-  _reordered=0;
 
   _currentAudioTrack=0;
   _access=NULL;
@@ -877,7 +1030,7 @@ uint64_t mkvHeader::getVideoDuration(void)
 {
     uint32_t limit=_tracks[0].index.size();
     if(!limit) return 0;
-    return _tracks[0].index[limit-1].Pts+_tracks[0]._defaultFrameDuration;
+    return _tracks[0].duration;
 }
 
 /**
@@ -929,6 +1082,7 @@ uint8_t  mkvHeader::getExtraHeaderData(uint32_t *len, uint8_t **data)
  * @param hd
  * @return
  */
+#if 0
 static int xypheLacingRead(uint8_t **hd)
 {
       int x=0;
@@ -943,7 +1097,7 @@ static int xypheLacingRead(uint8_t **hd)
       *hd=p;
       return x;
 }
-
+#endif
 /**
     \fn mkreformatVorbisHeader
     \brief reformat oggvorbis header to avidemux style
@@ -1106,5 +1260,30 @@ bool    mkvHeader::readSeekHead(ADM_ebml_file *body)
     return true;
 }
 
+/**
+ * 
+ * @param trk
+ * \brief  Make sure the one chunk is not too big, split it else
+ *          ~ 40ms worth of data are good enough
+ * @return 
+ */
+int mkvHeader::isBufferingNeeded(mkvTrak *trk)
+{
+   int n=trk->index.size();
+   int max=0;
+   for(int i=0;i<n;i++)
+   {
+       int sz=trk->index[i].size;
+       if(max<sz) max=sz;
+   }
+   if(max<64*1024)
+   {
+       ADM_info("No big packet detected\n");
+       return 0;
+   }
+   max=((max>>10)+1)<<10;
+   ADM_warning("Big packet detected : %d kBytes \n",max>>10);
+   return max;
+}
 //****************************************
 //EOF
