@@ -62,6 +62,7 @@ public:
         _sizeInBytes=0;
         _defaultFrameDuration=0;
         language=ADM_UNKNOWN_LANGUAGE;
+        _needBuffering=0;
   }
   /* Index in mkv */
   uint32_t  streamIndex;
@@ -83,9 +84,34 @@ public:
   uint32_t  _sizeInBytes; // Approximate size in bytes of that stream
   uint32_t  _defaultFrameDuration; // Duration of ONE frame in us!
   std::string language;
+  int       _needBuffering;
 };
 
 #define MKV_MAX_LACES 101 // ?
+/**
+ */
+class mkvAccess;
+class mkvAccessBuffered : public ADM_audioAccess
+{
+public:
+            mkvAccessBuffered(mkvAccess *ac,int maxSize);
+            ~mkvAccessBuffered();
+                virtual bool      isCBR(void) { return false;}
+                virtual bool      canSeekTime(void) {return true;};
+                virtual bool      canSeekOffset(void) {return false;};
+                virtual bool      canGetDuration(void) {return true;};
+                virtual uint64_t  getDurationInUs(void);
+                virtual bool      goToTime(uint64_t timeUs);
+                virtual bool      getPacket(uint8_t *buffer, uint32_t *size, uint32_t maxSize,uint64_t *dts);
+                virtual bool      getExtraData(uint32_t *l, uint8_t **d);  
+protected:
+            mkvAccess   *_son;
+            int         _maxSize;
+            uint8_t     *_buffer;
+            int         _inBuffer;
+            int         _offset;
+};
+
 /**
     \class mkvAccess
     \brief Matroska audio demuxer
@@ -105,9 +131,14 @@ protected:
 
     uint8_t                     goToBlock(uint32_t x);
     bool                        initLaces(uint32_t nbLaces,uint64_t time);
-     int              readAndRepeat(uint8_t *buffer, uint32_t len)
+     int              readAndRepeat(uint8_t *buffer, uint32_t len, uint32_t max)
                         {
                              uint32_t rpt=_track->headerRepeatSize;
+                             if(rpt+len>max)
+                               {
+                                 ADM_error("Overflow in reading  mkv audio : %u (%u) max was %d\n",rpt+len,rpt,max);
+                                 ADM_assert(0);
+                               }
                               _parser->readBin(buffer+rpt,len);
                               if(rpt)
                                 memcpy(buffer,_track->headerRepeat,rpt);
@@ -145,7 +176,7 @@ class mkvHeader         :public vidHeader
 {
   protected:
     uint64_t                 _timeBase;  // Time base in us, default is 1000=1 ms
-    mkvAccess               **_access;
+    ADM_audioAccess         **_access;
     ADM_audioStream         **_audioStreams;
     ADM_ebml_file           *_parser;
     char                    *_filename;
@@ -154,10 +185,12 @@ class mkvHeader         :public vidHeader
     BVector <mkvIndex    >   _clusters;
     BVector <uint64_t>       _cueTime;
 
+    std::vector <uint64_t>  _sortedPts;
+    std::vector <uint32_t>  _framesNoPts;
+
     uint32_t                _nbAudioTrack;
     uint32_t                _currentAudioTrack;
-    uint32_t                _reordered;
-    //
+
     uint64_t                _segmentPosition;
     uint64_t                _cuePosition;
     uint64_t                _trackPosition;
@@ -171,6 +204,8 @@ class mkvHeader         :public vidHeader
     int                     searchTrackFromTid(uint32_t tid);
     bool                    readSeekHead(ADM_ebml_file *body);
     //
+    //                      
+    int                     isBufferingNeeded(mkvTrak *trk); // Split audio blocks into smaller pieces if needed
     uint8_t                 reformatVorbisHeader(mkvTrak *trk);
     // Indexers
 
@@ -192,6 +227,7 @@ class mkvHeader         :public vidHeader
                                                 MKV_ELEM_ID searchedId,const char *txt);
     int                     checkDeviation(int num, int den);
     bool                    enforceFixedFrameRate(int num, int den);
+    bool                    setDtsFromListOfSortedPts(void);
   public:
 
 
