@@ -24,7 +24,7 @@
 #include "ADM_mkv.h"
 
 #include "mkv_tags.h"
-
+#include "ADM_aacinfo.h"
 class entryDesc
 {
   public:
@@ -84,7 +84,7 @@ static int aac_get_sample_rate_index(uint32_t sample_rate)
  * @param needle
  * @return 
  */
-static bool hasNeedle(const char *haystack, char *needle)
+static bool hasNeedle(const char *haystack, const char *needle)
 {
     if(NULL!=strstr( (char *)(haystack+12),needle)) return true;
     return false;
@@ -108,16 +108,15 @@ static void createAACExtraData(const char *codec,entryDesc *entry)
     entry->extraDataLen=2;
     if(hasNeedle(codec,"SBR"))
     {
-        entry->extraDataLen=5;    
-        sampleRateIndex=aac_get_sample_rate_index(entry->fq*2);
+        entry->extraDataLen=5;
+        entry->fq*=2;
+        sampleRateIndex=aac_get_sample_rate_index(entry->fq);
         e[2] = AAC_SYNC_EXTENSION_TYPE >> 3;
         e[3] = ((AAC_SYNC_EXTENSION_TYPE & 0x07) << 5) | 5;
         e[4] = (1 << 7) | (sampleRateIndex << 3);
     }
-    ADM_info("Created %d bytes ",entry->extraDataLen);
-    for(int i=0;i<entry->extraDataLen;i++)
-        ADM_info(" %02x",entry->extraData[i]);
-    ADM_info("\n");
+    ADM_info("Created %d bytes\n",entry->extraDataLen);
+    mixDump(entry->extraData,entry->extraDataLen);
 }
 
 /**
@@ -190,7 +189,7 @@ uint8_t mkvHeader::analyzeOneTrack(void *head,uint32_t headlen)
           printf("[MKV] No duration, assuming 25 fps\n");
           _videostream.dwScale=1000;
           _videostream.dwRate=25000;
-          _tracks[0]._defaultFrameDuration=25000;
+          _tracks[0]._defaultFrameDuration=40000;
 
         }
 
@@ -220,7 +219,6 @@ uint8_t mkvHeader::analyzeOneTrack(void *head,uint32_t headlen)
                 memcpy(_tracks[0].extraData,entry.extraData +sizeof(ADM_BITMAPINFOHEADER),l);
                 ADM_info("VFW Header+%d bytes of extradata\n",l);   
                 mixDump(_tracks[0].extraData,l);
-                printf("\n");
           }
           delete [] entry.extraData;
           entry.extraData=NULL;
@@ -247,8 +245,8 @@ uint8_t mkvHeader::analyzeOneTrack(void *head,uint32_t headlen)
       //***************** Audio tracks *****************
       if(entry.trackType==2 && _nbAudioTrack<ADM_MKV_MAX_TRACKS)
       {
-         uint32_t  streamIndex;
-         mkvTrak *t=&(_tracks[1+_nbAudioTrack]);
+        mkvTrak *t=&(_tracks[1+_nbAudioTrack]);
+        t->extraDataLen=entry.extraDataLen;
         ADM_info("This track has %d bytes of  extradata\n",t->extraDataLen);
         // MS/ACM : ACMX
         if(0x100001==entry.fcc)
@@ -258,7 +256,7 @@ uint8_t mkvHeader::analyzeOneTrack(void *head,uint32_t headlen)
             ADM_info("Found ACM compatibility header (%d / %d)\n",l,wavSize);
             if(l>=wavSize) // we need at least a wavheader
             {
-                mixDump(entry.extraData,l); printf("\n");
+                mixDump(entry.extraData,l);
                 memcpy(&(t->wavHeader),entry.extraData,wavSize);
                 ADM_info("Encoding : %d\n",t->wavHeader.encoding);
                 int x=l-wavSize;
@@ -282,13 +280,28 @@ uint8_t mkvHeader::analyzeOneTrack(void *head,uint32_t headlen)
             }
         }
         //**
-                  if(entry.fcc==WAV_AAC && !entry.extraDataLen)
-                  {
-                      ADM_info("Recreating aac extradata..\n");
-                      entry.extraData = new uint8_t[5];
-                      createAACExtraData(entry.codecId.c_str(),&entry);
-                  }
-        
+        if(entry.fcc==WAV_AAC)
+        {
+            if(!entry.extraDataLen)
+            {
+                ADM_info("Recreating aac extradata..\n");
+                entry.extraData = new uint8_t[5];
+                createAACExtraData(entry.codecId.c_str(),&entry);
+            }else
+            {
+                // check AAC infoata
+                 AacAudioInfo info;
+                if(ADM_getAacInfoFromConfig(entry.extraDataLen,entry.extraData,info))
+                {
+                    ADM_info("Decoding AAC extra data gives :\n");
+                    ADM_info("Fq= %d\n",info.frequency);
+                    ADM_info("channels= %d\n",info.channels);
+                    ADM_info("SBR= %d\n",info.sbr);
+                    entry.chan=info.channels;
+                    entry.fq=info.frequency;
+                }
+            }
+        }
         //**
          t->language=entry.language;
          t->wavHeader.encoding=entry.fcc;
@@ -342,14 +355,14 @@ uint8_t entryWalk(ADM_ebml_file *head,uint32_t headlen,entryDesc *entry)
       father.readElemId(&id,&len);
       if(!ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type))
       {
-        printf("[MKV] Tag 0x%"PRIx64" not found (len %"PRIu64")\n",id,len);
+        printf("[MKV] Tag 0x%" PRIx64" not found (len %" PRIu64")\n",id,len);
         father.skip(len);
         continue;
       }
       switch(id)
       {
         case  MKV_CONTENT_COMPRESSION_SETTINGS: 
-#warning todo: check it is stripping
+//#warning todo: check it is stripping
                     if(len<=MKV_MAX_REPEAT_HEADER_SIZE)
                     {
                         father.readBin(entry->headerRepeat,len);
@@ -371,7 +384,7 @@ uint8_t entryWalk(ADM_ebml_file *head,uint32_t headlen,entryDesc *entry)
         case  MKV_TIMECODE_SCALE:
         case  MKV_TRACK_TIMECODESCALE:
                                 {
-                                    ADM_warning("[Mkv] TimeCodeScale=%"PRIu64"\n",father.readUnsignedInt(len));
+                                    ADM_warning("[Mkv] TimeCodeScale=%" PRIu64"\n",father.readUnsignedInt(len));
                                 };break; //FIXME
 
         case  MKV_FRAME_DEFAULT_DURATION: entry->defaultDuration=father.readUnsignedInt(len)/1000; break; // In us
@@ -397,7 +410,16 @@ uint8_t entryWalk(ADM_ebml_file *head,uint32_t headlen,entryDesc *entry)
                  father.readString(s,len);
                  if(!strlen(s))
                      strcpy(s,"eng"); // english is default
-                 ADM_info("Found language  = %s\n",s);
+                 if(!strcmp(s,"unknown")) // we were using "unknown", which is not a valid ISO 639 code
+                 {
+                     memset(s,'\0',100);
+                     std::string und=ADM_UNKNOWN_LANGUAGE;
+                     strcpy(s,und.c_str());
+                     ADM_info("Found 'unknown' as language code, replacing it with '%s'\n",und.c_str());
+                 }else
+                 {
+                     ADM_info("Found language  = %s\n",s);
+                 }
                  entry->language=std::string(s);
                 }
             break;
