@@ -38,14 +38,13 @@
 */
 uint8_t mkvHeader::videoIndexer(ADM_ebml_file *parser)
 {
-  uint64_t fileSize,len,bsize;
-  uint32_t alen,vlen;
-  uint64_t id;
+  uint64_t len,id;
   ADM_MKV_TYPE type;
   const char *ss;
 
    parser->seek(0);
    DIA_workingBase *work=createWorking(QT_TRANSLATE_NOOP("matroskademuxer","Matroska Images"));
+   uint8_t res=1;
 
    readBufferSize=200*1024;
    readBuffer=new uint8_t[readBufferSize];
@@ -66,6 +65,11 @@ uint8_t mkvHeader::videoIndexer(ADM_ebml_file *parser)
    int thiscluster=0;
    while(!cluster.finished())
    {
+      if(!work->isAlive())
+      {
+        res=ADM_IGN;
+        break;
+      }
       work->update(clusters,nbClusters);
       cluster.readElemId(&id,&len);
       if(!ADM_searchMkvTag( (MKV_ELEM_ID)id,&ss,&type))
@@ -125,7 +129,7 @@ uint8_t mkvHeader::videoIndexer(ADM_ebml_file *parser)
      delete work;
      delete [] readBuffer;
      readBuffer=NULL;
-     return 1;
+     return (res==ADM_IGN)? res : !!VIDEO.index.size();
 }
 /**
       \fn indexBlock
@@ -133,7 +137,7 @@ uint8_t mkvHeader::videoIndexer(ADM_ebml_file *parser)
 */
 uint8_t mkvHeader::indexBlock(ADM_ebml_file *parser,uint32_t len,uint32_t clusterTimeCodeMs)
 {
-  int lacing,nbLaces,entryFlags=0;
+  int entryFlags=0;
   //
   uint64_t tail=parser->tell()+len;
   // Read Track id
@@ -152,7 +156,7 @@ uint8_t mkvHeader::indexBlock(ADM_ebml_file *parser,uint32_t len,uint32_t cluste
       //if(!track) printf("TC: %d\n",timecode);
       uint8_t flags=parser->readu8();
 
-      lacing=((flags>>1)&3);
+      //int lacing=((flags>>1)&3);
 
       addIndexEntry(track,parser,blockBegin,tail-blockBegin,entryFlags,clusterTimeCodeMs+timecode);
       parser->seek(tail);
@@ -232,7 +236,7 @@ uint8_t mkvHeader::addIndexEntry(uint32_t track,ADM_ebml_file *parser,uint64_t w
             parser->readBin(readBuffer+rpt,size-3);
             // Search the frame type...
 
-             uint32_t nb,vopType,timeinc=16;
+             uint32_t nb,timeinc=16;
              ADM_vopS vops[10];
              vops[0].type=AVI_KEY_FRAME;
              ADM_searchVop(readBuffer,readBuffer+rpt+size-3,&nb,vops, &timeinc);
@@ -247,7 +251,7 @@ uint8_t mkvHeader::addIndexEntry(uint32_t track,ADM_ebml_file *parser,uint64_t w
                 if(rpt)
                         memcpy(readBuffer,_tracks[0].headerRepeat,rpt);
                 parser->readBin(readBuffer+rpt,size-3);
-                extractH264FrameType(2,readBuffer,rpt+size-3,&flags,&_H264Recovery); // Nal size is not used in that case
+                extractH264FrameType(readBuffer,rpt+size-3,&flags,NULL,NULL,&_H264Recovery);
                 if(flags & AVI_KEY_FRAME)
                 {
                     printf("[MKV/H264] Frame %" PRIu32" is a keyframe\n",(uint32_t)Track->index.size());
@@ -279,7 +283,6 @@ uint8_t mkvHeader::addIndexEntry(uint32_t track,ADM_ebml_file *parser,uint64_t w
                 parser->readBin(readBuffer+rpt,size-3);
                 uint8_t *begin=readBuffer;
                 uint8_t *end=readBuffer+size-3+rpt;
-                uint32_t flags=0;
                 while(begin<end)
                 {
                     int code=mkvFindStartCode(begin,end);
@@ -444,13 +447,13 @@ bool     mkvHeader::readCue(ADM_ebml_file *parser)
 */
 uint8_t   mkvHeader::indexClusters(ADM_ebml_file *parser)
 {
-  uint64_t fileSize,len,bsize;
+  uint64_t fileSize,len;
   uint64_t alen,vlen;
   uint64_t id;
   ADM_MKV_TYPE type;
   const char *ss;
-  uint64_t time;
   uint64_t pos;
+  uint8_t res=1;
 
     mkvIndex tmpCluster;
 
@@ -474,7 +477,11 @@ uint8_t   mkvHeader::indexClusters(ADM_ebml_file *parser)
    DIA_workingBase *work=createWorking(QT_TRANSLATE_NOOP("matroskademuxer","Matroska clusters"));
    while(segment.simplefind(MKV_CLUSTER,&alen,0))
    {
-
+     if(!work->isAlive())
+     {
+        res=ADM_IGN;
+        break;
+     }
      // UI update
      work->update(segment.tell()>>10,fileSize>>10);
      tmpCluster.pos=segment.tell();
@@ -512,7 +519,7 @@ tryAgain:
    }
    delete work;
    ADM_info("[MKV] Found %u clusters\n",(int)_clusters.size());
-   return 1;
+   return res;
 }
 /**
  * \fn updateFlagsWithCue
