@@ -88,7 +88,8 @@ MP4Track::MP4Track(void)
     nbIndex=0;
     id=0;
     memset(&_rdWav,0,sizeof(_rdWav));
-
+    delay=0;
+    totalDataSize=0;
 }
 MP4Track::~MP4Track()
 {
@@ -156,10 +157,34 @@ bool MP4Header::refineFps(void)
     {
         double f=1000000./(double)minDelta;
         f*=1000.;
-        ADM_info("MinDelta=%d us\n",(int)minDelta);
-        ADM_info("Computed fps1000=%d\n",(int)f);
         uint32_t fps1000=floor(f+0.49);
-        if(fps1000>  _videostream.dwRate)
+        ADM_info("MinDelta=%d us\n",(int)minDelta);
+        ADM_info("Computed fps1000=%d\n",fps1000);
+        if(fps1000 == _videostream.dwRate)
+        {
+            ADM_info("Computed fps1000 matches the average one.\n");
+            return true;
+        }
+        int score=0;
+        uint64_t avgDelta=_mainaviheader.dwMicroSecPerFrame;
+        int64_t halfway=avgDelta-minDelta+1;
+        halfway/=2;
+        halfway+=minDelta;
+        for(int i=0;i<n-1;i++)
+        {
+            MP4Index *dex=&(_tracks[0].index[i]);
+            MP4Index *next=&(_tracks[0].index[i+1]);
+            if(dex->dts==ADM_NO_PTS) continue;
+            if(next->dts==ADM_NO_PTS) continue;
+            uint64_t delta=next->dts-dex->dts;
+            if(delta==minDelta) score++;
+            if(delta<halfway) score++;
+        }
+        float weighted=score*1000.;
+        weighted/=n;
+        ADM_info("Original fps1000 = %d, score = %d, weighted score = %d\n",_videostream.dwRate,score,(int)weighted);
+        // require that at least 10% of video better or 5% perfectly matches max fps
+        if(fps1000 > _videostream.dwRate && (int)weighted > 100)
         {
             ADM_info("Adjusting fps, the computed is higher than average, dropped frames ?\n");
            _videostream.dwRate=fps1000;
@@ -182,8 +207,16 @@ MP4Index *idx=&(VDEO.index[framenum]);
     uint64_t offset=idx->offset; //+_mdatOffset;
 
 
-    fseeko(_fd,offset,SEEK_SET);
-    fread(img->data, idx->size, 1, _fd);
+    if(fseeko(_fd,offset,SEEK_SET))
+    {
+        ADM_error("Seeking past the end of the file! Broken index?\n");
+        return 0;
+    }
+    if(!fread(img->data, (size_t)idx->size, 1, _fd))
+    {
+        ADM_error("Incomplete frame %" PRIu32". Broken index?\n",framenum);
+        return 0;
+    }
     img->dataLength=idx->size;
 	img->flags = idx->intra;
 
@@ -272,7 +305,6 @@ uint8_t      MP4Header::getNbAudioStreams(void)
 
 uint8_t   MP4Header::getExtraHeaderData(uint32_t *len, uint8_t **data)
 {
-uint32_t old;
         *len=0;*data=NULL;
         if(_tracks[0].extraDataSize)
         {
@@ -339,7 +371,7 @@ uint8_t    MP4Header::open(const char *name)
                                           if(of>fileSize) of=hi;
                                         }
                                         fseeko(_fd,of,SEEK_SET);
-                                        printf("Header starts at %"PRIx64"\n",of);
+                                        printf("Header starts at %" PRIx64"\n",of);
                                         delete atom;
                                         atom=new adm_atom(_fd);
         }
@@ -348,7 +380,7 @@ uint8_t    MP4Header::open(const char *name)
         if(!lookupMainAtoms((void*) atom))
         {
           printf("Cannot find needed atom\n");   
-          if(!_tracks[0].fragments.size() || !indexVideoFragments(0)) // fixme audio
+          if(!_tracks[0].fragments.size() || !indexVideoFragments(0))
           {
             fclose(_fd);
             _fd=NULL;
@@ -360,6 +392,8 @@ uint8_t    MP4Header::open(const char *name)
               {
                   if(_tracks[i].fragments.size())
                       indexAudioFragments(i);
+                  if(_tracks[i].index==NULL)
+                      nbAudioTrack--;
               }
           }
         }
@@ -394,7 +428,7 @@ uint8_t    MP4Header::open(const char *name)
                 uint32_t w,h,ti;
                 if(extractMpeg4Info(VDEO.extraData,VDEO.extraDataSize,&w,&h,&ti))
                 {
-                    printf("MP4 Corrected size : %"PRIu32" x %"PRIu32"\n",w,h);
+                    printf("MP4 Corrected size : %" PRIu32" x %" PRIu32"\n",w,h);
                     _video_bih.biWidth=_mainaviheader.dwWidth=w ;
                     _video_bih.biHeight=_mainaviheader.dwHeight=h;
                 }
@@ -421,13 +455,13 @@ uint8_t    MP4Header::open(const char *name)
                         {
                         if(extractH263Info(bfer,sz,&w,&h))
                         {
-                            printf("H263 Corrected size : %"PRIu32" x %"PRIu32"\n",w,h);
+                            printf("H263 Corrected size : %" PRIu32" x %" PRIu32"\n",w,h);
                             _video_bih.biWidth=_mainaviheader.dwWidth=w ;
                             _video_bih.biHeight=_mainaviheader.dwHeight=h;
                         }
                         else
                         {
-                                  printf("H263 COULD NOT EXTRACT SIZE, using : %"PRIu32" x %"PRIu32"\n",
+                                  printf("H263 COULD NOT EXTRACT SIZE, using : %" PRIu32" x %" PRIu32"\n",
                                       _video_bih.biWidth,  _video_bih.biHeight);
                         }
                         }
@@ -511,20 +545,39 @@ uint8_t    MP4Header::open(const char *name)
             audioStream[audio]=ADM_audioCreateStream(&(_tracks[1+audio]._rdWav), audioAccess[audio]);
         }
         fseeko(_fd,0,SEEK_SET);
-        refineFps();
         uint64_t duration1=_movieDuration*1000LL;
-        uint64_t duration2=_tracks[0].index[nb-1].pts;
-        
-        ADM_info("3gp/mov file successfully read..\n");
-        if(duration2!=ADM_NO_PTS && duration2>duration1)
+        uint64_t duration2=0;
+        uint32_t lastFrame=0;
+        for(int i=nb-32;i<nb;i++)
         {
-            ADM_warning("Last PTS is after movie duration, increasing movie duration\n");
-            _movieDuration=(_tracks[0].index[nb-1].pts/1000)+1;
+            if(i<0) continue;
+            if(_tracks[0].index[i].pts==ADM_NO_PTS) continue;
+            if(duration2<_tracks[0].index[i].pts)
+            {
+                duration2=_tracks[0].index[i].pts;
+                lastFrame=i;
+            }
         }
-        ADM_info("Nb images      : %d\n",nb);
-        ADM_info("Movie duration : %s\n",ADM_us2plain(_movieDuration*1000LL));
-        ADM_info("Last video PTS : %s\n",ADM_us2plain(_tracks[0].index[nb-1].pts));
-        ADM_info("Last video DTS : %s\n",ADM_us2plain(_tracks[0].index[nb-1].dts));
+        ADM_info("3gp/mov file successfully read..\n");
+        if(duration2!=ADM_NO_PTS && duration2>=duration1)
+        { // video duration must be > max PTS, otherwise we drop the last frame
+            ADM_warning("Last PTS is at or after movie duration, increasing movie duration\n");
+            _movieDuration=(duration2/1000)+1;
+            // adjust calculated average FPS and time increment
+            double f=_movieDuration;
+            f=1000.*_tracks[0].nbIndex/f;
+            f*=1000.;
+            _videostream.dwRate=(uint32_t)floor(f+0.49);
+            _mainaviheader.dwMicroSecPerFrame=ADM_UsecFromFps1000(_videostream.dwRate);
+            ADM_info("Adjusted fps1000: %d = %" PRIu64" us per frame.\n",_videostream.dwRate,_mainaviheader.dwMicroSecPerFrame);
+        }
+        refineFps();
+        if(nb>1 && !lastFrame)
+            lastFrame=nb-1;
+        ADM_info("Nb images       : %d\n",nb);
+        ADM_info("Movie duration  : %s\n",ADM_us2plain(_movieDuration*1000LL));
+        ADM_info("Last video PTS  : %s\n",ADM_us2plain(_tracks[0].index[lastFrame].pts));
+        ADM_info("Last video DTS  : %s\n",ADM_us2plain(_tracks[0].index[nb-1].dts));
 
         checkDuplicatedPts();
         
@@ -557,15 +610,36 @@ bool MP4Header::checkDuplicatedPts(void)
  * \fn adjustElstDelay
  * @return 
  */
-bool MP4Header::adjustElstDelay()
+bool MP4Header::adjustElstDelay(void)
 {
-    if(this->delayRelativeToVideo)
+    int xmin=10000000;
+    int xscaledDelay[_3GP_MAX_TRACKS];
+    for(int i=0;i<1+nbAudioTrack;i++)
     {
-        ADM_info("Compensating for a/v delay\n");
-        shiftAudioTimeBy(this->delayRelativeToVideo);
+        double scaledDelay=_tracks[i].delay;
+        double scaledStartOffset=_tracks[i].startOffset;
+        scaledDelay/=_movieScale;
+        scaledStartOffset/=_tracks[i].scale;
+        scaledDelay*=1000000;
+        scaledStartOffset*=1000000;
+        ADM_info("Delay for track %d : raw = %d, scaled  = %d with scale = %d\n",i,_tracks[i].delay,(int)scaledDelay,_movieScale);
+        ADM_info("Start offset for track %d : raw = %d, scaled = %d with scale = %d\n",i,_tracks[i].startOffset,(int)scaledStartOffset,_tracks[i].scale);
+        scaledDelay-=scaledStartOffset;
+        xscaledDelay[i]=scaledDelay;
+        if(scaledDelay<xmin)
+            xmin=scaledDelay;
+    }
+    ADM_info("Elst minimum = %d us\n",xmin);
+    for(int i=0;i<1+nbAudioTrack;i++)
+    {
+        int d=xscaledDelay[i]-xmin;
+        if(d)
+        {
+            ADM_info("    Shifting track %d by %s\n",i,ADM_us2plain(d));
+            shiftTrackByTime(i,d);
+        }
     }
     return true;
-
 }
         
     
@@ -589,25 +663,37 @@ bool MP4Header::shiftTimeBy(uint64_t shift)
             pts+=shift;
             VDEO.index[i].pts=pts;
         }
-
-        shiftAudioTimeBy(shift);
+        for(int i=1;i<nbAudioTrack;i++)            
+            shiftTrackByTime(i,shift);
         return true;        
 }
-bool MP4Header::shiftAudioTimeBy(uint64_t shift)
+/**
+ * 
+ * @param dex
+ * @param shift
+ * @return 
+ */
+bool MP4Header::shiftTrackByTime(int dex,uint64_t shift)
 {
-    int nb;
-      for(int audioTrack=0;audioTrack<nbAudioTrack;audioTrack++)
-        {
-            nb=(int)_tracks[1+audioTrack].nbIndex;
-            for(int i=0;i<nb;i++)
-            {
-                     uint64_t dts;
-                        dts=_tracks[audioTrack+1].index[i].dts;
-                        if(dts==ADM_COMPRESSED_NO_PTS) continue;
-                        dts+=shift;
-                        _tracks[audioTrack+1].index[i].dts=dts;
-             }
-        }
+    int nb=(int)_tracks[dex].nbIndex;
+    MP4Index *myIndex=_tracks[dex].index;
+    for(int i=0;i<nb;i++)
+    {
+             uint64_t dts,pts;
+                dts=myIndex[i].dts;
+                pts=myIndex[i].pts;
+                if(dts!=ADM_COMPRESSED_NO_PTS)
+                {
+                    dts+=shift;
+                }
+                if(pts!=ADM_COMPRESSED_NO_PTS)
+                {
+                    pts+=shift;
+                }
+                myIndex[i].dts=dts;
+                myIndex[i].pts=pts;
+     }
+
     return true;
 }
 
@@ -653,7 +739,7 @@ bool    MP4Header::getPtsDts(uint32_t frame,uint64_t *pts,uint64_t *dts)
 
     if(frame>=VDEO.nbIndex)
     {
-      printf("[MP4] Frame %"PRIu32" exceeds # of frames %"PRIu32"\n",frame,VDEO.nbIndex);
+      printf("[MP4] Frame %" PRIu32" exceeds # of frames %" PRIu32"\n",frame,VDEO.nbIndex);
       return false;
     }
 
@@ -670,7 +756,7 @@ bool    MP4Header::setPtsDts(uint32_t frame,uint64_t pts,uint64_t dts)
 {
     if(frame>=VDEO.nbIndex)
     {
-      printf("[MP4] Frame %"PRIu32" exceeds # of frames %"PRIu32"\n",frame,VDEO.nbIndex);
+      printf("[MP4] Frame %" PRIu32" exceeds # of frames %" PRIu32"\n",frame,VDEO.nbIndex);
       return 0;
     }
 
