@@ -25,6 +25,7 @@
 #include "ADM_coreDemuxerMpegTemplate.cpp.h"
 
 uint32_t ADM_UsecFromFps1000(uint32_t fps1000);
+uint8_t psIndexer(const char *file);
 
 /**
       \fn open
@@ -34,15 +35,35 @@ uint32_t ADM_UsecFromFps1000(uint32_t fps1000);
 uint8_t psHeader::open(const char *name)
 {
     char *idxName=(char *)malloc(strlen(name)+6);
-    bool r=false;
+    uint8_t r=1;
+
+    sprintf(idxName,"%s.idx2",name);
+    if(!ADM_fileExist(idxName))
+        r=psIndexer(name);
+    if(r==ADM_IGN)
+    {
+        ADM_warning("Indexing cancelled by the user, deleting the index file. Bye.\n");
+        if(!ADM_eraseFile(idxName))
+            ADM_warning("Could not delete %s\n",idxName);
+        free(idxName);
+        return r;
+    }
+    if(!r)
+    {
+        ADM_error("Indexing of %s failed, aborting\n",name);
+        // Currently, indexer returns 0 only if it can't create the .idx2 file, nothing to remove.
+        free(idxName);
+        return r;
+    }
+
     FP_TYPE appendType=FP_DONT_APPEND;
-    uint32_t append;
     char *type;
     uint64_t startDts;
     uint32_t version=0;
-
-    sprintf(idxName,"%s.idx2",name);
+    bool reindex=false;
     indexFile index;
+    r=0;
+
     if(!index.open(idxName))
     {
         printf("[psDemux] Cannot open index file %s\n",idxName);
@@ -58,7 +79,8 @@ uint8_t psHeader::open(const char *name)
     version=index.getAsUint32("Version");
     if(version!=ADM_INDEX_FILE_VERSION)
     {
-        GUI_Error_HIG(QT_TRANSLATE_NOOP("psdemuxer","Error"), QT_TRANSLATE_NOOP("psdemuxer","This file's index has been created with an older version of avidemux.\nPlease delete the idx2 file and reopen."));
+        if(GUI_Question(QT_TRANSLATE_NOOP("psdemuxer","This file's index has been created with an older version of avidemux.\nThe file must be re-indexed. Proceed?")))
+            reindex=true;
         goto abt;
     }
     type=index.getAsString("Type");
@@ -67,7 +89,7 @@ uint8_t psHeader::open(const char *name)
         printf("[psDemux] Incorrect or not found type\n");
         goto abt;
     }
-    append=index.getAsUint32("Append");
+    append=(bool)index.getAsUint32("Append");
     printf("[psDemux] Append=%" PRIu32"\n",append);
     if(append) appendType=FP_APPEND;
     if(!parser.open(name,&appendType))
@@ -128,7 +150,7 @@ uint8_t psHeader::open(const char *name)
         printf("psDemux] Cannot psPacket open the file\n");
         goto abt;
     }
-    r=true;
+    r=1;
     for(int i=0;i<listOfAudioTracks.size();i++)
     {
         ADM_psTrackDescriptor *desc=listOfAudioTracks[i];
@@ -143,8 +165,22 @@ uint8_t psHeader::open(const char *name)
     }
 abt:
     index.close();
-    free(idxName);
-    printf("[psDemuxer] Loaded %d\n",r);
+    if(reindex)
+    {
+        uint8_t success=ADM_eraseFile(idxName);
+        free(idxName);
+        if(success)
+            r=open(name);
+        else
+            ADM_error("Can't delete old index file.\n");
+    }else
+    {
+        free(idxName);
+        if(r)
+            ADM_info("Loaded %s successfully\n",name);
+        else
+            ADM_warning("Loading %s failed\n",name);
+    }
     return r;
 }
 
@@ -216,7 +252,7 @@ uint8_t  psHeader::getFrame(uint32_t frame,ADMCompressedImage *img)
 {
     if(frame>=ListOfFrames.size()) return 0;
     dmxFrame *pk=ListOfFrames[frame];
-    if(frame==(lastFrame+1) && pk->type!=1)
+    if(frame==(lastFrame+1) && pk->type!=1) // the next frame, not an intra
     {
         lastFrame++;
         bool r=psPacket->read(pk->len,img->data);
@@ -228,7 +264,7 @@ uint8_t  psHeader::getFrame(uint32_t frame,ADMCompressedImage *img)
              getFlags(frame,&(img->flags));
              return r;
     }
-    if(pk->type==1)
+    if(pk->type==1) // an intra
     {
         if(!psPacket->seek(pk->startAt,pk->index)) return false;
          bool r=psPacket->read(pk->len,img->data);
@@ -240,10 +276,45 @@ uint8_t  psHeader::getFrame(uint32_t frame,ADMCompressedImage *img)
              //printf("[>>>] %d:%02x %02x %02x %02x\n",frame,img->data[0],img->data[1],img->data[2],img->data[3]);
              lastFrame=frame;
              return r;
-
     }
-    printf(" [PsDemux] lastFrame :%d this frame :%d\n",lastFrame,frame);
-    return false;
+
+    // a random frame: need to rewind first, then seek forward
+    uint32_t startPoint=frame;
+    while(startPoint && ListOfFrames[startPoint]->type!=1)
+        startPoint--;
+    printf("[psDemux] Wanted frame %" PRIu32", going back to frame %" PRIu32", last frame was %" PRIu32",\n",frame,startPoint,lastFrame);
+    pk=ListOfFrames[startPoint];
+    if(!psPacket->seek(pk->startAt,pk->index))
+    {
+        printf("[psDemux] Failed to rewind to frame %" PRIu32"\n",startPoint);
+        return false;
+    }
+
+    // now seek forward
+    while(startPoint<frame)
+    {
+        pk=ListOfFrames[startPoint];
+        if(!psPacket->read(pk->len,img->data))
+        {
+            printf("[psDemux] Read failed for frame %" PRIu32"\n",startPoint);
+            lastFrame=0xffffffff;
+            return false;
+        }
+        lastFrame=startPoint;
+        startPoint++;
+    }
+    pk=ListOfFrames[frame];
+    lastFrame++;
+
+    bool r=psPacket->read(pk->len,img->data);
+
+    img->dataLength=pk->len;
+    img->demuxerFrameNo=frame;
+    img->demuxerDts=pk->dts;
+    img->demuxerPts=pk->pts;
+    //printf("[>>>] %d:%02x %02x %02x %02x\n",frame,img->data[0],img->data[1],img->data[2],img->data[3]);
+    getFlags(frame,&(img->flags));
+    return r;
 }
 
 /**
