@@ -27,8 +27,8 @@ typedef struct
 typedef BVector <psAudioTrackInfo*> listOfPsAudioTracks;
 
 /// Returns a list of audio tracks found in the file.
-listOfPsAudioTracks *psProbeAudio(const char *fileName);
+listOfPsAudioTracks *psProbeAudio(const char *fileName, FP_TYPE appendType);
 bool DestroyListOfPsAudioTracks(listOfPsAudioTracks *list);
 
 
-#endif
\ No newline at end of file
+#endif
