@@ -122,7 +122,7 @@ protected:
 public:
                 PsIndexer(void);
                 ~PsIndexer();
-        bool    run(const char *file);
+        uint8_t run(const char *file);
         bool    writeVideo(PSVideo *video);
         bool    writeAudio(void);
         bool    writeSystem(const char *filename,bool append);
@@ -141,7 +141,7 @@ public:
 */
 uint8_t   psIndexer(const char *file)
 {
-bool r;
+uint8_t r;
     PsIndexer *dx=new PsIndexer;
     r=dx->run(file);
     delete dx;
@@ -176,11 +176,12 @@ PsIndexer::~PsIndexer()
 /**
     \fn run
 */  
-bool PsIndexer::run(const char *file)
+uint8_t PsIndexer::run(const char *file)
 {
 uint32_t temporal_ref,val;
 uint64_t fullSize;
 uint8_t buffer[50*1024];
+uint8_t res=1;
 bool seq_found=false;
 
 PSVideo video;
@@ -213,7 +214,7 @@ bool bAppend=false;
     writeSystem(file,bAppend);
     pkt=new psPacketLinearTracker(0xE0);
 
-    audioTracks=psProbeAudio(file);
+    audioTracks=psProbeAudio(file,append);
     if(audioTracks)
     {
         for(int i=0;i<audioTracks->size();i++)
@@ -234,7 +235,11 @@ bool bAppend=false;
     fullSize=pkt->getSize();
       while(1)
       {
-        
+        if(!ui->isAlive())
+        {
+            res=ADM_IGN;
+            break;
+        }
         uint8_t startCode=pkt->findStartCode();
         if(!pkt->stillOk()) break;
         pkt->getInfo(&info);
@@ -335,11 +340,14 @@ bool bAppend=false;
 
                             }
 
+#define PREAMBLE qfprintf(index,"[Data]\n"); \
+                 qfprintf(index,"# Warning, picture structure markers F/T/B are shifted one image to the right.\n"); \
+                 qfprintf(index,"# To compensate for that, they are shifted one image to the left in readIndex.");
+
                           if(!seq_found) continue;
                           if(headerDumped==false)
                           {
-                               
-                                qfprintf(index,"[Data]");
+                                PREAMBLE
                                 headerDumped=true;
                           }
                           if(data.state==idx_startAtGopOrSeq) 
@@ -361,8 +369,7 @@ bool bAppend=false;
                           }
                           if(headerDumped==false)
                           {
-                               
-                                qfprintf(index,"[Data]");
+                                PREAMBLE
                                 headerDumped=true;
                           }
                           val=pkt->readi16();
@@ -404,6 +411,7 @@ bool bAppend=false;
                     break;
                   }
       }
+      if(res==ADM_IGN) goto cleanup;
 theEnd:    
         printf("\n");
         // Dump progressive/frame gop
@@ -419,6 +427,7 @@ theEnd:
         writeAudio();
         writeScrReset();
         qfprintf(index,"\n[End]\n");
+cleanup:
         qfclose(index);
         index=NULL;
         if(audioTracks) DestroyListOfPsAudioTracks( audioTracks);
@@ -426,7 +435,7 @@ theEnd:
         delete pkt;
         pkt=NULL;
 	free(indexName);
-        return 1; 
+        return res;
 }
 /**
     \fn   Mark
