@@ -53,7 +53,6 @@ FP_TYPE fp=FP_DONT_APPEND;
             memcpy(extraData,myExtra,extraDataLen);
             ADM_info("Creating ts audio access with %d bytes of extradata.",myLen);
             mixDump(extraData,extraDataLen);
-            ADM_info("\n");
         }
 }
 
@@ -86,8 +85,9 @@ ADM_mpgAudioSeekPoint s;
 */
 uint32_t  ADM_tsAccess::getLength(void)
 {
-  return (seekPoints[seekPoints.size()-1].size);
-
+    if(seekPoints.size())
+        return (seekPoints[seekPoints.size()-1].size);
+    return 0;
 }
 /**
     \fn getDurationInUs
@@ -116,11 +116,13 @@ uint64_t  ADM_tsAccess::getDurationInUs(void)
 */                              
 bool      ADM_tsAccess::goToTime(uint64_t timeUs)
 {
-    // Convert time in us to scaled 90 kHz tick
     latm.flush();
+    if(!seekPoints.size())
+        return false;
+
     if(timeUs<seekPoints[0].dts)
     {
-            aprintf("[PsAudio] Requested %"PRIu32" tick before 1st seek point at :%"PRIu32"\n",(uint32_t)timeUs/1000,(uint32_t)seekPoints[0].dts/1000);
+            aprintf("[PsAudio] Requested %" PRIu32" tick before 1st seek point at :%" PRIu32"\n",(uint32_t)timeUs/1000,(uint32_t)seekPoints[0].dts/1000);
             demuxer.setPos(seekPoints[0].position);
             return true;
     }
@@ -129,7 +131,7 @@ bool      ADM_tsAccess::goToTime(uint64_t timeUs)
     {
         if(seekPoints[i].dts >=timeUs )
         {
-            aprintf("[PsAudio] Requested %"PRIu32" tick seeking to  at :%"PRIu32" us (next is %"PRIu32"ms \n",(uint32_t)timeUs/1000,
+            aprintf("[PsAudio] Requested %" PRIu32" tick seeking to  at :%" PRIu32" us (next is %" PRIu32"ms \n",(uint32_t)timeUs/1000,
                     (uint32_t)seekPoints[i-1].dts/1000,
                     (uint32_t)seekPoints[i].dts/1000);
             demuxer.setPos(seekPoints[i-1].position);
@@ -232,7 +234,7 @@ uint64_t p,d,start;
      }
     if(*dts!=ADM_NO_PTS) 
     {
-        aprintf("[psAudio] getPacket dts = %"PRIu32" ms\n",(uint32_t)*dts/1000);
+        aprintf("[psAudio] getPacket dts = %" PRIu32" ms\n",(uint32_t)*dts/1000);
     }
     return true;
 }
