@@ -59,6 +59,7 @@ uint64_t pts,dts,startAt;
         {
             case ADM_TS_MPEG_AUDIO:
             case ADM_TS_AC3:
+            case ADM_TS_DTS:
             case ADM_TS_EAC3:
             case ADM_TS_AAC_ADTS:
             case ADM_TS_AAC_LATM:
@@ -112,7 +113,7 @@ again:
                 case ADM_TS_AAC_LATM:
                 {
                     ADM_latm2aac latm;
-                    ADM_info("Looking up LATM info");
+                    ADM_info("Looking up LATM info\n");
                     retries=20;
                     while(retries)
                     {
@@ -123,7 +124,9 @@ again:
                         if(latm.getFrequency())
                         {
                             ADM_assert(latm.getExtraData(&eLen,&eData));
-                            trackInfo->wav.frequency=latm.getFrequency();
+                            uint32_t fq=latm.getFrequency();
+                            if(fq<=24000) fq*=2; // implicit SBR?
+                            trackInfo->wav.frequency=fq;
                             trackInfo->wav.channels=latm.getChannels();
                             trackInfo->wav.byterate=128000>>3;
                             trackInfo->extraDataLen=eLen;
@@ -159,7 +162,7 @@ again:
                     adts.getExtraData(&eLen,&eData);
                     if( eLen!=2) 
                     {
-                        ADM_error("%d bytes of extradata, expecting 2");
+                        ADM_error("%d bytes of extradata, expecting 2\n");
                         return false;
                     }
                     trackInfo->extraDataLen=eLen;
@@ -180,7 +183,7 @@ again:
         int rd=PROBE_ANALYZE_SIZE;
         if(!p->read(PROBE_ANALYZE_SIZE,audioBuffer))
         {
-            printf("[tsAudioProbe] Cannot get info about pid %d 0x%x\n",trackInfo->esId,trackInfo->esId);
+            ADM_error("Cannot get info about pid %d 0x%x\n",trackInfo->esId,trackInfo->esId);
             return false;
         }
         uint32_t fq,br,chan,off;
@@ -190,7 +193,7 @@ again:
                             {
                                 if(! tsCheckMp2Audio(&(trackInfo->wav),audioBuffer,rd))
                                 {
-                                    printf("[PsProbeAudio] Failed to get info on track :%x (MP2)\n",trackInfo->esId);
+                                    ADM_error("Failed to get info on track : 0x%x (MP2)\n",trackInfo->esId);
                                     goto er;
                                 }
                             }
@@ -201,28 +204,41 @@ again:
                                 trackInfo->wav.channels=2;
                                 trackInfo->wav.byterate=2*2*48000;
                                 break;
-            case ADM_TS_AC3: // AC3 or DTS
-                            // AC3
+            case ADM_TS_AC3:
                             {
                                 trackInfo->wav.encoding=WAV_AC3;
                                 if(!ADM_AC3GetInfo(audioBuffer, rd, &fq, &br, &chan,&off))
                                 {
-                                        printf("[PsProbeAudio] Failed to get info on track :%x\n",trackInfo->esId);
-                                        goto er;
+                                    ADM_error("Failed to get info on track : 0x%x (AC3)\n",trackInfo->esId);
+                                    goto er;
                                 }
                                 trackInfo->wav.frequency=fq;
                                 trackInfo->wav.channels=chan;
                                 trackInfo->wav.byterate=(br);
                                 break;
                             }
+            case ADM_TS_DTS:
+                            {
+                                trackInfo->wav.encoding=WAV_DTS;
+                                ADM_DCA_INFO dcainfo;
+                                if(false==ADM_DCAGetInfo(audioBuffer, rd, &dcainfo, &off))
+                                {
+                                    ADM_error("Failed to get info on track : 0x%x (DTS)\n",trackInfo->esId);
+                                    goto er;
+                                }
+                                trackInfo->wav.frequency=dcainfo.frequency;
+                                trackInfo->wav.channels=dcainfo.channels;
+                                trackInfo->wav.byterate=dcainfo.bitrate/8;
+                                break;
+                            }
             case ADM_TS_EAC3: 
                             {
                                 trackInfo->wav.encoding=WAV_EAC3;
                                 ADM_EAC3_INFO info;
                                 if(!ADM_EAC3GetInfo(audioBuffer, rd, &off,&info))
                                 {
-                                        printf("[PsProbeAudio] Failed to get info on track :%x\n",trackInfo->esId);
-                                        goto er;
+                                    ADM_error("Failed to get info on track : 0x%x (EAC3)\n",trackInfo->esId);
+                                    goto er;
                                 }
                                 trackInfo->wav.frequency=info.frequency;
                                 trackInfo->wav.channels=info.channels;
@@ -231,7 +247,7 @@ again:
                             }
                             
             default:
-                        printf("[tsAudioProbe] Unsupported audio format pid %d 0x%x\n",trackInfo->esId,trackInfo->esId);
+                        ADM_error("Unsupported audio format pid %d (0x%x)\n",trackInfo->esId,trackInfo->esId);
                         return false;
 
         }
