@@ -26,7 +26,7 @@
 TODO / FIXME : Handle wrap
 TODO / FIXME : Handle PTS reordering 
 */
-uint64_t wrapIt(uint64_t val, uint64_t start)
+static uint64_t wrapIt(uint64_t val, uint64_t start)
 {
     if(val==ADM_NO_PTS) return val;
     if(val>=start) return val-start;
@@ -126,6 +126,8 @@ bool tsHeader::updatePtsDts(void)
         // We are sure to have both PTS & DTS for 1st image
         // Guess missing DTS/PTS for video
         int noUpdate=0;
+        startDts=ListOfFrames[0]->dts;
+        ListOfFrames[0]->dts=0;
         for(int i=0;i<ListOfFrames.size();i++)
         {
             dmxFrame *frame=ListOfFrames[i];
@@ -160,12 +162,13 @@ bool tsHeader::updatePtsDts(void)
             }else    // We got both, use them  
 #endif
             {
-                
-                frame->dts=lastDts=timeConvert(frame->dts);
+                if(i) // We may not modify the dts of the first frame yet.
+                    frame->dts=lastDts=timeConvert(frame->dts);
                 frame->pts=lastPts=timeConvert(frame->pts);
             }
-
         }
+        // Now take care of the first frame dts.
+        ListOfFrames[0]->dts=timeConvert(startDts);
         // convert to us for Audio tracks (seek points)
         for(int i=0;i<listOfAudioTracks.size();i++)
         {
