@@ -43,8 +43,7 @@ static const uint32_t FPS[16]={
 */
 uint8_t   tsIndexer(const char *file)
 {
-bool r;
-
+    uint8_t r;
     ADM_TS_TRACK *tracks;
     uint32_t nbTracks;
     listOfTsAudioTracks audioTrack;
@@ -55,7 +54,7 @@ bool r;
             if(TS_guessContent(file,&nbTracks,&tracks)==false) 
             {
                 printf("[Ts Indexer] Brute force scan failed\n");
-                return false;
+                return 0;
             }
     }
     ADM_assert(tracks);
@@ -102,7 +101,7 @@ bool r;
     if(!dx)
     {
         ADM_warning("Unsupported video codec \n");
-        r=false;
+        r=0;
     }else
     {
         r=dx->run( file,&(tracks[0]));
@@ -248,8 +247,9 @@ bool TsIndexerBase::dumpUnits(indexerData &data,uint64_t nextConsumed,const dmxP
                 case unitTypeSps: mustFlush=true;;break;
                 case unitTypePic: 
                             picIndex=i;
+                            pictStruct=listOfUnits[i].imageStructure;
                             if(listOfUnits[i].imageType==1 || listOfUnits[i].imageType==4)
-                            mustFlush=true;
+                                mustFlush=true;
                             break;
                 case unitTypeSei:
                             pictStruct=listOfUnits[i].imageStructure;
@@ -335,6 +335,30 @@ bool TsIndexerBase::addUnit(indexerData &data,int unitType2,const H264Unit &unit
         listOfUnits.push_back(myUnit);
         return true;
 }
+
+bool    TsIndexerBase::updateLastUnitStructure(int t)
+{
+    int n=listOfUnits.size();
+    if(!n)
+    {
+        ADM_error("Cannot update last pic, we have none.");
+        return false;
+    }
+    H264Unit &lastUnit=listOfUnits[n-1];
+    switch(t)
+    {
+        case 3: 
+                lastUnit.imageStructure=pictureFrame;
+                break;
+        case 1:  lastUnit.imageStructure=pictureTopField;
+                 break;
+        case 2:  lastUnit.imageStructure=pictureBottomField;
+                 break;
+        default: ADM_warning("frame type 0 met, this is illegal\n");
+    }
+    return true;
+}
+
 /********************************************************************************************/
 /********************************************************************************************/
 /********************************************************************************************/
