@@ -85,79 +85,72 @@ bool TsIndexerH264::findH264SPS(tsPacketLinearTracker *pkt,TSVideo &video)
     TS_PESpacket SEI_nal(0);
     while(keepRunning)
     {
-      int startCode=pkt->findStartCode();
+        int startCode=pkt->findStartCode();
 
-      if(!pkt->stillOk())
-      {
-          keepRunning=false;
-          continue;
-      }
-      if(startCode&0x80) continue; // Marker missing
-      startCode&=0x1f;
-      if(startCode!=NAL_SPS) 
-          continue;
+        if(!pkt->stillOk())
+        {
+            keepRunning=false;
+            continue;
+        }
+        if(startCode&0x80) continue; // Marker missing
+        startCode&=0x1f;
+        if(startCode!=NAL_SPS)
+            continue;
 
         // Got SPS!
-
-        uint32_t xA,xR;
         // Get info
         pkt->getInfo(&tmpInfo);
         // Read just enough...
+        SEI_nal.empty();
+        uint32_t code=0xffff+0xffff0000;
+        while(((code&0xffffff)!=1) && pkt->stillOk())
+        {
+            uint8_t r=pkt->readi8();
+            code=(code<<8)+r;
+            SEI_nal.pushByte(r);
+        }
+        if(!pkt->stillOk()) break;
+        pkt->seek(tmpInfo.startAt,tmpInfo.offset-5);
+        if (extractSPSInfo(SEI_nal.payload, SEI_nal.payloadSize-3,&spsInfo))
         {
-          SEI_nal.empty();
-          uint32_t code=0xffff+0xffff0000;
-          while(((code&0xffffff)!=1) && pkt->stillOk())
-          {
-                  uint8_t r=pkt->readi8();
-                  code=(code<<8)+r;
-                  SEI_nal.pushByte(r);
-          }
-          if(!pkt->stillOk()) break;;
-          pkt->seek(tmpInfo.startAt,tmpInfo.offset-5);
-          if (extractSPSInfo(SEI_nal.payload, SEI_nal.payloadSize-4,&spsInfo))
-          {
             ADM_info("[TsIndexer] Found video %" PRIu32"x%" PRIu32", fps=%" PRIu32"\n",video.w,video.h,video.fps);
             ADM_info("[TsIndexer] SPS says %" PRIu32"x%" PRIu32"\n",spsInfo.width,spsInfo.height);
             seq_found=1;
             video.w=spsInfo.width;
             video.h=spsInfo.height;
             video.fps=spsInfo.fps1000;
-            xA=spsInfo.darNum;
-            xR=spsInfo.darDen;
             writeVideo(&video,ADM_TS_H264);
             writeAudio();
             qfprintf(index,"[Data]");
             // Rewind
 
             break;
-        };
-      }
+        }
+        pkt->seek(tmpInfo.startAt,tmpInfo.offset);
     }
     return seq_found;
 }
 /**
-    \fn runH264
+    \fn run
     \brief Index H264 stream
 */
-bool TsIndexerH264::run(const char *file,ADM_TS_TRACK *videoTrac)
+uint8_t TsIndexerH264::run(const char *file, ADM_TS_TRACK *videoTrac)
 {
+    bool seq_found=false;
+    bool firstSps=true;
+    TS_PESpacket SEI_nal(0);
+    TSVideo video;
+    indexerData data;
 
-bool    seq_found=false;
-bool    firstSps=true;
-TS_PESpacket SEI_nal(0);
-TSVideo video;
-indexerData  data;
-
-bool result=false;
-bool bAppend=false;
+    uint8_t result=0;
+    bool bAppend=false;
 
     beginConsuming=0;
     listOfUnits.clear();
 
     printf("Starting H264 indexer\n");
     if(!videoTrac) return false;
-    if(videoTrac[0].trackType!=ADM_TS_H264 
-       )
+    if(videoTrac[0].trackType!=ADM_TS_H264)
     {
         printf("[Ts Indexer] Only H264 video supported\n");
         return false;
@@ -172,21 +165,27 @@ bool bAppend=false;
 
     if(!index)
     {
-        printf("[PsIndex] Cannot create %s\n",indexName.c_str());
-        return false;
+        ADM_error("[TsIndexerH264] Cannot create %s\n",indexName.c_str());
+        return 0;
     }
 
+    uint64_t lastAudOffset=0;
+    int audCount=0;
+    int audStartCodeLen=5;
+    dmxPacketInfo packetInfo;
 
     pkt=new tsPacketLinearTracker(videoTrac->trackPid, audioTracks);
 
     FP_TYPE append=FP_DONT_APPEND;
+#ifdef ASK_APPEND_SEQUENCED
     if(true==ADM_probeSequencedFile(file))
     {
         if(true==GUI_Question(QT_TRANSLATE_NOOP("tsdemuxer","There are several files with sequential file names. Should they be all loaded ?")))
-                bAppend=true;
+            bAppend=true;
     }
     if(bAppend==true)
         append=FP_APPEND;
+#endif
     writeSystem(file,bAppend);
     pkt->open(file,append);
     data.pkt=pkt;
@@ -207,14 +206,14 @@ bool bAppend=false;
     }    
     if(!seq_found) goto the_end;
 
-        decodingImage=false;
+    decodingImage=false;
     //******************
     // 2 Index
     //******************
-        bool fourBytes;
-      while(keepRunning)
-      {
-          fourBytes=false;
+    bool fourBytes;
+    while(keepRunning)
+    {
+        fourBytes=false;
         int startCode=pkt->findStartCode2(fourBytes);
 resume:
         if(!pkt->stillOk()) break;
@@ -235,133 +234,182 @@ resume:
 
         aprintf("[%02x] Nal :0x%x,ref=%d,lastRef=%d at : %d \n",fullStartCode,startCode,ref,lastRefIdc,pkt->getConsumed()-beginConsuming);
 
-          // Ignore multiple chunk of the same pic
-          if((startCode==NAL_NON_IDR || startCode==NAL_IDR)&&decodingImage )
-          {
+        // Ignore multiple chunk of the same pic
+        if((startCode==NAL_NON_IDR || startCode==NAL_IDR)&&decodingImage )
+        {
             aprintf("Still capturing, ignore\n");
             continue;
-          }
+        }
 
-          switch(startCode)
-                  {
-                  case NAL_AU_DELIMITER:
-                        {
-                          aprintf("AU DELIMITER\n");
-                          decodingImage = false;
-                        }
-                          break;
-                  case NAL_SEI:
-                        {
-                        // Load the whole NAL
-                            SEI_nal.empty();
-                            uint32_t code=0xffff+0xffff0000;
-                            while(((0xffffff&code)!=1) && pkt->stillOk())
-                            {
-                                uint8_t r=pkt->readi8();
-                                code=(code<<8)+r;
-                                SEI_nal.pushByte(r);
-                            }
-                            if(!pkt->stillOk()) goto resume;
-                            aprintf("[SEI] Nal size :%d\n",SEI_nal.payloadSize);
-                            if(SEI_nal.payloadSize>=7)
-                                decodeSEI(SEI_nal.payloadSize-4,
-                                    SEI_nal.payload,&(thisUnit.recoveryCount),&(thisUnit.imageStructure));
-                            else
-                                    printf("[SEI] Too short size+4=%d\n",*(SEI_nal.payload));
-                            startCode=pkt->readi8();
+        switch(startCode)
+        {
+            case NAL_AU_DELIMITER:
+            {
+                aprintf("AU DELIMITER\n");
+                decodingImage = false;
+                pkt->getInfo(&packetInfo,startCodeLength);
+                lastAudOffset=pkt->getConsumed();
+                audStartCodeLen=startCodeLength;
+                audCount++;
+            }
+                break;
+            case NAL_SEI:
+            {
+                if(!audCount)
+                {
+                    pkt->getInfo(&packetInfo,startCodeLength);
+                    thisUnit.consumedSoFar=pkt->getConsumed();
+                }else
+                {
+                    thisUnit.consumedSoFar=lastAudOffset;
+                    startCodeLength=audStartCodeLen;
+                }
+                thisUnit.packetInfo=packetInfo;
+                // Load the whole NAL
+                SEI_nal.empty();
+                uint32_t code=0xffff+0xffff0000;
+                while(((0xffffff&code)!=1) && pkt->stillOk())
+                {
+                    uint8_t r=pkt->readi8();
+                    code=(code<<8)+r;
+                    SEI_nal.pushByte(r);
+                }
+                if(!pkt->stillOk()) goto resume;
+                aprintf("[SEI] Nal size :%d\n",SEI_nal.payloadSize);
+                if(SEI_nal.payloadSize>=7)
+                    decodeSEI(SEI_nal.payloadSize-4,
+                              SEI_nal.payload,
+                              &(thisUnit.recoveryCount),
+                              &(thisUnit.imageStructure));
+                else
+                    printf("[SEI] Too short size+4=%d\n",*(SEI_nal.payload));
+                startCode=pkt->readi8();
 
-                            decodingImage=false;
-                            pkt->getInfo(&thisUnit.packetInfo);
-                            thisUnit.consumedSoFar=pkt->getConsumed();
-                            if(!addUnit(data,unitTypeSei,thisUnit,startCodeLength+SEI_nal.payloadSize+1))
-                                keepRunning=false;
-                            fourBytes=true;
-                            goto resume;
-                            }
-                            break;
+                decodingImage=false;
+                if(!addUnit(data,unitTypeSei,thisUnit,startCodeLength))
+                {
+                    result=ADM_IGN;
+                    goto the_end;
+                }
+                fourBytes=true;
+                goto resume;
+            }
+                break;
 
-                  case NAL_SPS:
-                                decodingImage=false;
-                                pkt->getInfo(&thisUnit.packetInfo);
-                                if(firstSps)
-                                {
-                                    pkt->setConsumed(startCodeLength); // reset consume counter
-                                    firstSps=false;
-                                }
-                                thisUnit.consumedSoFar=pkt->getConsumed();
-                                if(!addUnit(data,unitTypeSps,thisUnit,startCodeLength))
-                                    keepRunning=false;
-                          break;
-
-                  case NAL_IDR:
-                  case NAL_NON_IDR:
+            case NAL_SPS:
+            {
+                decodingImage=false;
+                if(!audCount)
+                    pkt->getInfo(&packetInfo,startCodeLength);
+                thisUnit.packetInfo=packetInfo;
+                if(firstSps)
+                {
+                    uint64_t pos=0;
+                    if(audCount) // Currently always false because we don't rewind after findH264SPS()
                     {
-#define NON_IDR_PRE_READ 8
-                      aprintf("Pic start last ref:%d cur ref:%d nb=%d\n",lastRefIdc,ref,data.nbPics);
-                      lastRefIdc=ref;
-
-                      uint8_t bufr[NON_IDR_PRE_READ+4];
-                      uint8_t header[NON_IDR_PRE_READ+4];
-
-
-                        pkt->read(NON_IDR_PRE_READ,bufr);
-                        // unescape...
-                        ADM_unescapeH264(NON_IDR_PRE_READ,bufr,header);
-                        //
-                        getBits bits(NON_IDR_PRE_READ,header);
-                        int first_mb_in_slice,slice_type;
-
-                        first_mb_in_slice= bits.getUEG();
-                        slice_type= bits.getUEG31();
-                        if(slice_type>9)
-                        {
-                            printf("[TsIndexer] Bad slice type\n");
-                        }
-                        if(slice_type>4) slice_type-=5;
-                        switch(slice_type)
-                        {
-
-                            case 0 : thisUnit.imageType=2;break; // P
-                            case 1 : thisUnit.imageType=3;break; // B
-                            case 2 : thisUnit.imageType=1;break; // I
-                            default : thisUnit.imageType=2;break; // SP/SI
-                        }
-                      if(startCode==NAL_IDR) thisUnit.imageType=4; // IDR
-                      aprintf("[>>>>>>>>] Pic Type %" PRIu32" Recovery %" PRIu32"\n",thisUnit.imageType,recoveryCount);
-                      if(thisUnit.imageType==1 && !thisUnit.recoveryCount)
-                                thisUnit.imageType=4; //I  + Recovery=0 = IDR!
-
-                      data.nbPics++;
-
-
-
-                      decodingImage=true;
-                      pkt->getInfo(&thisUnit.packetInfo);
-                      thisUnit.consumedSoFar=pkt->getConsumed();
-
-                      if(!addUnit(data,unitTypePic,thisUnit,startCodeLength+NON_IDR_PRE_READ))
-                          keepRunning=false;
-                        // reset to default
-                      thisUnit.imageStructure=pictureFrame;
-                      thisUnit.recoveryCount=0xff;
-                      pkt->invalidatePtsDts();
+                        pos+=pkt->getConsumed();
+                        pos-=lastAudOffset;
+                        startCodeLength=audStartCodeLen;
                     }
+                    pos+=startCodeLength;
+                    pkt->setConsumed(pos); // reset consume counter
+                    thisUnit.consumedSoFar=pos;
+                    firstSps=false;
+                }else
+                {
+                    if(audCount)
+                    {
+                        thisUnit.consumedSoFar=lastAudOffset;
+                        startCodeLength=audStartCodeLen;
+                    }else
+                    {
+                        thisUnit.consumedSoFar=pkt->getConsumed();
+                    }
+                }
+                if(!addUnit(data,unitTypeSps,thisUnit,startCodeLength))
+                {
+                    result=ADM_IGN;
+                    goto the_end;
+                }
+            }
+                break;
 
-                    break;
-                  default:
-                      break;
-          }
-      } // End while
-      result=true;
+            case NAL_IDR:
+            case NAL_NON_IDR:
+            {
+                if(!audCount)
+                {
+                    pkt->getInfo(&packetInfo,startCodeLength);
+                    thisUnit.consumedSoFar=pkt->getConsumed();
+                }else
+                {
+                    thisUnit.consumedSoFar=lastAudOffset;
+                    startCodeLength=audStartCodeLen;
+                }
+                thisUnit.packetInfo=packetInfo;
+#define NON_IDR_PRE_READ 8
+                aprintf("Pic start last ref:%d cur ref:%d nb=%d\n",lastRefIdc,ref,data.nbPics);
+                lastRefIdc=ref;
+
+                uint8_t bufr[NON_IDR_PRE_READ+4];
+                uint8_t header[NON_IDR_PRE_READ+4];
+
+
+                pkt->read(NON_IDR_PRE_READ,bufr);
+                // unescape...
+                ADM_unescapeH264(NON_IDR_PRE_READ,bufr,header);
+                //
+                getBits bits(NON_IDR_PRE_READ,header);
+                int first_mb_in_slice,slice_type;
+
+                first_mb_in_slice= bits.getUEG();
+                slice_type= bits.getUEG31();
+                if(slice_type>9)
+                {
+                    printf("[TsIndexer] Bad slice type\n");
+                }
+                if(slice_type>4) slice_type-=5;
+                switch(slice_type)
+                {
+                    case 0 : thisUnit.imageType=2;break; // P
+                    case 1 : thisUnit.imageType=3;break; // B
+                    case 2 : thisUnit.imageType=1;break; // I
+                    default: thisUnit.imageType=2;break; // SP/SI
+                }
+                if(startCode==NAL_IDR) thisUnit.imageType=4; // IDR
+                aprintf("[>>>>>>>>] Pic Type %" PRIu32" Recovery %" PRIu32"\n",thisUnit.imageType,recoveryCount);
+                if(thisUnit.imageType==1 && !thisUnit.recoveryCount)
+                    thisUnit.imageType=4; //I  + Recovery=0 = IDR!
+
+                data.nbPics++;
+                decodingImage=true;
+                audCount=0;
+
+                if(!addUnit(data,unitTypePic,thisUnit,startCodeLength))
+                {
+                    result=ADM_IGN;
+                    goto the_end;
+                }
+                // reset to default
+                thisUnit.imageStructure=pictureFrame;
+                thisUnit.recoveryCount=0xff;
+                pkt->invalidatePtsDts();
+            }
+                break;
+            default:
+                break;
+        }
+    } // End while
+    result=1;
 the_end:
-        printf("\n");
-        qfprintf(index,"\n[End]\n");
-        qfclose(index);
-        index=NULL;
-        audioTracks=NULL;
-        delete pkt;
-        pkt=NULL;
-        return result;
+    printf("\n");
+    qfprintf(index,"\n[End]\n");
+    qfclose(index);
+    index=NULL;
+    audioTracks=NULL;
+    delete pkt;
+    pkt=NULL;
+    return result;
 }
 
 
