@@ -16,6 +16,12 @@
 #include "ADM_tsIndex.h"
 #include "ADM_h265_tag.h"
 #include "ADM_vidMisc.h"
+
+#if 1
+#define aprintf(...) {}
+#else
+#define aprintf printf
+#endif
 /**
  * 
  * @param sc
@@ -100,15 +106,14 @@ bool TsIndexerH265::decodeSEIH265(uint32_t nalSize, uint8_t *org,uint32_t *recov
  */
 static bool findGivenStartCode(tsPacketLinearTracker *pkt,int match, const char *name)
 {
-    bool keepRunning=true;    
-    while(keepRunning)
+    while(true)
     {
       int startCode=pkt->findStartCode();
       if(!pkt->stillOk())
       {
           return false;
       }     
-      printf("Match %x %d\n",startCode,((startCode>>1)&0x3f));
+      aprintf("Match %x %d\n",startCode,((startCode>>1)&0x3f));
       startCode=((startCode>>1)&0x3f);  
       
       if(startCode!=match && match) 
@@ -136,7 +141,7 @@ static uint8_t * findGivenStartCodeInBuffer(uint8_t *start, uint8_t *end,int mat
         if(!start[0]&&!start[1] && start[2]==0x01)
         {
             uint8_t code=(start[3]>>1)&0x3f;
-            printf(" Matcho = %d\n",code);
+            aprintf(" Matcho = %d\n",code);
             if(code==match || !match) return start;
         }
         start++;
@@ -151,7 +156,6 @@ static uint8_t * findGivenStartCodeInBuffer(uint8_t *start, uint8_t *end,int mat
  */
 bool TsIndexerH265::findH265VPS(tsPacketLinearTracker *pkt,TSVideo &video)
 {    
-    bool keepRunning=true;
     dmxPacketInfo packetInfo;
     uint8_t headerBuffer[512+5]={0,0,0,1,(NAL_H265_VPS<<1)}; // we are forcing some bits to be zero...
     // This is a bit naive...
@@ -243,11 +247,11 @@ int  TsIndexerH265::decodePictureTypeH265(int nalType,getBits &bits)
             segmentFlag=bits.get(1);
         }
         int address=bits.get(info.address_coding_length); //  log2 ( width*height/64*64)
-        printf("Adr=%d / %d\n",address,64*34);
+        aprintf("Adr=%d / %d\n",address,64*34);
     }
     if(segmentFlag)
     {
-        printf("Nope\n");
+        aprintf("Nope\n");
         return -1; 
     }
     
@@ -269,32 +273,23 @@ int  TsIndexerH265::decodePictureTypeH265(int nalType,getBits &bits)
                 ADM_warning("Unknown slice type %d \n",sliceType);
                 break;
     }
-    printf("SliceType==> %d xxx\n",slice);
+    aprintf("SliceType==> %d xxx\n",slice);
     return slice;
 }
 /**
-    \fn runH264
-    \brief Index H264 stream
+    \fn run
+    \brief Index H265 stream
 */
-bool TsIndexerH265::run(const char *file,ADM_TS_TRACK *videoTrac)
+uint8_t TsIndexerH265::run(const char *file,ADM_TS_TRACK *videoTrac)
 {
+    TSVideo video;
+    indexerData data;
 
-bool    seq_found=false;
-bool    firstSps=true;
-TS_PESpacket SEI_nal(0);
-TSVideo video;
-indexerData  data;
-
-bool result=false;
-bool bAppend=false;
-
-    beginConsuming=0;
     listOfUnits.clear();
 
-    printf("Starting H264 indexer\n");
+    printf("Starting H265 indexer\n");
     if(!videoTrac) return false;
-    if(videoTrac[0].trackType!=ADM_TS_H265 
-       )
+    if(videoTrac[0].trackType!=ADM_TS_H265)
     {
         printf("[Ts Indexer] Only H265 video supported\n");
         return false;
@@ -309,14 +304,22 @@ bool bAppend=false;
 
     if(!index)
     {
-        printf("[PsIndex] Cannot create %s\n",indexName.c_str());
+        printf("[TsIndexerH265] Cannot create %s\n",indexName.c_str());
         return false;
     }
 
+    uint8_t result=0;
+    bool bAppend=false;
+    bool seq_found=false;
+    bool firstSps=true;
+    uint64_t lastAudOffset=0;
+    int audCount=0;
+    dmxPacketInfo packetInfo;
 
     pkt=new tsPacketLinearTracker(videoTrac->trackPid, audioTracks);
 
     FP_TYPE append=FP_DONT_APPEND;
+#ifdef ASK_APPEND_SEQUENCED
     if(true==ADM_probeSequencedFile(file))
     {
         if(true==GUI_Question(QT_TRANSLATE_NOOP("tsdemuxer","There are several files with sequential file names. Should they be all loaded ?")))
@@ -324,13 +327,13 @@ bool bAppend=false;
     }
     if(bAppend==true)
         append=FP_APPEND;
+#endif
     writeSystem(file,bAppend);
     pkt->open(file,append);
     data.pkt=pkt;
     fullSize=pkt->getSize();
     gui=createProcessing(QT_TRANSLATE_NOOP("tsdemuxer","Indexing"),pkt->getSize());
     int lastRefIdc=0;
-    bool keepRunning=true;
     //******************
     // 1 search SPS
     //******************
@@ -343,109 +346,126 @@ bool bAppend=false;
             break;
     }    
     if(!seq_found) goto the_end;
-
-    
-
     
      decodingImage=false;
     //******************
     // 2 Index
     //******************
-      bool fourBytes;
-      while(keepRunning)
-      {
-          fourBytes=false;
+    bool fourBytes;
+    while(true)
+    {
+        fourBytes=false;
         int startCode=pkt->findStartCode2(fourBytes);
-resume:
         if(!pkt->stillOk()) break;
 
         int startCodeLength=4;
         if(fourBytes==true) startCodeLength++;
 
         startCode=((startCode>>1)&0x3f);   
-        
-        {
-          dmxPacketInfo packetInfo;
-          pkt->getInfo(&packetInfo);
-          printf("Startcode =%d:%s, decoding image=%d,%s\n",startCode,startCodeToString(startCode),decodingImage,ADM_us2plain(packetInfo.dts));
-        }
+
+#if 0
+        pkt->getInfo(&packetInfo);
+        aprintf("Startcode =%d:%s, decoding image=%d,%s\n",startCode,startCodeToString(startCode),decodingImage,ADM_us2plain(packetInfo.dts));
+#endif
 #define NON_IDR_PRE_READ 32 
 
-          switch(startCode)
-                  {
-                  case NAL_H265_AUD:
-                        {
-                          aprintf("AU DELIMITER\n");
-                          decodingImage = false;
-                        }
-                          break;
-                case NAL_H265_TRAIL_R:
-                case NAL_H265_TRAIL_N:
-                case NAL_H265_TSA_N:
-                case NAL_H265_TSA_R:
-                case NAL_H265_STSA_N:
-                case NAL_H265_STSA_R:
-                case NAL_H265_BLA_W_LP:
-                case NAL_H265_BLA_W_RADL:
-                case NAL_H265_BLA_N_LP:
-                case NAL_H265_IDR_W_RADL:
-                case NAL_H265_IDR_N_LP:
-                case NAL_H265_CRA_NUT:
-                case NAL_H265_RADL_N:
-                case NAL_H265_RADL_R:
-                case NAL_H265_RASL_N:
-                case NAL_H265_RASL_R:
+        switch(startCode)
+        {
+            case NAL_H265_AUD:
+            {
+                aprintf("AU DELIMITER\n");
+                decodingImage = false;
+                pkt->getInfo(&packetInfo);
+                lastAudOffset=pkt->getConsumed();
+                audCount++;
+                break;
+            }
+            case NAL_H265_TRAIL_R:
+            case NAL_H265_TRAIL_N:
+            case NAL_H265_TSA_N:
+            case NAL_H265_TSA_R:
+            case NAL_H265_STSA_N:
+            case NAL_H265_STSA_R:
+            case NAL_H265_BLA_W_LP:
+            case NAL_H265_BLA_W_RADL:
+            case NAL_H265_BLA_N_LP:
+            case NAL_H265_IDR_W_RADL:
+            case NAL_H265_IDR_N_LP:
+            case NAL_H265_CRA_NUT:
+            case NAL_H265_RADL_N:
+            case NAL_H265_RADL_R:
+            case NAL_H265_RASL_N:
+            case NAL_H265_RASL_R:
+            {
+                uint8_t buffer[NON_IDR_PRE_READ],header[NON_IDR_PRE_READ];
+                int preRead=NON_IDR_PRE_READ;
+                if(!audCount)
                 {
-#if 0
-                    if(decodingImage)
-                        continue;
-#endif                    
-                    uint8_t buffer[NON_IDR_PRE_READ],header[NON_IDR_PRE_READ];
-                    int preRead=NON_IDR_PRE_READ;
-                    dmxPacketInfo packetInfo;
-                        pkt->getInfo(&packetInfo,startCodeLength);
-                        thisUnit.consumedSoFar=pkt->getConsumed();
-                        
-                        // Read the beginning of the picture to get its type...
-                        pkt->read(preRead,buffer);
-                        ADM_unescapeH264(preRead,buffer,header);
-                        //
-                        getBits bits(preRead,header);
-                        // Try to see if we have a valid beginning of image
-                        int picType=decodePictureTypeH265(startCode,bits);
-                        if(picType!=-1)
-                        {
-                            data.nbPics++;
-                            decodingImage=true;
-                            thisUnit.packetInfo=packetInfo;
-                            thisUnit.imageType=picType;
-                            thisUnit.unitType=unitTypePic;
-                            if(!addUnit(data,unitTypePic,thisUnit,startCodeLength))
-                                keepRunning=false;
-                            // reset to default
-                            thisUnit.imageStructure=pictureFrame;
-                            thisUnit.recoveryCount=0xff;
-                            pkt->invalidatePtsDts();
-                        }
+                    pkt->getInfo(&packetInfo,startCodeLength);
+                    thisUnit.consumedSoFar=pkt->getConsumed();
+                }else
+                {
+                    thisUnit.consumedSoFar=lastAudOffset;
                 }
-                    break;
-                  case NAL_H265_VPS:
-                        decodingImage=false;
-                        pkt->getInfo(&thisUnit.packetInfo,startCodeLength);
-                        if(firstSps)
-                        {
-                            pkt->setConsumed(startCodeLength); // reset consume counter
-                            firstSps=false;
-                        }
-                        thisUnit.consumedSoFar=pkt->getConsumed();
-                        if(!addUnit(data,unitTypeSps,thisUnit,startCodeLength))
-                            keepRunning=false;
-                        break;      
-                  default:
-                      break;
-          }
-      } // End while
-      result=true;
+                // Read the beginning of the picture to get its type...
+                pkt->read(preRead,buffer);
+                ADM_unescapeH264(preRead,buffer,header);
+                getBits bits(preRead,header);
+                // Try to see if we have a valid beginning of image
+                int picType=decodePictureTypeH265(startCode,bits);
+                if(picType!=-1)
+                {
+                    data.nbPics++;
+                    decodingImage=true;
+                    thisUnit.packetInfo=packetInfo;
+                    thisUnit.imageType=picType;
+                    thisUnit.unitType=unitTypePic;
+                    if(!addUnit(data,unitTypePic,thisUnit,startCodeLength))
+                    {
+                        result=ADM_IGN;
+                        goto the_end;
+                    }
+                    // reset to default
+                    thisUnit.imageStructure=pictureFrame;
+                    thisUnit.recoveryCount=0xff;
+                    pkt->invalidatePtsDts();
+                    audCount=0;
+                }
+                break;
+            }
+            case NAL_H265_VPS:
+            {
+                decodingImage=false;
+                if(!audCount)
+                    pkt->getInfo(&packetInfo,startCodeLength);
+                thisUnit.packetInfo=packetInfo;
+                if(firstSps)
+                {
+                    uint64_t pos=startCodeLength;
+                    if(audCount)
+                    {
+                        pos+=pkt->getConsumed();
+                        pos-=lastAudOffset;
+                    }
+                    pkt->setConsumed(pos); // reset consume counter
+                    thisUnit.consumedSoFar=pos;
+                    firstSps=false;
+                }else
+                {
+                    thisUnit.consumedSoFar=lastAudOffset;
+                }
+                if(!addUnit(data,unitTypeSps,thisUnit,startCodeLength))
+                {
+                    result=ADM_IGN;
+                    goto the_end;
+                }
+                break;
+            }
+            default:
+                break;
+        }
+    } // End while
+    result=1;
 the_end:
         printf("\n");
         qfprintf(index,"\n[End]\n");
