@@ -23,8 +23,6 @@
 #include "dmxTSPacket.h"
 #include "dmx_mpegstartcode.h"
 
-#define TS_PES_MAX_LIMIT (1024*1024*2)
-
 #include "ADM_getbits.h"
 #include "ADM_tsCrc.cpp"
 
@@ -80,12 +78,13 @@ bool tsPacket::open(const char *filenames,FP_TYPE append)
     // Detect TS1/TS2...
         printf("[TsPacket] Detecting TS/TS2...\n");
 
-        uint8_t r;
+#define MAX_BOGUS_TS_MARKERS 4
+    for(int i=0;i<MAX_BOGUS_TS_MARKERS;i++)
+    {
         int tryMe=250;
         while(tryMe--)
         {
-            r=_file->read8i();
-            if(r==TS_MARKER) break;
+            if(_file->read8i()==TS_MARKER) break;
             if(_file->end()) 
             {
                 tryMe=0;
@@ -104,7 +103,14 @@ bool tsPacket::open(const char *filenames,FP_TYPE append)
         setPos(startPos);
         score2=score(_file,20,4);
         printf("[TsPacket] Score : 188:%d, 192:%d out of 20\n",score1,score2);
-        if(score2 && score2>score1)
+        if(score1==1 && score2==1)
+        {
+            startPos++;
+            ADM_info("Probably bogus sync byte detection, retrying at offset %" PRIu64"\n",startPos);
+            setPos(startPos);
+            continue;
+        }
+        if(score2>score1)
         {
             printf("[TsPacket] Probably TS2 (192)...\n");
             extraCrap=4;
@@ -112,8 +118,10 @@ bool tsPacket::open(const char *filenames,FP_TYPE append)
         {
             printf("[TsPacket] Probably TS1 (188)...\n");
         }
-        setPos(0);
-        return true;
+        break;
+    }
+    setPos(0);
+    return true;
 }
 /**
     \fn close
@@ -155,7 +163,7 @@ bool    tsPacket::setPos(uint64_t pos)
 */
 bool tsPacket::getSinglePacket(uint8_t *buffer)
 {
-#define MAX_SKIPPED_PACKET 150*100 // 150*180*100=~ 2.2 MBytes
+#define MAX_SKIPPED_PACKET 300*100 // 150*180*100=~ 2.2 MBytes
 #define MAX_SEARCH 2048
     int count=0;
 again:
@@ -286,12 +294,20 @@ bool        tsPacket::updateStats(uint8_t *data)
 #endif
 bool tsPacket::getNextPSI(uint32_t pid,TS_PSIpacketInfo *psi)
 {
+    int multiPacketPsi=0;
+    int nbRetries=0;
+    uint64_t startOffset=0;
     TSpacketInfo pkt;
 nextPack2:
-
+    if(nbRetries && pkt.startAt-startOffset>(1<<25)) // max. 32 MiB
+    {
+        ADM_warning("Giving up after %d retries, consumed %" PRId64" bytes\n",nbRetries,pkt.startAt-startOffset);
+        return false;
+    }
     if(false==getNextPacket_NoHeader(pid,&pkt,true)) return false;    
-
-    
+    if(!nbRetries)
+        startOffset=pkt.startAt;
+    nbRetries++;
     uint32_t tableId;
     uint32_t sectionLength=0;
     uint32_t transportStreamId=0;
@@ -301,21 +317,28 @@ nextPack2:
 
     DUMMY(tableId,8);
     int section_syntax_indicator=bits.get(1);
-    if(section_syntax_indicator)
-        ADM_warning("Section Syntax is set to private\n");
-    if(bits.get(1))             // Marker
+#ifdef VERBOSE_PSI
+    if(!section_syntax_indicator)
+        ADM_warning("Syntax section indicator not set\n");
+#endif
+    if(bits.get(1)) // Private bit
     {
-          printf("[MpegTs] getNextPSI Missing 0 marker\n");
-          goto nextPack2;
+        ADM_warning("Section syntax is set to private\n");
+        goto nextPack2;
     }
-    bits.get(2);
-    sectionLength=bits.get(12);    // Section Length
+    bits.get(2); // 2 Reserved bits
+    sectionLength=bits.get(12); // Section Length
 #if 1
     if(sectionLength+3>pkt.payloadSize) 
     {
-        ADM_warning("[MpegTs] Multi Packet PSI ? sectionLength=%d, len=%d\n",sectionLength,pkt.payloadSize);
+        if(!multiPacketPsi)
+            ADM_warning("[MpegTs] Multi Packet PSI ? sectionLength=%d, len=%d\n",sectionLength,pkt.payloadSize);
+        multiPacketPsi++;
         goto nextPack2;
     }
+    if(multiPacketPsi>1)
+        ADM_warning("Multi packet PSI warning repeated %d times\n",multiPacketPsi);
+    multiPacketPsi=0;
 #endif
     transportStreamId=bits.get(16);// transportStreamId
 #ifdef VERBOSE_PSI
@@ -362,13 +385,24 @@ bool        tsPacket::getNextPES(TS_PESpacket *pes)
 #if 1
 #define zprintf(...) {}
 #else
-#define zprintf printf
+#define zprintf ADM_info
 #endif
+    int nbRetries=0,counter=0;
+    uint64_t startOffset=0;
     TSpacketInfo pkt;
+    pkt.startAt=0;
     pes->fresh=false;
 nextPack3:
+    if(pkt.startAt-startOffset>(1<<25)) // max. 32 MiB
+    {
+        ADM_warning("Giving up after %d retries, consumed %" PRId64" bytes\n",nbRetries,pkt.startAt-startOffset);
+        return false;
+    }
     // Sync at source
-    if(false==getNextPacket_NoHeader(pes->pid,&pkt,false)) return false;    
+    if(false==getNextPacket_NoHeader(pes->pid,&pkt,false)) return false;
+    if(!nbRetries)
+        startOffset=pkt.startAt;
+    nbRetries++;
     // If it does not contain a payload start continue
     bool mark=false;
     uint32_t code=(pkt.payload[0]<<24)+(pkt.payload[1]<<16)+(pkt.payload[2]<<8)+pkt.payload[3];
@@ -376,9 +410,15 @@ nextPack3:
     zprintf("Mark=%x\n",code);
     if(!pkt.payloadStart && !mark)
     {
-        printf("[Ts Demuxer] Pes for Pid =0x%d does not contain payload start\n",pes->pid);
+#if 0
+        printf("[Ts Demuxer] Pes for Pid = 0x%x (%d) does not contain payload start\n",pes->pid,pes->pid);
+#endif
+        counter++;
         goto nextPack3;
     }
+    if(counter>1)
+        zprintf("Payload start for Pid = 0x%x (%d) found after %d retries\n",pes->pid,pes->pid,counter);
+    counter=0;
     //____________________
     //____________________
     // 1- Start Headers
@@ -387,10 +427,14 @@ nextPack3:
     zprintf("[TS Demuxer] Code=0x%x pid=0x%x\n",code,pes->pid);
     if((code&0xffffff00)!=0x100)
     {
+#if 0
         printf("[Ts Demuxer] No PES startcode at 0x%" PRIx64"\n",pkt.startAt);
         printf("0x:%02x %02x %02x %02x\n",pkt.payload[4],pkt.payload[5],pkt.payload[6],pkt.payload[7]);
+#endif
         goto nextPack3;
     }
+    if(nbRetries>1)
+        ADM_info("PES startcode found at 0x%" PRIx64" after %d retries\n",pkt.startAt,nbRetries);
     //mixDump(pkt.payload,pkt.payloadSize);
     uint32_t pesPacketLen=(pkt.payload[4]<<8)+(pkt.payload[5]);
     zprintf("[TS Demuxer] Pes Packet Len=%d\n",pesPacketLen);
@@ -425,7 +469,7 @@ nextPack3:
         goto nextPack3;
 
     //
-    zprintf("[Ts Demuxer] Found PES of len %d\n",pes->payloadSize);  
+    zprintf("[Ts Demuxer] Found PES of len %d, offset=%d\n",pes->payloadSize, pes->offset);  
     pes->fresh=true;
     
     return true;
@@ -433,7 +477,7 @@ nextPack3:
 /**
     \fn decodePesHeader
 */
-#define fail(x) {printf("[Ts Demuxer]*********" x"*******\n");return false;}
+#define fail(x) {ADM_warning("[Ts Demuxer]*********" x"*******\n");return false;}
 bool tsPacket::decodePesHeader(TS_PESpacket *pes)
 {
     uint8_t  *start=pes->payload+6;
@@ -448,7 +492,7 @@ bool tsPacket::decodePesHeader(TS_PESpacket *pes)
 
     if(pes->payloadSize<(4+2+1+2)) 
     {
-            printf("[Ts] Pes size too small\n");
+            ADM_warning("[Ts] Pes size too small\n");
             return false;
     }
     while(*start==0xff && start<end) start++; // Padding
@@ -523,15 +567,21 @@ bool tsPacket::decodePesHeader(TS_PESpacket *pes)
             {
                 tail=sizeCheck-packLen;
                 pes->payloadSize-=tail; 
-                printf("[TS Packet]extra crap at the end %d\n",tail);
+                ADM_warning("[TS Packet]extra crap at the end %d\n",tail);
             }
             else
                 if(packLen>sizeCheck)
                 {
-                    printf("[TS Packet] PackLen=%d, avalailble=%d\n",packLen,sizeCheck);
+                    ADM_warning("[TS Packet] PackLen=%d, avalailble=%d\n",packLen,sizeCheck);
                     fail("Pes too long");
                 }   
         }
+        zprintf("[decodePesHeader] payloadSize=%d, offset=%d \n",pes->payloadSize,pes->offset);
+        if(pes->offset>pes->payloadSize)
+        {
+            ADM_warning("[decodePesHeader] Inconsistent size, dropping\n");
+            return false;
+        }
         return true;
 }
 /**
@@ -614,7 +664,7 @@ bool tsPacketLinear::refill(void)
         oldBufferLen=pesPacket->payloadSize;
         if(false==getNextPES(pesPacket))
         {
-                printf("[tsPacketLinear] Refill failed for pid :%x\n",pesPacket->pid);
+                printf("[tsPacketLinear] Refill failed for pid : 0x%x (%d)\n",pesPacket->pid,pesPacket->pid);
                 return false;
         }
         return true;
