@@ -17,7 +17,9 @@ using std::vector;
 #define TS_MARKER       0x47
 #define TS_PACKET_LEN   188
 #define TS_PSI_MAX_LEN  1024
-#define TS_PES_MAX_LEN  (10*1024)
+// TS_PES_MAX_LIMIT represents an arbitrarily chosen payload size limit.
+// The largest size seen in the wild has been 2637792 in a 4k HEVC stream so far.
+#define TS_PES_MAX_LIMIT (1024*1024*3)
 /**
     \class TSpacketInfo
 */
@@ -52,6 +54,7 @@ public:
     \brief Handle exactly one PES packet
             It is assumed PES packet starts & ends at TS packet boundaries
 */
+#define TS_PES_PACKET_MIN_SIZE 32
 class TS_PESpacket
 {
 public:
@@ -80,7 +83,7 @@ public:
                 }
     bool        expandPayload()
 		{
-                        payloadLimit*=2;
+                        payloadLimit=payloadLimit*2+TS_PES_PACKET_MIN_SIZE;
 			uint8_t *newPayload=(uint8_t *)ADM_alloc(payloadLimit);
 			memcpy(newPayload,payload,payloadSize);
 			ADM_dealloc(payload);
@@ -90,7 +93,7 @@ public:
 		}
     bool        addData(uint32_t len,uint8_t *data)
                 {
-                    if(len+payloadSize>payloadLimit)
+                    if((len+payloadSize+TS_PES_PACKET_MIN_SIZE)>payloadLimit)
                     {
 				expandPayload();
                     }
@@ -105,7 +108,7 @@ public:
      }
      bool pushByte(uint8_t byte)
      {
-            if(payloadSize>=payloadLimit)
+            if((payloadSize+TS_PES_PACKET_MIN_SIZE)>=payloadLimit)
             {
 			expandPayload();
             }
@@ -165,7 +168,15 @@ public:
                 ~tsPacketLinear();
         uint32_t getConsumed(void);
         bool     stillOk(void) {return !eof;};
-        bool     invalidatePtsDts(void);
+        bool     invalidatePtsDts(int startCodeLength=4);
+ virtual bool    setPos(uint64_t pos)
+ {
+   pesPacket->payloadSize=0;
+   pesPacket->offset=0;
+   return tsPacket::setPos(pos);
+   
+ }
+ 
 #ifndef TS_PACKET_INLINE
         uint8_t  readi8();
         uint16_t readi16();
@@ -213,7 +224,7 @@ uint16_t readi16(void)
         bool    read(uint32_t len, uint8_t *buffer);
         bool    forward(uint32_t v);
         
-        bool    getInfo(dmxPacketInfo *info);
+        bool    getInfo(dmxPacketInfo *info,int startCodeLength=4);
         bool    seek(uint64_t packetStart, uint32_t offset);
         bool    changePid(uint32_t pid) ;
         bool    setConsumed(uint32_t v);
