@@ -308,16 +308,23 @@ VBRext   mp3vbr;
 	case WAV_WMA:
           {
             memset(extra,0,12);
+#if 0
             extra[16-16]=0x0a;
             extra[19-16]=0x08;
             extra[22-16]=0x01;
             extra[24-16]=0x74;
             extra[25-16]=01;
+#endif
 			header->dwScale 	    = wav->blockalign;
 			header->dwSampleSize 	= wav->blockalign;
 			header->dwInitialFrames =1;
 			header->dwSuggestedBufferSize=10*wav->blockalign;
 			*extraLen=12;
+            uint32_t aLen;
+            uint8_t *aData;
+            stream->getExtraData(&aLen,&aData);
+            extra[0]=0x0a;
+            memcpy(extra+2,aData,(aLen<10)? aLen : 10);
           }
 		  break;
     case WAV_PCM:
@@ -451,7 +458,7 @@ uint8_t aviWrite::saveBegin (
 	delete LMain;
 	LMain=NULL;
     for(int i=0;i<3;i++)
-        ADM_info("SuperIndex position so far %d : %"PRId64"\n",i,openDmlHeaderPosition[i]);
+        ADM_info("SuperIndex position so far %d : %" PRId64"\n",i,openDmlHeaderPosition[i]);
   //___________________________________
   // Write the beginning of the movie part
   //___________________________________
@@ -522,7 +529,7 @@ uint8_t aviWrite::setEnd (void)
   printf("\n End of movie, \n video frames : %u\n",vframe);
     for(int i=0;i<nb_audio;i++)
     {
-        printf("Track %d Size :%"PRIu32" bytes, %"PRIu32" blocks\n",i,audioTracks[i].sizeInBytes,audioTracks[i].nbBlocks);
+        printf("Track %d Size :%" PRIu32" bytes, %" PRIu32" blocks\n",i,audioTracks[i].sizeInBytes,audioTracks[i].nbBlocks);
     }
 
   // cleanup
@@ -560,7 +567,7 @@ bool aviWrite::setVideoStreamInfo (ADMFile * fo,
   // Place holder for odml super index
   uint64_t pos;
   alist->writeDummyChunk(AVI_SUPER_INDEX_CHUNK_SIZE,&pos);
-  printf("[ODML] videoTrack : using ODML placeholder of size %u bytes at pos 0x%"PRIx64"\n",AVI_SUPER_INDEX_CHUNK_SIZE,pos);  
+  printf("[ODML] videoTrack : using ODML placeholder of size %u bytes at pos 0x%" PRIx64"\n",AVI_SUPER_INDEX_CHUNK_SIZE,pos);  
   openDmlHeaderPosition[0]=pos;
   alist->End ();
   delete alist;
@@ -594,7 +601,7 @@ bool aviWrite::setAudioStreamInfo (ADMFile * fo,
   
   uint64_t pos;
   alist->writeDummyChunk(AVI_SUPER_INDEX_CHUNK_SIZE,&pos);
-  ADM_info("[ODML] Audio track %d, using ODML placeholder of size %u bytes, odmltrack=%d, pos=0x%"PRIx64"\n",
+  ADM_info("[ODML] Audio track %d, using ODML placeholder of size %u bytes, odmltrack=%d, pos=0x%" PRIx64"\n",
                             audioTrackNumber,AVI_SUPER_INDEX_CHUNK_SIZE,1+audioTrackNumber,pos);  
   openDmlHeaderPosition[1+audioTrackNumber]=pos;
   alist->End ();
