@@ -95,6 +95,8 @@ bool muxerWebm::open(const char *file, ADM_videoStream *s,uint32_t nbAudioTrack,
     
         AVCodecContext *c;
         c = video_st->codec;
+        AVCodecParameters *par;
+        par = video_st->codecpar;
         rescaleFps(s->getAvgFps1000(),&(c->time_base));
         video_st->time_base=c->time_base;
         c->gop_size=15;
@@ -104,8 +106,8 @@ bool muxerWebm::open(const char *file, ADM_videoStream *s,uint32_t nbAudioTrack,
             //sar=display/code  
             int num=1,den=1;
             av_reduce(&num, &den, WebmMuxerConfig.displayWidth, s->getWidth(),65535);
-            c->sample_aspect_ratio.num=num;
-            c->sample_aspect_ratio.den=den;
+            par->sample_aspect_ratio.num=num;
+            par->sample_aspect_ratio.den=den;
             video_st->sample_aspect_ratio.num=num;
             video_st->sample_aspect_ratio.den=den;
             ADM_info("Forcing display width of %d\n",(int)WebmMuxerConfig.displayWidth);
