@@ -44,9 +44,9 @@ extern "C"
         0*ADM_ENC_CAP_SAME \
     }, \
     99, /* Threads : auto */ \
-    NULL, /* Preset */ \
-    NULL, /* Tuning */ \
-    NULL, /* Profile */ \
+    std::string(""), /* Preset */ \
+    std::string(""), /* Tuning */ \
+    std::string(""), /* Profile */ \
     false, /* Fast decode */ \
     false, /* Zero latency */ \
     true /* Fast first pass */ \
@@ -108,7 +108,7 @@ extern "C"
 	1.0,	/* float rate_tolerance; */ \
 	0,	/* uint32_t vbv_max_bitrate; */ \
 	0,	/* uint32_t vbv_buffer_size; */ \
-	0.9,	/* uint32_t vbv_buffer_init; */ \
+	1,	/* uint32_t vbv_buffer_init; */ \
 	1.4,	/* float ip_factor; */ \
 	1.3,	/* float pb_factor; */ \
 	1,	/* uint32_t aq_mode; */ \
