@@ -128,6 +128,8 @@ x264Dialog::x264Dialog(QWidget *parent, void *param) : QDialog(parent)
         connect(ui.loopFilterCheckBox, SIGNAL(toggled(bool)), this, SLOT(loopFilterCheckBox_toggled(bool)));
         connect(ui.mbTreeCheckBox, SIGNAL(toggled(bool)), this, SLOT(mbTreeCheckBox_toggled(bool)));
         connect(ui.aqVarianceCheckBox, SIGNAL(toggled(bool)), this, SLOT(aqVarianceCheckBox_toggled(bool)));
+        connect(ui.trellisCheckBox, SIGNAL(toggled(bool)), this, SLOT(trellisCheckBox_toggled(bool)));
+        connect(ui.trellisComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(trellisComboBox_currentIndexChanged(int)));
 #if 0
         connect(ui.maxCrfSlider, SIGNAL(valueChanged(int)), this, SLOT(maxCrfSlider_valueChanged(int)));
         connect(ui.maxCrfSpinBox, SIGNAL(valueChanged(int)), this, SLOT(maxCrfSpinBox_valueChanged(int)));
@@ -185,28 +187,34 @@ x264Dialog::x264Dialog(QWidget *parent, void *param) : QDialog(parent)
         }
 
         upload();
-        ADM_pluginInstallSystem( std::string("x264"),std::string(".json"),pluginVersion);
+        ADM_pluginInstallSystem( std::string("x264"),std::string("json"),pluginVersion);
         updatePresetList();
-        int n=ui.configurationComboBox->count();
-        ui.configurationComboBox->setCurrentIndex(n-1);
+        adjustSize();
 }
 /**
     \fn updatePresetList
 */
-bool x264Dialog::updatePresetList(void)
+bool x264Dialog::updatePresetList(const char *match)
 {
     QComboBox *combo=ui.configurationComboBox;
     std::string rootPath;
     vector <std::string >  list;
     ADM_pluginGetPath("x264",pluginVersion,rootPath);
-    ADM_listFile(rootPath,".json",list);
+    ADM_listFile(rootPath,"json",list);
     int l=list.size();
+    int idx=l;
+    std::string current;
+    if(match)
+        current=std::string(match);
     combo->clear();
     for( int i=0;i<l;i++)
     {
+        if(match && list[i]==current)
+            idx=i;
         combo->addItem(list[i].c_str());
     }
     combo->addItem(QString(QT_TRANSLATE_NOOP("x264","Custom")));
+    combo->setCurrentIndex(idx);
     return true;
 }
 
@@ -672,7 +680,13 @@ void x264Dialog::quantiserSlider_valueChanged(int value)
 
 void x264Dialog::meSlider_valueChanged(int value)
 {
-        ui.meSpinBox->setValue(value);
+    ui.meSpinBox->setValue(value);
+    if(value>9)
+    {
+        ui.aqVarianceCheckBox->setChecked(true);
+        ui.trellisCheckBox->setChecked(true);
+        ui.trellisComboBox->setCurrentIndex(1);
+    }
 }
 void x264Dialog::quantiserSpinBox_valueChanged(int value)
 {
@@ -718,11 +732,31 @@ void x264Dialog::aqVarianceCheckBox_toggled(bool checked)
         {
                  QString st=QT_TRANSLATE_NOOP("x264","Macroblock-Tree optimisation requires Variance Adaptive Quantisation to be enabled.  Macroblock-Tree optimisation will automatically be disabled.\n\nDo you wish to continue?");
                 if (GUI_Question(st.toUtf8().constData()))
+                {
                         ui.mbTreeCheckBox->setChecked(false);
-                else
+                        if(ui.meSpinBox->value()>9)
+                                ui.meSpinBox->setValue(9);
+                }else
                         ui.aqVarianceCheckBox->setChecked(true);
         }
 }
+
+void x264Dialog::trellisCheckBox_toggled(bool checked)
+{
+    if(!checked && ui.meSpinBox->value()>9)
+    {
+        ui.meSpinBox->setValue(9);
+    }
+}
+
+void x264Dialog::trellisComboBox_currentIndexChanged(int index)
+{
+    if(index<1 && ui.meSpinBox->value()>9)
+    {
+        ui.meSpinBox->setValue(9);
+    }
+}
+
 #if 0
 void x264Dialog::maxCrfSlider_valueChanged(int value)
 {
@@ -769,9 +803,9 @@ void x264Dialog::configurationComboBox_currentIndexChanged(int index)
     \fn getProfileName  
     \brief Popup a dialog that asks the user the preset name
 */
-static char *getProfileName(void)
+static char *getProfileName(QDialog *parent)
 {
-  QDialog dialog;
+  QDialog dialog(parent);
   dialog.setWindowTitle(QString::fromUtf8(QT_TRANSLATE_NOOP("x264","Save Profile")));
   QDialogButtonBox *buttonBox = new QDialogButtonBox();  
   QVBoxLayout *vboxLayout = new QVBoxLayout();
@@ -783,22 +817,22 @@ static char *getProfileName(void)
   QLineEdit *text=new QLineEdit;
 //  text->setAcceptRichText(false);
 
-  text->setText(QT_TRANSLATE_NOOP("x264","my profile"));
+  text->setText("my profile");
   text->selectAll();
 
   vboxLayout->addWidget(text);
   vboxLayout->addWidget(buttonBox);
-
+//  dialog.setModal(true);
+//  dialog.setWindowModality(Qt::ApplicationModal);
   dialog.setLayout(vboxLayout);
 
   if(dialog.exec()!=QDialog::Accepted)
   {
-        ADM_info("Canceled");
+        ADM_info("Cancelled");
         return NULL;
   }
-  QString fileName=text->text();
-  const char *out=fileName.toUtf8().constData();
-  return ADM_strdup(out);
+  std::string st = std::string(text->text().toUtf8().constData());
+  return ADM_strdup(st.c_str());
 }
 /**
         \fn saveAsButton_pressed
@@ -807,13 +841,14 @@ static char *getProfileName(void)
 void x264Dialog::saveAsButton_pressed(void)
 {
   // 1-ask name
-  char *out=getProfileName();
+  char *out=getProfileName(this);
   if(!out) return;
   ADM_info("Using %s\n",out);
   download();
   std::string rootPath;
   ADM_pluginGetPath("x264",pluginVersion,rootPath);
-  std::string fullpath=rootPath+std::string(ADM_SEPARATOR)+out+std::string(".json");
+  std::string name=std::string(out);
+  std::string fullpath=rootPath+std::string(ADM_SEPARATOR)+name+std::string(".json");
 
   if(ADM_fileExist(fullpath.c_str()))
   {
@@ -829,7 +864,7 @@ void x264Dialog::saveAsButton_pressed(void)
         GUI_Error_HIG(QT_TRANSLATE_NOOP("x264","Error"),QT_TRANSLATE_NOOP("x264","Cannot save preset"));
         ADM_error("Cannot write to %s\n",out);
   }
-  updatePresetList();
+  updatePresetList(name.c_str());
 }
 /**
 
@@ -846,15 +881,15 @@ void x264Dialog::deleteButton_pressed(void)
   QString preset=ui.configurationComboBox->itemText(n);
   QString msg=QString(QT_TRANSLATE_NOOP("x264","Do you really want to delete the "))+preset+
             QString(QT_TRANSLATE_NOOP("x264"," profile ?.\nIf it is a system profile it will be recreated next time."));
-  if(true==GUI_Confirmation_HIG(QT_TRANSLATE_NOOP("x264","Delete preset"),QT_TRANSLATE_NOOP("x264","Delete"),msg.toUtf8().constData()))
+  if(true==GUI_Confirmation_HIG(QT_TRANSLATE_NOOP("x264","Delete"),QT_TRANSLATE_NOOP("x264","Delete preset"),msg.toUtf8().constData()))
   {
     std::string rootPath;
     ADM_pluginGetPath("x264",pluginVersion,rootPath);
     QString text=QString("/")+ui.configurationComboBox->itemText(n);
     text=QString(rootPath.c_str())+text+QString(".json");
-    unlink(text.toUtf8().constData());
+    if(!ADM_eraseFile(text.toUtf8().constData()))
+        ADM_warning("Could not delete %s\n",text.toUtf8().constData());
   }
   updatePresetList();
 }
 
-
