@@ -193,28 +193,34 @@ x265Dialog::x265Dialog(QWidget *parent, void *param) : QDialog(parent)
         }
 
         upload();
-        ADM_pluginInstallSystem( std::string("x265"),std::string(".json"),pluginVersion);
+        ADM_pluginInstallSystem( std::string("x265"),std::string("json"),pluginVersion);
         updatePresetList();
-        int n=ui.configurationComboBox->count();
-        ui.configurationComboBox->setCurrentIndex(n-1);
+        adjustSize();
 }
 /**
     \fn updatePresetList
 */
-bool x265Dialog::updatePresetList(void)
+bool x265Dialog::updatePresetList(const char *match)
 {
     QComboBox *combo=ui.configurationComboBox;
     std::string rootPath;
     vector <std::string >  list;
     ADM_pluginGetPath("x265",pluginVersion,rootPath);
-    ADM_listFile(rootPath,".json",list);
+    ADM_listFile(rootPath,"json",list);
     int l=list.size();
+    int idx=l;
+    std::string current;
+    if(match)
+        current=std::string(match);
     combo->clear();
     for( int i=0;i<l;i++)
     {
+        if(match && list[i]==current)
+            idx=i;
         combo->addItem(list[i].c_str());
     }
     combo->addItem(QString(QT_TRANSLATE_NOOP("x265","Custom")));
+    combo->setCurrentIndex(idx);
     return true;
 }
 
@@ -409,7 +415,9 @@ bool x265Dialog::upload(void)
     DISABLE(noiseReductionInterSpinBox);
     MK_UINT(noiseReductionSpinBox,noise_reduction);
 #endif
-    
+
+    MK_CHECKBOX(strongIntraSmoothingCheckBox,strong_intra_smoothing);
+
 #if X265_BUILD >= 41
     DISABLE(avgBitrateToleranceSpinBox);
     MK_CHECKBOX(strictCbrCheckBox,ratecontrol.strict_cbr);
@@ -512,7 +520,9 @@ bool x265Dialog::download(void)
 #else
           MK_UINT(noiseReductionSpinBox,noise_reduction);
 #endif
-          
+
+          MK_CHECKBOX(strongIntraSmoothingCheckBox,strong_intra_smoothing);
+
           MK_COMBOBOX_STR(presetComboBox, general.preset, listOfPresets, NB_PRESET);
           MK_COMBOBOX_STR(profileComboBox, general.profile, listOfProfiles, NB_PROFILE);
           MK_COMBOBOX_STR(tuningComboBox, general.tuning, listOfTunings, NB_TUNE);
@@ -757,9 +767,9 @@ void x265Dialog::configurationComboBox_currentIndexChanged(int index)
     \fn getProfileName  
     \brief Popup a dialog that asks the user the preset name
 */
-static char *getProfileName(void)
+static char *getProfileName(QDialog *parent)
 {
-  QDialog dialog;
+  QDialog dialog(parent);
   dialog.setWindowTitle(QString::fromUtf8(QT_TRANSLATE_NOOP("x265","Save Profile")));
   QDialogButtonBox *buttonBox = new QDialogButtonBox();  
   QVBoxLayout *vboxLayout = new QVBoxLayout();
@@ -776,17 +786,17 @@ static char *getProfileName(void)
 
   vboxLayout->addWidget(text);
   vboxLayout->addWidget(buttonBox);
-
   dialog.setLayout(vboxLayout);
+//  dialog.setModal(true);
+//  dialog.setWindowModality(Qt::ApplicationModal);
 
   if(dialog.exec()!=QDialog::Accepted)
   {
-        ADM_info("Canceled");
+        ADM_info("Cancelled");
         return NULL;
   }
-  QString fileName=text->text();
-  const char *out=fileName.toUtf8().constData();
-  return ADM_strdup(out);
+  std::string st = std::string( text->text().toUtf8().constData());
+  return ADM_strdup(st.c_str());
 }
 /**
         \fn saveAsButton_pressed
@@ -795,13 +805,14 @@ static char *getProfileName(void)
 void x265Dialog::saveAsButton_pressed(void)
 {
   // 1-ask name
-  char *out=getProfileName();
+  char *out=getProfileName(this);
   if(!out) return;
   ADM_info("Using %s\n",out);
   download();
   std::string rootPath;
   ADM_pluginGetPath("x265",pluginVersion,rootPath);
-  std::string fullpath=rootPath+std::string(ADM_SEPARATOR)+out+std::string(".json");
+  std::string name=std::string(out);
+  std::string fullpath=rootPath+std::string(ADM_SEPARATOR)+name+std::string(".json");
 
   if(ADM_fileExist(fullpath.c_str()))
   {
@@ -817,7 +828,7 @@ void x265Dialog::saveAsButton_pressed(void)
         GUI_Error_HIG(QT_TRANSLATE_NOOP("x265","Error"),QT_TRANSLATE_NOOP("x265","Cannot save preset"));
         ADM_error("Cannot write to %s\n",out);
   }
-  updatePresetList();
+  updatePresetList(name.c_str());
 }
 /**
 
@@ -834,15 +845,15 @@ void x265Dialog::deleteButton_pressed(void)
   QString preset=ui.configurationComboBox->itemText(n);
   QString msg=QString(QT_TRANSLATE_NOOP("x265","Do you really want to delete the "))+preset+
             QString(QT_TRANSLATE_NOOP("x265"," profile ?.\nIf it is a system profile it will be recreated next time."));
-  if(true==GUI_Confirmation_HIG(QT_TRANSLATE_NOOP("x265","Delete preset"),QT_TRANSLATE_NOOP("x265","Delete"),msg.toUtf8().constData()))
+  if(true==GUI_Confirmation_HIG(QT_TRANSLATE_NOOP("x265","Delete"),QT_TRANSLATE_NOOP("x265","Delete preset"),msg.toUtf8().constData()))
   {
     std::string rootPath;
     ADM_pluginGetPath("x265",pluginVersion,rootPath);
     QString text=QString("/")+ui.configurationComboBox->itemText(n);
     text=QString(rootPath.c_str())+text+QString(".json");
-    unlink(text.toUtf8().constData());
+    if(!ADM_eraseFile(text.toUtf8().constData()))
+        ADM_warning("Could not delete %s\n",text.toUtf8().constData());
   }
   updatePresetList();
 }
 
-
