@@ -60,6 +60,7 @@ public:
                             changeFps(ADM_coreVideoFilter *previous,CONFcouple *conf);
                             ~changeFps();
         bool                goToTime(uint64_t usSeek);
+        virtual bool        getTimeRange(uint64_t *start, uint64_t *end);
         virtual const char   *getConfiguration(void);                   /// Return  current configuration as a human readable string
         virtual bool         getNextFrame(uint32_t *fn,ADMImage *image);    /// Return the next image
         virtual bool         getCoupledConf(CONFcouple **couples) ;   /// Return the current filter configuration
@@ -83,7 +84,7 @@ DECLARE_VIDEO_FILTER(   changeFps,   // Class
 const char *changeFps::getConfiguration( void )
 {
 static char buf[100];
- snprintf(buf,99," Resample from %2.2f to %2.2f fps",
+ snprintf(buf,99," Change FPS from %2.2f to %2.2f fps",
         (double)configuration.oldFpsNum/configuration.oldFpsDen,
         (double)configuration.newFpsNum/configuration.newFpsDen);
  return buf;  
@@ -128,6 +129,17 @@ bool         changeFps::goToTime(uint64_t usSeek)
     return  previousFilter->goToTime((uint64_t)timing);
 }
 
+/**
+    \fn getTimeRange
+    \brief Report updated time range to the next filter
+*/
+bool changeFps::getTimeRange(uint64_t *start, uint64_t *end)
+{
+    *start=0;
+    *end=info.totalDuration;
+    return true;
+}
+
 /**
     \fn getCoupledConf
 */ 
