@@ -7,6 +7,7 @@ class flyCrop : public ADM_flyDialogRgb
   friend class Ui_cropWindow;
   public:
    uint32_t left,right,top,bottom;
+   bool rubber_is_hidden;
   public:
    uint8_t    processRgb(uint8_t *imageIn, uint8_t *imageOut);
    uint8_t    download(void);
@@ -21,8 +22,9 @@ protected:
     ADM_rubberControl *rubber;
     bool        blockChanges(bool block);
     int         autoRun(uint8_t *in,int w,int h, int increment);
-    int         autoRunV(uint8_t *in,int w,int h, int increment);
+    int         autoRunV(uint8_t *in, int stride, int w, int increment);
 private:
     int _ox,_oy,_ow,_oh;
+    void dimensions();
 };
 
