@@ -37,9 +37,11 @@ private slots:
 	void valueChanged(int foo);
 	void autoCrop(bool f);
 	void reset(bool f);
+	void toggleRubber(int checkState);
 
 private:
         void resizeEvent(QResizeEvent *event);
+        void showEvent(QShowEvent *event);
 };
 
 #endif	// Q_crop_h
